module Data.Poker.Brute
     ( numericalToConsecutive
     , handValueToConsecutive
     , consecutiveToHandValue
     ) where

import Data.Poker.Deck          ( HandValue(..), Rank(..), NumericalHandValue, ConsecutiveHandValue(..) )
import Data.Poker.Interface     ( numericalToHandValue )

import qualified Data.Vector as V

-- It's quite ugly to have these instance here. Please fix it.
instance Show NumericalHandValue where
     show = show . numericalToHandValue

instance Show ConsecutiveHandValue where
     show = show . fromConsecutiveId

instance Enum HandValue where
     toEnum n
          | n < 0 || n > 7472 = error "Data.Poker.Brute.toEnum: bad argument"
          | otherwise         = fromConsecutiveId . ConsecutiveHandValue . fromIntegral $ n
     fromEnum = fromIntegral . unConsecutiveHandValue . handValueToConsecutive
     enumFrom x = enumFromTo x maxBound
     enumFromThen x step
          | x > step  = enumFromThenTo x step minBound
          | otherwise = enumFromThenTo x step maxBound

numericalToConsecutive :: NumericalHandValue -> ConsecutiveHandValue
numericalToConsecutive = toConsecutiveIdInline . numericalToHandValue

{-# NOINLINE handValueToConsecutive #-}
handValueToConsecutive :: HandValue -> ConsecutiveHandValue
handValueToConsecutive = toConsecutiveIdInline

consecutiveToHandValue :: ConsecutiveHandValue -> HandValue
consecutiveToHandValue = fromConsecutiveId


{-# INLINE toConsecutiveIdInline #-}
toConsecutiveIdInline :: HandValue -> ConsecutiveHandValue
toConsecutiveIdInline hand = ConsecutiveHandValue $
     case hand of
          NoPair Seven Five Four Three Two   -> 0
          NoPair Seven Six Four Three Two    -> 1
          NoPair Seven Six Five Three Two    -> 2
          NoPair Seven Six Five Four Two     -> 3
          NoPair Eight Five Four Three Two   -> 4
          NoPair Eight Six Four Three Two    -> 5
          NoPair Eight Six Five Three Two    -> 6
          NoPair Eight Six Five Four Two     -> 7
          NoPair Eight Six Five Four Three   -> 8
          NoPair Eight Seven Four Three Two  -> 9
          NoPair Eight Seven Five Three Two  -> 10
          NoPair Eight Seven Five Four Two   -> 11
          NoPair Eight Seven Five Four Three -> 12
          NoPair Eight Seven Six Three Two   -> 13
          NoPair Eight Seven Six Four Two    -> 14
          NoPair Eight Seven Six Four Three  -> 15
          NoPair Eight Seven Six Five Two    -> 16
          NoPair Eight Seven Six Five Three  -> 17
          NoPair Nine Five Four Three Two    -> 18
          NoPair Nine Six Four Three Two     -> 19
          NoPair Nine Six Five Three Two     -> 20
          NoPair Nine Six Five Four Two      -> 21
          NoPair Nine Six Five Four Three    -> 22
          NoPair Nine Seven Four Three Two   -> 23
          NoPair Nine Seven Five Three Two   -> 24
          NoPair Nine Seven Five Four Two    -> 25
          NoPair Nine Seven Five Four Three  -> 26
          NoPair Nine Seven Six Three Two    -> 27
          NoPair Nine Seven Six Four Two     -> 28
          NoPair Nine Seven Six Four Three   -> 29
          NoPair Nine Seven Six Five Two     -> 30
          NoPair Nine Seven Six Five Three   -> 31
          NoPair Nine Seven Six Five Four    -> 32
          NoPair Nine Eight Four Three Two   -> 33
          NoPair Nine Eight Five Three Two   -> 34
          NoPair Nine Eight Five Four Two    -> 35
          NoPair Nine Eight Five Four Three  -> 36
          NoPair Nine Eight Six Three Two    -> 37
          NoPair Nine Eight Six Four Two     -> 38
          NoPair Nine Eight Six Four Three   -> 39
          NoPair Nine Eight Six Five Two     -> 40
          NoPair Nine Eight Six Five Three   -> 41
          NoPair Nine Eight Six Five Four    -> 42
          NoPair Nine Eight Seven Three Two  -> 43
          NoPair Nine Eight Seven Four Two   -> 44
          NoPair Nine Eight Seven Four Three -> 45
          NoPair Nine Eight Seven Five Two   -> 46
          NoPair Nine Eight Seven Five Three -> 47
          NoPair Nine Eight Seven Five Four  -> 48
          NoPair Nine Eight Seven Six Two    -> 49
          NoPair Nine Eight Seven Six Three  -> 50
          NoPair Nine Eight Seven Six Four   -> 51
          NoPair Ten Five Four Three Two     -> 52
          NoPair Ten Six Four Three Two      -> 53
          NoPair Ten Six Five Three Two      -> 54
          NoPair Ten Six Five Four Two       -> 55
          NoPair Ten Six Five Four Three     -> 56
          NoPair Ten Seven Four Three Two    -> 57
          NoPair Ten Seven Five Three Two    -> 58
          NoPair Ten Seven Five Four Two     -> 59
          NoPair Ten Seven Five Four Three   -> 60
          NoPair Ten Seven Six Three Two     -> 61
          NoPair Ten Seven Six Four Two      -> 62
          NoPair Ten Seven Six Four Three    -> 63
          NoPair Ten Seven Six Five Two      -> 64
          NoPair Ten Seven Six Five Three    -> 65
          NoPair Ten Seven Six Five Four     -> 66
          NoPair Ten Eight Four Three Two    -> 67
          NoPair Ten Eight Five Three Two    -> 68
          NoPair Ten Eight Five Four Two     -> 69
          NoPair Ten Eight Five Four Three   -> 70
          NoPair Ten Eight Six Three Two     -> 71
          NoPair Ten Eight Six Four Two      -> 72
          NoPair Ten Eight Six Four Three    -> 73
          NoPair Ten Eight Six Five Two      -> 74
          NoPair Ten Eight Six Five Three    -> 75
          NoPair Ten Eight Six Five Four     -> 76
          NoPair Ten Eight Seven Three Two   -> 77
          NoPair Ten Eight Seven Four Two    -> 78
          NoPair Ten Eight Seven Four Three  -> 79
          NoPair Ten Eight Seven Five Two    -> 80
          NoPair Ten Eight Seven Five Three  -> 81
          NoPair Ten Eight Seven Five Four   -> 82
          NoPair Ten Eight Seven Six Two     -> 83
          NoPair Ten Eight Seven Six Three   -> 84
          NoPair Ten Eight Seven Six Four    -> 85
          NoPair Ten Eight Seven Six Five    -> 86
          NoPair Ten Nine Four Three Two     -> 87
          NoPair Ten Nine Five Three Two     -> 88
          NoPair Ten Nine Five Four Two      -> 89
          NoPair Ten Nine Five Four Three    -> 90
          NoPair Ten Nine Six Three Two      -> 91
          NoPair Ten Nine Six Four Two       -> 92
          NoPair Ten Nine Six Four Three     -> 93
          NoPair Ten Nine Six Five Two       -> 94
          NoPair Ten Nine Six Five Three     -> 95
          NoPair Ten Nine Six Five Four      -> 96
          NoPair Ten Nine Seven Three Two    -> 97
          NoPair Ten Nine Seven Four Two     -> 98
          NoPair Ten Nine Seven Four Three   -> 99
          NoPair Ten Nine Seven Five Two     -> 100
          NoPair Ten Nine Seven Five Three   -> 101
          NoPair Ten Nine Seven Five Four    -> 102
          NoPair Ten Nine Seven Six Two      -> 103
          NoPair Ten Nine Seven Six Three    -> 104
          NoPair Ten Nine Seven Six Four     -> 105
          NoPair Ten Nine Seven Six Five     -> 106
          NoPair Ten Nine Eight Three Two    -> 107
          NoPair Ten Nine Eight Four Two     -> 108
          NoPair Ten Nine Eight Four Three   -> 109
          NoPair Ten Nine Eight Five Two     -> 110
          NoPair Ten Nine Eight Five Three   -> 111
          NoPair Ten Nine Eight Five Four    -> 112
          NoPair Ten Nine Eight Six Two      -> 113
          NoPair Ten Nine Eight Six Three    -> 114
          NoPair Ten Nine Eight Six Four     -> 115
          NoPair Ten Nine Eight Six Five     -> 116
          NoPair Ten Nine Eight Seven Two    -> 117
          NoPair Ten Nine Eight Seven Three  -> 118
          NoPair Ten Nine Eight Seven Four   -> 119
          NoPair Ten Nine Eight Seven Five   -> 120
          NoPair Jack Five Four Three Two    -> 121
          NoPair Jack Six Four Three Two     -> 122
          NoPair Jack Six Five Three Two     -> 123
          NoPair Jack Six Five Four Two      -> 124
          NoPair Jack Six Five Four Three    -> 125
          NoPair Jack Seven Four Three Two   -> 126
          NoPair Jack Seven Five Three Two   -> 127
          NoPair Jack Seven Five Four Two    -> 128
          NoPair Jack Seven Five Four Three  -> 129
          NoPair Jack Seven Six Three Two    -> 130
          NoPair Jack Seven Six Four Two     -> 131
          NoPair Jack Seven Six Four Three   -> 132
          NoPair Jack Seven Six Five Two     -> 133
          NoPair Jack Seven Six Five Three   -> 134
          NoPair Jack Seven Six Five Four    -> 135
          NoPair Jack Eight Four Three Two   -> 136
          NoPair Jack Eight Five Three Two   -> 137
          NoPair Jack Eight Five Four Two    -> 138
          NoPair Jack Eight Five Four Three  -> 139
          NoPair Jack Eight Six Three Two    -> 140
          NoPair Jack Eight Six Four Two     -> 141
          NoPair Jack Eight Six Four Three   -> 142
          NoPair Jack Eight Six Five Two     -> 143
          NoPair Jack Eight Six Five Three   -> 144
          NoPair Jack Eight Six Five Four    -> 145
          NoPair Jack Eight Seven Three Two  -> 146
          NoPair Jack Eight Seven Four Two   -> 147
          NoPair Jack Eight Seven Four Three -> 148
          NoPair Jack Eight Seven Five Two   -> 149
          NoPair Jack Eight Seven Five Three -> 150
          NoPair Jack Eight Seven Five Four  -> 151
          NoPair Jack Eight Seven Six Two    -> 152
          NoPair Jack Eight Seven Six Three  -> 153
          NoPair Jack Eight Seven Six Four   -> 154
          NoPair Jack Eight Seven Six Five   -> 155
          NoPair Jack Nine Four Three Two    -> 156
          NoPair Jack Nine Five Three Two    -> 157
          NoPair Jack Nine Five Four Two     -> 158
          NoPair Jack Nine Five Four Three   -> 159
          NoPair Jack Nine Six Three Two     -> 160
          NoPair Jack Nine Six Four Two      -> 161
          NoPair Jack Nine Six Four Three    -> 162
          NoPair Jack Nine Six Five Two      -> 163
          NoPair Jack Nine Six Five Three    -> 164
          NoPair Jack Nine Six Five Four     -> 165
          NoPair Jack Nine Seven Three Two   -> 166
          NoPair Jack Nine Seven Four Two    -> 167
          NoPair Jack Nine Seven Four Three  -> 168
          NoPair Jack Nine Seven Five Two    -> 169
          NoPair Jack Nine Seven Five Three  -> 170
          NoPair Jack Nine Seven Five Four   -> 171
          NoPair Jack Nine Seven Six Two     -> 172
          NoPair Jack Nine Seven Six Three   -> 173
          NoPair Jack Nine Seven Six Four    -> 174
          NoPair Jack Nine Seven Six Five    -> 175
          NoPair Jack Nine Eight Three Two   -> 176
          NoPair Jack Nine Eight Four Two    -> 177
          NoPair Jack Nine Eight Four Three  -> 178
          NoPair Jack Nine Eight Five Two    -> 179
          NoPair Jack Nine Eight Five Three  -> 180
          NoPair Jack Nine Eight Five Four   -> 181
          NoPair Jack Nine Eight Six Two     -> 182
          NoPair Jack Nine Eight Six Three   -> 183
          NoPair Jack Nine Eight Six Four    -> 184
          NoPair Jack Nine Eight Six Five    -> 185
          NoPair Jack Nine Eight Seven Two   -> 186
          NoPair Jack Nine Eight Seven Three -> 187
          NoPair Jack Nine Eight Seven Four  -> 188
          NoPair Jack Nine Eight Seven Five  -> 189
          NoPair Jack Nine Eight Seven Six   -> 190
          NoPair Jack Ten Four Three Two     -> 191
          NoPair Jack Ten Five Three Two     -> 192
          NoPair Jack Ten Five Four Two      -> 193
          NoPair Jack Ten Five Four Three    -> 194
          NoPair Jack Ten Six Three Two      -> 195
          NoPair Jack Ten Six Four Two       -> 196
          NoPair Jack Ten Six Four Three     -> 197
          NoPair Jack Ten Six Five Two       -> 198
          NoPair Jack Ten Six Five Three     -> 199
          NoPair Jack Ten Six Five Four      -> 200
          NoPair Jack Ten Seven Three Two    -> 201
          NoPair Jack Ten Seven Four Two     -> 202
          NoPair Jack Ten Seven Four Three   -> 203
          NoPair Jack Ten Seven Five Two     -> 204
          NoPair Jack Ten Seven Five Three   -> 205
          NoPair Jack Ten Seven Five Four    -> 206
          NoPair Jack Ten Seven Six Two      -> 207
          NoPair Jack Ten Seven Six Three    -> 208
          NoPair Jack Ten Seven Six Four     -> 209
          NoPair Jack Ten Seven Six Five     -> 210
          NoPair Jack Ten Eight Three Two    -> 211
          NoPair Jack Ten Eight Four Two     -> 212
          NoPair Jack Ten Eight Four Three   -> 213
          NoPair Jack Ten Eight Five Two     -> 214
          NoPair Jack Ten Eight Five Three   -> 215
          NoPair Jack Ten Eight Five Four    -> 216
          NoPair Jack Ten Eight Six Two      -> 217
          NoPair Jack Ten Eight Six Three    -> 218
          NoPair Jack Ten Eight Six Four     -> 219
          NoPair Jack Ten Eight Six Five     -> 220
          NoPair Jack Ten Eight Seven Two    -> 221
          NoPair Jack Ten Eight Seven Three  -> 222
          NoPair Jack Ten Eight Seven Four   -> 223
          NoPair Jack Ten Eight Seven Five   -> 224
          NoPair Jack Ten Eight Seven Six    -> 225
          NoPair Jack Ten Nine Three Two     -> 226
          NoPair Jack Ten Nine Four Two      -> 227
          NoPair Jack Ten Nine Four Three    -> 228
          NoPair Jack Ten Nine Five Two      -> 229
          NoPair Jack Ten Nine Five Three    -> 230
          NoPair Jack Ten Nine Five Four     -> 231
          NoPair Jack Ten Nine Six Two       -> 232
          NoPair Jack Ten Nine Six Three     -> 233
          NoPair Jack Ten Nine Six Four      -> 234
          NoPair Jack Ten Nine Six Five      -> 235
          NoPair Jack Ten Nine Seven Two     -> 236
          NoPair Jack Ten Nine Seven Three   -> 237
          NoPair Jack Ten Nine Seven Four    -> 238
          NoPair Jack Ten Nine Seven Five    -> 239
          NoPair Jack Ten Nine Seven Six     -> 240
          NoPair Jack Ten Nine Eight Two     -> 241
          NoPair Jack Ten Nine Eight Three   -> 242
          NoPair Jack Ten Nine Eight Four    -> 243
          NoPair Jack Ten Nine Eight Five    -> 244
          NoPair Jack Ten Nine Eight Six     -> 245
          NoPair Queen Five Four Three Two   -> 246
          NoPair Queen Six Four Three Two    -> 247
          NoPair Queen Six Five Three Two    -> 248
          NoPair Queen Six Five Four Two     -> 249
          NoPair Queen Six Five Four Three   -> 250
          NoPair Queen Seven Four Three Two  -> 251
          NoPair Queen Seven Five Three Two  -> 252
          NoPair Queen Seven Five Four Two   -> 253
          NoPair Queen Seven Five Four Three -> 254
          NoPair Queen Seven Six Three Two   -> 255
          NoPair Queen Seven Six Four Two    -> 256
          NoPair Queen Seven Six Four Three  -> 257
          NoPair Queen Seven Six Five Two    -> 258
          NoPair Queen Seven Six Five Three  -> 259
          NoPair Queen Seven Six Five Four   -> 260
          NoPair Queen Eight Four Three Two  -> 261
          NoPair Queen Eight Five Three Two  -> 262
          NoPair Queen Eight Five Four Two   -> 263
          NoPair Queen Eight Five Four Three -> 264
          NoPair Queen Eight Six Three Two   -> 265
          NoPair Queen Eight Six Four Two    -> 266
          NoPair Queen Eight Six Four Three  -> 267
          NoPair Queen Eight Six Five Two    -> 268
          NoPair Queen Eight Six Five Three  -> 269
          NoPair Queen Eight Six Five Four   -> 270
          NoPair Queen Eight Seven Three Two -> 271
          NoPair Queen Eight Seven Four Two  -> 272
          NoPair Queen Eight Seven Four Three-> 273
          NoPair Queen Eight Seven Five Two  -> 274
          NoPair Queen Eight Seven Five Three-> 275
          NoPair Queen Eight Seven Five Four -> 276
          NoPair Queen Eight Seven Six Two   -> 277
          NoPair Queen Eight Seven Six Three -> 278
          NoPair Queen Eight Seven Six Four  -> 279
          NoPair Queen Eight Seven Six Five  -> 280
          NoPair Queen Nine Four Three Two   -> 281
          NoPair Queen Nine Five Three Two   -> 282
          NoPair Queen Nine Five Four Two    -> 283
          NoPair Queen Nine Five Four Three  -> 284
          NoPair Queen Nine Six Three Two    -> 285
          NoPair Queen Nine Six Four Two     -> 286
          NoPair Queen Nine Six Four Three   -> 287
          NoPair Queen Nine Six Five Two     -> 288
          NoPair Queen Nine Six Five Three   -> 289
          NoPair Queen Nine Six Five Four    -> 290
          NoPair Queen Nine Seven Three Two  -> 291
          NoPair Queen Nine Seven Four Two   -> 292
          NoPair Queen Nine Seven Four Three -> 293
          NoPair Queen Nine Seven Five Two   -> 294
          NoPair Queen Nine Seven Five Three -> 295
          NoPair Queen Nine Seven Five Four  -> 296
          NoPair Queen Nine Seven Six Two    -> 297
          NoPair Queen Nine Seven Six Three  -> 298
          NoPair Queen Nine Seven Six Four   -> 299
          NoPair Queen Nine Seven Six Five   -> 300
          NoPair Queen Nine Eight Three Two  -> 301
          NoPair Queen Nine Eight Four Two   -> 302
          NoPair Queen Nine Eight Four Three -> 303
          NoPair Queen Nine Eight Five Two   -> 304
          NoPair Queen Nine Eight Five Three -> 305
          NoPair Queen Nine Eight Five Four  -> 306
          NoPair Queen Nine Eight Six Two    -> 307
          NoPair Queen Nine Eight Six Three  -> 308
          NoPair Queen Nine Eight Six Four   -> 309
          NoPair Queen Nine Eight Six Five   -> 310
          NoPair Queen Nine Eight Seven Two  -> 311
          NoPair Queen Nine Eight Seven Three-> 312
          NoPair Queen Nine Eight Seven Four -> 313
          NoPair Queen Nine Eight Seven Five -> 314
          NoPair Queen Nine Eight Seven Six  -> 315
          NoPair Queen Ten Four Three Two    -> 316
          NoPair Queen Ten Five Three Two    -> 317
          NoPair Queen Ten Five Four Two     -> 318
          NoPair Queen Ten Five Four Three   -> 319
          NoPair Queen Ten Six Three Two     -> 320
          NoPair Queen Ten Six Four Two      -> 321
          NoPair Queen Ten Six Four Three    -> 322
          NoPair Queen Ten Six Five Two      -> 323
          NoPair Queen Ten Six Five Three    -> 324
          NoPair Queen Ten Six Five Four     -> 325
          NoPair Queen Ten Seven Three Two   -> 326
          NoPair Queen Ten Seven Four Two    -> 327
          NoPair Queen Ten Seven Four Three  -> 328
          NoPair Queen Ten Seven Five Two    -> 329
          NoPair Queen Ten Seven Five Three  -> 330
          NoPair Queen Ten Seven Five Four   -> 331
          NoPair Queen Ten Seven Six Two     -> 332
          NoPair Queen Ten Seven Six Three   -> 333
          NoPair Queen Ten Seven Six Four    -> 334
          NoPair Queen Ten Seven Six Five    -> 335
          NoPair Queen Ten Eight Three Two   -> 336
          NoPair Queen Ten Eight Four Two    -> 337
          NoPair Queen Ten Eight Four Three  -> 338
          NoPair Queen Ten Eight Five Two    -> 339
          NoPair Queen Ten Eight Five Three  -> 340
          NoPair Queen Ten Eight Five Four   -> 341
          NoPair Queen Ten Eight Six Two     -> 342
          NoPair Queen Ten Eight Six Three   -> 343
          NoPair Queen Ten Eight Six Four    -> 344
          NoPair Queen Ten Eight Six Five    -> 345
          NoPair Queen Ten Eight Seven Two   -> 346
          NoPair Queen Ten Eight Seven Three -> 347
          NoPair Queen Ten Eight Seven Four  -> 348
          NoPair Queen Ten Eight Seven Five  -> 349
          NoPair Queen Ten Eight Seven Six   -> 350
          NoPair Queen Ten Nine Three Two    -> 351
          NoPair Queen Ten Nine Four Two     -> 352
          NoPair Queen Ten Nine Four Three   -> 353
          NoPair Queen Ten Nine Five Two     -> 354
          NoPair Queen Ten Nine Five Three   -> 355
          NoPair Queen Ten Nine Five Four    -> 356
          NoPair Queen Ten Nine Six Two      -> 357
          NoPair Queen Ten Nine Six Three    -> 358
          NoPair Queen Ten Nine Six Four     -> 359
          NoPair Queen Ten Nine Six Five     -> 360
          NoPair Queen Ten Nine Seven Two    -> 361
          NoPair Queen Ten Nine Seven Three  -> 362
          NoPair Queen Ten Nine Seven Four   -> 363
          NoPair Queen Ten Nine Seven Five   -> 364
          NoPair Queen Ten Nine Seven Six    -> 365
          NoPair Queen Ten Nine Eight Two    -> 366
          NoPair Queen Ten Nine Eight Three  -> 367
          NoPair Queen Ten Nine Eight Four   -> 368
          NoPair Queen Ten Nine Eight Five   -> 369
          NoPair Queen Ten Nine Eight Six    -> 370
          NoPair Queen Ten Nine Eight Seven  -> 371
          NoPair Queen Jack Four Three Two   -> 372
          NoPair Queen Jack Five Three Two   -> 373
          NoPair Queen Jack Five Four Two    -> 374
          NoPair Queen Jack Five Four Three  -> 375
          NoPair Queen Jack Six Three Two    -> 376
          NoPair Queen Jack Six Four Two     -> 377
          NoPair Queen Jack Six Four Three   -> 378
          NoPair Queen Jack Six Five Two     -> 379
          NoPair Queen Jack Six Five Three   -> 380
          NoPair Queen Jack Six Five Four    -> 381
          NoPair Queen Jack Seven Three Two  -> 382
          NoPair Queen Jack Seven Four Two   -> 383
          NoPair Queen Jack Seven Four Three -> 384
          NoPair Queen Jack Seven Five Two   -> 385
          NoPair Queen Jack Seven Five Three -> 386
          NoPair Queen Jack Seven Five Four  -> 387
          NoPair Queen Jack Seven Six Two    -> 388
          NoPair Queen Jack Seven Six Three  -> 389
          NoPair Queen Jack Seven Six Four   -> 390
          NoPair Queen Jack Seven Six Five   -> 391
          NoPair Queen Jack Eight Three Two  -> 392
          NoPair Queen Jack Eight Four Two   -> 393
          NoPair Queen Jack Eight Four Three -> 394
          NoPair Queen Jack Eight Five Two   -> 395
          NoPair Queen Jack Eight Five Three -> 396
          NoPair Queen Jack Eight Five Four  -> 397
          NoPair Queen Jack Eight Six Two    -> 398
          NoPair Queen Jack Eight Six Three  -> 399
          NoPair Queen Jack Eight Six Four   -> 400
          NoPair Queen Jack Eight Six Five   -> 401
          NoPair Queen Jack Eight Seven Two  -> 402
          NoPair Queen Jack Eight Seven Three-> 403
          NoPair Queen Jack Eight Seven Four -> 404
          NoPair Queen Jack Eight Seven Five -> 405
          NoPair Queen Jack Eight Seven Six  -> 406
          NoPair Queen Jack Nine Three Two   -> 407
          NoPair Queen Jack Nine Four Two    -> 408
          NoPair Queen Jack Nine Four Three  -> 409
          NoPair Queen Jack Nine Five Two    -> 410
          NoPair Queen Jack Nine Five Three  -> 411
          NoPair Queen Jack Nine Five Four   -> 412
          NoPair Queen Jack Nine Six Two     -> 413
          NoPair Queen Jack Nine Six Three   -> 414
          NoPair Queen Jack Nine Six Four    -> 415
          NoPair Queen Jack Nine Six Five    -> 416
          NoPair Queen Jack Nine Seven Two   -> 417
          NoPair Queen Jack Nine Seven Three -> 418
          NoPair Queen Jack Nine Seven Four  -> 419
          NoPair Queen Jack Nine Seven Five  -> 420
          NoPair Queen Jack Nine Seven Six   -> 421
          NoPair Queen Jack Nine Eight Two   -> 422
          NoPair Queen Jack Nine Eight Three -> 423
          NoPair Queen Jack Nine Eight Four  -> 424
          NoPair Queen Jack Nine Eight Five  -> 425
          NoPair Queen Jack Nine Eight Six   -> 426
          NoPair Queen Jack Nine Eight Seven -> 427
          NoPair Queen Jack Ten Three Two    -> 428
          NoPair Queen Jack Ten Four Two     -> 429
          NoPair Queen Jack Ten Four Three   -> 430
          NoPair Queen Jack Ten Five Two     -> 431
          NoPair Queen Jack Ten Five Three   -> 432
          NoPair Queen Jack Ten Five Four    -> 433
          NoPair Queen Jack Ten Six Two      -> 434
          NoPair Queen Jack Ten Six Three    -> 435
          NoPair Queen Jack Ten Six Four     -> 436
          NoPair Queen Jack Ten Six Five     -> 437
          NoPair Queen Jack Ten Seven Two    -> 438
          NoPair Queen Jack Ten Seven Three  -> 439
          NoPair Queen Jack Ten Seven Four   -> 440
          NoPair Queen Jack Ten Seven Five   -> 441
          NoPair Queen Jack Ten Seven Six    -> 442
          NoPair Queen Jack Ten Eight Two    -> 443
          NoPair Queen Jack Ten Eight Three  -> 444
          NoPair Queen Jack Ten Eight Four   -> 445
          NoPair Queen Jack Ten Eight Five   -> 446
          NoPair Queen Jack Ten Eight Six    -> 447
          NoPair Queen Jack Ten Eight Seven  -> 448
          NoPair Queen Jack Ten Nine Two     -> 449
          NoPair Queen Jack Ten Nine Three   -> 450
          NoPair Queen Jack Ten Nine Four    -> 451
          NoPair Queen Jack Ten Nine Five    -> 452
          NoPair Queen Jack Ten Nine Six     -> 453
          NoPair Queen Jack Ten Nine Seven   -> 454
          NoPair King Five Four Three Two    -> 455
          NoPair King Six Four Three Two     -> 456
          NoPair King Six Five Three Two     -> 457
          NoPair King Six Five Four Two      -> 458
          NoPair King Six Five Four Three    -> 459
          NoPair King Seven Four Three Two   -> 460
          NoPair King Seven Five Three Two   -> 461
          NoPair King Seven Five Four Two    -> 462
          NoPair King Seven Five Four Three  -> 463
          NoPair King Seven Six Three Two    -> 464
          NoPair King Seven Six Four Two     -> 465
          NoPair King Seven Six Four Three   -> 466
          NoPair King Seven Six Five Two     -> 467
          NoPair King Seven Six Five Three   -> 468
          NoPair King Seven Six Five Four    -> 469
          NoPair King Eight Four Three Two   -> 470
          NoPair King Eight Five Three Two   -> 471
          NoPair King Eight Five Four Two    -> 472
          NoPair King Eight Five Four Three  -> 473
          NoPair King Eight Six Three Two    -> 474
          NoPair King Eight Six Four Two     -> 475
          NoPair King Eight Six Four Three   -> 476
          NoPair King Eight Six Five Two     -> 477
          NoPair King Eight Six Five Three   -> 478
          NoPair King Eight Six Five Four    -> 479
          NoPair King Eight Seven Three Two  -> 480
          NoPair King Eight Seven Four Two   -> 481
          NoPair King Eight Seven Four Three -> 482
          NoPair King Eight Seven Five Two   -> 483
          NoPair King Eight Seven Five Three -> 484
          NoPair King Eight Seven Five Four  -> 485
          NoPair King Eight Seven Six Two    -> 486
          NoPair King Eight Seven Six Three  -> 487
          NoPair King Eight Seven Six Four   -> 488
          NoPair King Eight Seven Six Five   -> 489
          NoPair King Nine Four Three Two    -> 490
          NoPair King Nine Five Three Two    -> 491
          NoPair King Nine Five Four Two     -> 492
          NoPair King Nine Five Four Three   -> 493
          NoPair King Nine Six Three Two     -> 494
          NoPair King Nine Six Four Two      -> 495
          NoPair King Nine Six Four Three    -> 496
          NoPair King Nine Six Five Two      -> 497
          NoPair King Nine Six Five Three    -> 498
          NoPair King Nine Six Five Four     -> 499
          NoPair King Nine Seven Three Two   -> 500
          NoPair King Nine Seven Four Two    -> 501
          NoPair King Nine Seven Four Three  -> 502
          NoPair King Nine Seven Five Two    -> 503
          NoPair King Nine Seven Five Three  -> 504
          NoPair King Nine Seven Five Four   -> 505
          NoPair King Nine Seven Six Two     -> 506
          NoPair King Nine Seven Six Three   -> 507
          NoPair King Nine Seven Six Four    -> 508
          NoPair King Nine Seven Six Five    -> 509
          NoPair King Nine Eight Three Two   -> 510
          NoPair King Nine Eight Four Two    -> 511
          NoPair King Nine Eight Four Three  -> 512
          NoPair King Nine Eight Five Two    -> 513
          NoPair King Nine Eight Five Three  -> 514
          NoPair King Nine Eight Five Four   -> 515
          NoPair King Nine Eight Six Two     -> 516
          NoPair King Nine Eight Six Three   -> 517
          NoPair King Nine Eight Six Four    -> 518
          NoPair King Nine Eight Six Five    -> 519
          NoPair King Nine Eight Seven Two   -> 520
          NoPair King Nine Eight Seven Three -> 521
          NoPair King Nine Eight Seven Four  -> 522
          NoPair King Nine Eight Seven Five  -> 523
          NoPair King Nine Eight Seven Six   -> 524
          NoPair King Ten Four Three Two     -> 525
          NoPair King Ten Five Three Two     -> 526
          NoPair King Ten Five Four Two      -> 527
          NoPair King Ten Five Four Three    -> 528
          NoPair King Ten Six Three Two      -> 529
          NoPair King Ten Six Four Two       -> 530
          NoPair King Ten Six Four Three     -> 531
          NoPair King Ten Six Five Two       -> 532
          NoPair King Ten Six Five Three     -> 533
          NoPair King Ten Six Five Four      -> 534
          NoPair King Ten Seven Three Two    -> 535
          NoPair King Ten Seven Four Two     -> 536
          NoPair King Ten Seven Four Three   -> 537
          NoPair King Ten Seven Five Two     -> 538
          NoPair King Ten Seven Five Three   -> 539
          NoPair King Ten Seven Five Four    -> 540
          NoPair King Ten Seven Six Two      -> 541
          NoPair King Ten Seven Six Three    -> 542
          NoPair King Ten Seven Six Four     -> 543
          NoPair King Ten Seven Six Five     -> 544
          NoPair King Ten Eight Three Two    -> 545
          NoPair King Ten Eight Four Two     -> 546
          NoPair King Ten Eight Four Three   -> 547
          NoPair King Ten Eight Five Two     -> 548
          NoPair King Ten Eight Five Three   -> 549
          NoPair King Ten Eight Five Four    -> 550
          NoPair King Ten Eight Six Two      -> 551
          NoPair King Ten Eight Six Three    -> 552
          NoPair King Ten Eight Six Four     -> 553
          NoPair King Ten Eight Six Five     -> 554
          NoPair King Ten Eight Seven Two    -> 555
          NoPair King Ten Eight Seven Three  -> 556
          NoPair King Ten Eight Seven Four   -> 557
          NoPair King Ten Eight Seven Five   -> 558
          NoPair King Ten Eight Seven Six    -> 559
          NoPair King Ten Nine Three Two     -> 560
          NoPair King Ten Nine Four Two      -> 561
          NoPair King Ten Nine Four Three    -> 562
          NoPair King Ten Nine Five Two      -> 563
          NoPair King Ten Nine Five Three    -> 564
          NoPair King Ten Nine Five Four     -> 565
          NoPair King Ten Nine Six Two       -> 566
          NoPair King Ten Nine Six Three     -> 567
          NoPair King Ten Nine Six Four      -> 568
          NoPair King Ten Nine Six Five      -> 569
          NoPair King Ten Nine Seven Two     -> 570
          NoPair King Ten Nine Seven Three   -> 571
          NoPair King Ten Nine Seven Four    -> 572
          NoPair King Ten Nine Seven Five    -> 573
          NoPair King Ten Nine Seven Six     -> 574
          NoPair King Ten Nine Eight Two     -> 575
          NoPair King Ten Nine Eight Three   -> 576
          NoPair King Ten Nine Eight Four    -> 577
          NoPair King Ten Nine Eight Five    -> 578
          NoPair King Ten Nine Eight Six     -> 579
          NoPair King Ten Nine Eight Seven   -> 580
          NoPair King Jack Four Three Two    -> 581
          NoPair King Jack Five Three Two    -> 582
          NoPair King Jack Five Four Two     -> 583
          NoPair King Jack Five Four Three   -> 584
          NoPair King Jack Six Three Two     -> 585
          NoPair King Jack Six Four Two      -> 586
          NoPair King Jack Six Four Three    -> 587
          NoPair King Jack Six Five Two      -> 588
          NoPair King Jack Six Five Three    -> 589
          NoPair King Jack Six Five Four     -> 590
          NoPair King Jack Seven Three Two   -> 591
          NoPair King Jack Seven Four Two    -> 592
          NoPair King Jack Seven Four Three  -> 593
          NoPair King Jack Seven Five Two    -> 594
          NoPair King Jack Seven Five Three  -> 595
          NoPair King Jack Seven Five Four   -> 596
          NoPair King Jack Seven Six Two     -> 597
          NoPair King Jack Seven Six Three   -> 598
          NoPair King Jack Seven Six Four    -> 599
          NoPair King Jack Seven Six Five    -> 600
          NoPair King Jack Eight Three Two   -> 601
          NoPair King Jack Eight Four Two    -> 602
          NoPair King Jack Eight Four Three  -> 603
          NoPair King Jack Eight Five Two    -> 604
          NoPair King Jack Eight Five Three  -> 605
          NoPair King Jack Eight Five Four   -> 606
          NoPair King Jack Eight Six Two     -> 607
          NoPair King Jack Eight Six Three   -> 608
          NoPair King Jack Eight Six Four    -> 609
          NoPair King Jack Eight Six Five    -> 610
          NoPair King Jack Eight Seven Two   -> 611
          NoPair King Jack Eight Seven Three -> 612
          NoPair King Jack Eight Seven Four  -> 613
          NoPair King Jack Eight Seven Five  -> 614
          NoPair King Jack Eight Seven Six   -> 615
          NoPair King Jack Nine Three Two    -> 616
          NoPair King Jack Nine Four Two     -> 617
          NoPair King Jack Nine Four Three   -> 618
          NoPair King Jack Nine Five Two     -> 619
          NoPair King Jack Nine Five Three   -> 620
          NoPair King Jack Nine Five Four    -> 621
          NoPair King Jack Nine Six Two      -> 622
          NoPair King Jack Nine Six Three    -> 623
          NoPair King Jack Nine Six Four     -> 624
          NoPair King Jack Nine Six Five     -> 625
          NoPair King Jack Nine Seven Two    -> 626
          NoPair King Jack Nine Seven Three  -> 627
          NoPair King Jack Nine Seven Four   -> 628
          NoPair King Jack Nine Seven Five   -> 629
          NoPair King Jack Nine Seven Six    -> 630
          NoPair King Jack Nine Eight Two    -> 631
          NoPair King Jack Nine Eight Three  -> 632
          NoPair King Jack Nine Eight Four   -> 633
          NoPair King Jack Nine Eight Five   -> 634
          NoPair King Jack Nine Eight Six    -> 635
          NoPair King Jack Nine Eight Seven  -> 636
          NoPair King Jack Ten Three Two     -> 637
          NoPair King Jack Ten Four Two      -> 638
          NoPair King Jack Ten Four Three    -> 639
          NoPair King Jack Ten Five Two      -> 640
          NoPair King Jack Ten Five Three    -> 641
          NoPair King Jack Ten Five Four     -> 642
          NoPair King Jack Ten Six Two       -> 643
          NoPair King Jack Ten Six Three     -> 644
          NoPair King Jack Ten Six Four      -> 645
          NoPair King Jack Ten Six Five      -> 646
          NoPair King Jack Ten Seven Two     -> 647
          NoPair King Jack Ten Seven Three   -> 648
          NoPair King Jack Ten Seven Four    -> 649
          NoPair King Jack Ten Seven Five    -> 650
          NoPair King Jack Ten Seven Six     -> 651
          NoPair King Jack Ten Eight Two     -> 652
          NoPair King Jack Ten Eight Three   -> 653
          NoPair King Jack Ten Eight Four    -> 654
          NoPair King Jack Ten Eight Five    -> 655
          NoPair King Jack Ten Eight Six     -> 656
          NoPair King Jack Ten Eight Seven   -> 657
          NoPair King Jack Ten Nine Two      -> 658
          NoPair King Jack Ten Nine Three    -> 659
          NoPair King Jack Ten Nine Four     -> 660
          NoPair King Jack Ten Nine Five     -> 661
          NoPair King Jack Ten Nine Six      -> 662
          NoPair King Jack Ten Nine Seven    -> 663
          NoPair King Jack Ten Nine Eight    -> 664
          NoPair King Queen Four Three Two   -> 665
          NoPair King Queen Five Three Two   -> 666
          NoPair King Queen Five Four Two    -> 667
          NoPair King Queen Five Four Three  -> 668
          NoPair King Queen Six Three Two    -> 669
          NoPair King Queen Six Four Two     -> 670
          NoPair King Queen Six Four Three   -> 671
          NoPair King Queen Six Five Two     -> 672
          NoPair King Queen Six Five Three   -> 673
          NoPair King Queen Six Five Four    -> 674
          NoPair King Queen Seven Three Two  -> 675
          NoPair King Queen Seven Four Two   -> 676
          NoPair King Queen Seven Four Three -> 677
          NoPair King Queen Seven Five Two   -> 678
          NoPair King Queen Seven Five Three -> 679
          NoPair King Queen Seven Five Four  -> 680
          NoPair King Queen Seven Six Two    -> 681
          NoPair King Queen Seven Six Three  -> 682
          NoPair King Queen Seven Six Four   -> 683
          NoPair King Queen Seven Six Five   -> 684
          NoPair King Queen Eight Three Two  -> 685
          NoPair King Queen Eight Four Two   -> 686
          NoPair King Queen Eight Four Three -> 687
          NoPair King Queen Eight Five Two   -> 688
          NoPair King Queen Eight Five Three -> 689
          NoPair King Queen Eight Five Four  -> 690
          NoPair King Queen Eight Six Two    -> 691
          NoPair King Queen Eight Six Three  -> 692
          NoPair King Queen Eight Six Four   -> 693
          NoPair King Queen Eight Six Five   -> 694
          NoPair King Queen Eight Seven Two  -> 695
          NoPair King Queen Eight Seven Three-> 696
          NoPair King Queen Eight Seven Four -> 697
          NoPair King Queen Eight Seven Five -> 698
          NoPair King Queen Eight Seven Six  -> 699
          NoPair King Queen Nine Three Two   -> 700
          NoPair King Queen Nine Four Two    -> 701
          NoPair King Queen Nine Four Three  -> 702
          NoPair King Queen Nine Five Two    -> 703
          NoPair King Queen Nine Five Three  -> 704
          NoPair King Queen Nine Five Four   -> 705
          NoPair King Queen Nine Six Two     -> 706
          NoPair King Queen Nine Six Three   -> 707
          NoPair King Queen Nine Six Four    -> 708
          NoPair King Queen Nine Six Five    -> 709
          NoPair King Queen Nine Seven Two   -> 710
          NoPair King Queen Nine Seven Three -> 711
          NoPair King Queen Nine Seven Four  -> 712
          NoPair King Queen Nine Seven Five  -> 713
          NoPair King Queen Nine Seven Six   -> 714
          NoPair King Queen Nine Eight Two   -> 715
          NoPair King Queen Nine Eight Three -> 716
          NoPair King Queen Nine Eight Four  -> 717
          NoPair King Queen Nine Eight Five  -> 718
          NoPair King Queen Nine Eight Six   -> 719
          NoPair King Queen Nine Eight Seven -> 720
          NoPair King Queen Ten Three Two    -> 721
          NoPair King Queen Ten Four Two     -> 722
          NoPair King Queen Ten Four Three   -> 723
          NoPair King Queen Ten Five Two     -> 724
          NoPair King Queen Ten Five Three   -> 725
          NoPair King Queen Ten Five Four    -> 726
          NoPair King Queen Ten Six Two      -> 727
          NoPair King Queen Ten Six Three    -> 728
          NoPair King Queen Ten Six Four     -> 729
          NoPair King Queen Ten Six Five     -> 730
          NoPair King Queen Ten Seven Two    -> 731
          NoPair King Queen Ten Seven Three  -> 732
          NoPair King Queen Ten Seven Four   -> 733
          NoPair King Queen Ten Seven Five   -> 734
          NoPair King Queen Ten Seven Six    -> 735
          NoPair King Queen Ten Eight Two    -> 736
          NoPair King Queen Ten Eight Three  -> 737
          NoPair King Queen Ten Eight Four   -> 738
          NoPair King Queen Ten Eight Five   -> 739
          NoPair King Queen Ten Eight Six    -> 740
          NoPair King Queen Ten Eight Seven  -> 741
          NoPair King Queen Ten Nine Two     -> 742
          NoPair King Queen Ten Nine Three   -> 743
          NoPair King Queen Ten Nine Four    -> 744
          NoPair King Queen Ten Nine Five    -> 745
          NoPair King Queen Ten Nine Six     -> 746
          NoPair King Queen Ten Nine Seven   -> 747
          NoPair King Queen Ten Nine Eight   -> 748
          NoPair King Queen Jack Three Two   -> 749
          NoPair King Queen Jack Four Two    -> 750
          NoPair King Queen Jack Four Three  -> 751
          NoPair King Queen Jack Five Two    -> 752
          NoPair King Queen Jack Five Three  -> 753
          NoPair King Queen Jack Five Four   -> 754
          NoPair King Queen Jack Six Two     -> 755
          NoPair King Queen Jack Six Three   -> 756
          NoPair King Queen Jack Six Four    -> 757
          NoPair King Queen Jack Six Five    -> 758
          NoPair King Queen Jack Seven Two   -> 759
          NoPair King Queen Jack Seven Three -> 760
          NoPair King Queen Jack Seven Four  -> 761
          NoPair King Queen Jack Seven Five  -> 762
          NoPair King Queen Jack Seven Six   -> 763
          NoPair King Queen Jack Eight Two   -> 764
          NoPair King Queen Jack Eight Three -> 765
          NoPair King Queen Jack Eight Four  -> 766
          NoPair King Queen Jack Eight Five  -> 767
          NoPair King Queen Jack Eight Six   -> 768
          NoPair King Queen Jack Eight Seven -> 769
          NoPair King Queen Jack Nine Two    -> 770
          NoPair King Queen Jack Nine Three  -> 771
          NoPair King Queen Jack Nine Four   -> 772
          NoPair King Queen Jack Nine Five   -> 773
          NoPair King Queen Jack Nine Six    -> 774
          NoPair King Queen Jack Nine Seven  -> 775
          NoPair King Queen Jack Nine Eight  -> 776
          NoPair King Queen Jack Ten Two     -> 777
          NoPair King Queen Jack Ten Three   -> 778
          NoPair King Queen Jack Ten Four    -> 779
          NoPair King Queen Jack Ten Five    -> 780
          NoPair King Queen Jack Ten Six     -> 781
          NoPair King Queen Jack Ten Seven   -> 782
          NoPair King Queen Jack Ten Eight   -> 783
          NoPair Ace Five Four Three Two     -> 784
          NoPair Ace Six Four Three Two      -> 785
          NoPair Ace Six Five Three Two      -> 786
          NoPair Ace Six Five Four Two       -> 787
          NoPair Ace Six Five Four Three     -> 788
          NoPair Ace Seven Four Three Two    -> 789
          NoPair Ace Seven Five Three Two    -> 790
          NoPair Ace Seven Five Four Two     -> 791
          NoPair Ace Seven Five Four Three   -> 792
          NoPair Ace Seven Six Three Two     -> 793
          NoPair Ace Seven Six Four Two      -> 794
          NoPair Ace Seven Six Four Three    -> 795
          NoPair Ace Seven Six Five Two      -> 796
          NoPair Ace Seven Six Five Three    -> 797
          NoPair Ace Seven Six Five Four     -> 798
          NoPair Ace Eight Four Three Two    -> 799
          NoPair Ace Eight Five Three Two    -> 800
          NoPair Ace Eight Five Four Two     -> 801
          NoPair Ace Eight Five Four Three   -> 802
          NoPair Ace Eight Six Three Two     -> 803
          NoPair Ace Eight Six Four Two      -> 804
          NoPair Ace Eight Six Four Three    -> 805
          NoPair Ace Eight Six Five Two      -> 806
          NoPair Ace Eight Six Five Three    -> 807
          NoPair Ace Eight Six Five Four     -> 808
          NoPair Ace Eight Seven Three Two   -> 809
          NoPair Ace Eight Seven Four Two    -> 810
          NoPair Ace Eight Seven Four Three  -> 811
          NoPair Ace Eight Seven Five Two    -> 812
          NoPair Ace Eight Seven Five Three  -> 813
          NoPair Ace Eight Seven Five Four   -> 814
          NoPair Ace Eight Seven Six Two     -> 815
          NoPair Ace Eight Seven Six Three   -> 816
          NoPair Ace Eight Seven Six Four    -> 817
          NoPair Ace Eight Seven Six Five    -> 818
          NoPair Ace Nine Four Three Two     -> 819
          NoPair Ace Nine Five Three Two     -> 820
          NoPair Ace Nine Five Four Two      -> 821
          NoPair Ace Nine Five Four Three    -> 822
          NoPair Ace Nine Six Three Two      -> 823
          NoPair Ace Nine Six Four Two       -> 824
          NoPair Ace Nine Six Four Three     -> 825
          NoPair Ace Nine Six Five Two       -> 826
          NoPair Ace Nine Six Five Three     -> 827
          NoPair Ace Nine Six Five Four      -> 828
          NoPair Ace Nine Seven Three Two    -> 829
          NoPair Ace Nine Seven Four Two     -> 830
          NoPair Ace Nine Seven Four Three   -> 831
          NoPair Ace Nine Seven Five Two     -> 832
          NoPair Ace Nine Seven Five Three   -> 833
          NoPair Ace Nine Seven Five Four    -> 834
          NoPair Ace Nine Seven Six Two      -> 835
          NoPair Ace Nine Seven Six Three    -> 836
          NoPair Ace Nine Seven Six Four     -> 837
          NoPair Ace Nine Seven Six Five     -> 838
          NoPair Ace Nine Eight Three Two    -> 839
          NoPair Ace Nine Eight Four Two     -> 840
          NoPair Ace Nine Eight Four Three   -> 841
          NoPair Ace Nine Eight Five Two     -> 842
          NoPair Ace Nine Eight Five Three   -> 843
          NoPair Ace Nine Eight Five Four    -> 844
          NoPair Ace Nine Eight Six Two      -> 845
          NoPair Ace Nine Eight Six Three    -> 846
          NoPair Ace Nine Eight Six Four     -> 847
          NoPair Ace Nine Eight Six Five     -> 848
          NoPair Ace Nine Eight Seven Two    -> 849
          NoPair Ace Nine Eight Seven Three  -> 850
          NoPair Ace Nine Eight Seven Four   -> 851
          NoPair Ace Nine Eight Seven Five   -> 852
          NoPair Ace Nine Eight Seven Six    -> 853
          NoPair Ace Ten Four Three Two      -> 854
          NoPair Ace Ten Five Three Two      -> 855
          NoPair Ace Ten Five Four Two       -> 856
          NoPair Ace Ten Five Four Three     -> 857
          NoPair Ace Ten Six Three Two       -> 858
          NoPair Ace Ten Six Four Two        -> 859
          NoPair Ace Ten Six Four Three      -> 860
          NoPair Ace Ten Six Five Two        -> 861
          NoPair Ace Ten Six Five Three      -> 862
          NoPair Ace Ten Six Five Four       -> 863
          NoPair Ace Ten Seven Three Two     -> 864
          NoPair Ace Ten Seven Four Two      -> 865
          NoPair Ace Ten Seven Four Three    -> 866
          NoPair Ace Ten Seven Five Two      -> 867
          NoPair Ace Ten Seven Five Three    -> 868
          NoPair Ace Ten Seven Five Four     -> 869
          NoPair Ace Ten Seven Six Two       -> 870
          NoPair Ace Ten Seven Six Three     -> 871
          NoPair Ace Ten Seven Six Four      -> 872
          NoPair Ace Ten Seven Six Five      -> 873
          NoPair Ace Ten Eight Three Two     -> 874
          NoPair Ace Ten Eight Four Two      -> 875
          NoPair Ace Ten Eight Four Three    -> 876
          NoPair Ace Ten Eight Five Two      -> 877
          NoPair Ace Ten Eight Five Three    -> 878
          NoPair Ace Ten Eight Five Four     -> 879
          NoPair Ace Ten Eight Six Two       -> 880
          NoPair Ace Ten Eight Six Three     -> 881
          NoPair Ace Ten Eight Six Four      -> 882
          NoPair Ace Ten Eight Six Five      -> 883
          NoPair Ace Ten Eight Seven Two     -> 884
          NoPair Ace Ten Eight Seven Three   -> 885
          NoPair Ace Ten Eight Seven Four    -> 886
          NoPair Ace Ten Eight Seven Five    -> 887
          NoPair Ace Ten Eight Seven Six     -> 888
          NoPair Ace Ten Nine Three Two      -> 889
          NoPair Ace Ten Nine Four Two       -> 890
          NoPair Ace Ten Nine Four Three     -> 891
          NoPair Ace Ten Nine Five Two       -> 892
          NoPair Ace Ten Nine Five Three     -> 893
          NoPair Ace Ten Nine Five Four      -> 894
          NoPair Ace Ten Nine Six Two        -> 895
          NoPair Ace Ten Nine Six Three      -> 896
          NoPair Ace Ten Nine Six Four       -> 897
          NoPair Ace Ten Nine Six Five       -> 898
          NoPair Ace Ten Nine Seven Two      -> 899
          NoPair Ace Ten Nine Seven Three    -> 900
          NoPair Ace Ten Nine Seven Four     -> 901
          NoPair Ace Ten Nine Seven Five     -> 902
          NoPair Ace Ten Nine Seven Six      -> 903
          NoPair Ace Ten Nine Eight Two      -> 904
          NoPair Ace Ten Nine Eight Three    -> 905
          NoPair Ace Ten Nine Eight Four     -> 906
          NoPair Ace Ten Nine Eight Five     -> 907
          NoPair Ace Ten Nine Eight Six      -> 908
          NoPair Ace Ten Nine Eight Seven    -> 909
          NoPair Ace Jack Four Three Two     -> 910
          NoPair Ace Jack Five Three Two     -> 911
          NoPair Ace Jack Five Four Two      -> 912
          NoPair Ace Jack Five Four Three    -> 913
          NoPair Ace Jack Six Three Two      -> 914
          NoPair Ace Jack Six Four Two       -> 915
          NoPair Ace Jack Six Four Three     -> 916
          NoPair Ace Jack Six Five Two       -> 917
          NoPair Ace Jack Six Five Three     -> 918
          NoPair Ace Jack Six Five Four      -> 919
          NoPair Ace Jack Seven Three Two    -> 920
          NoPair Ace Jack Seven Four Two     -> 921
          NoPair Ace Jack Seven Four Three   -> 922
          NoPair Ace Jack Seven Five Two     -> 923
          NoPair Ace Jack Seven Five Three   -> 924
          NoPair Ace Jack Seven Five Four    -> 925
          NoPair Ace Jack Seven Six Two      -> 926
          NoPair Ace Jack Seven Six Three    -> 927
          NoPair Ace Jack Seven Six Four     -> 928
          NoPair Ace Jack Seven Six Five     -> 929
          NoPair Ace Jack Eight Three Two    -> 930
          NoPair Ace Jack Eight Four Two     -> 931
          NoPair Ace Jack Eight Four Three   -> 932
          NoPair Ace Jack Eight Five Two     -> 933
          NoPair Ace Jack Eight Five Three   -> 934
          NoPair Ace Jack Eight Five Four    -> 935
          NoPair Ace Jack Eight Six Two      -> 936
          NoPair Ace Jack Eight Six Three    -> 937
          NoPair Ace Jack Eight Six Four     -> 938
          NoPair Ace Jack Eight Six Five     -> 939
          NoPair Ace Jack Eight Seven Two    -> 940
          NoPair Ace Jack Eight Seven Three  -> 941
          NoPair Ace Jack Eight Seven Four   -> 942
          NoPair Ace Jack Eight Seven Five   -> 943
          NoPair Ace Jack Eight Seven Six    -> 944
          NoPair Ace Jack Nine Three Two     -> 945
          NoPair Ace Jack Nine Four Two      -> 946
          NoPair Ace Jack Nine Four Three    -> 947
          NoPair Ace Jack Nine Five Two      -> 948
          NoPair Ace Jack Nine Five Three    -> 949
          NoPair Ace Jack Nine Five Four     -> 950
          NoPair Ace Jack Nine Six Two       -> 951
          NoPair Ace Jack Nine Six Three     -> 952
          NoPair Ace Jack Nine Six Four      -> 953
          NoPair Ace Jack Nine Six Five      -> 954
          NoPair Ace Jack Nine Seven Two     -> 955
          NoPair Ace Jack Nine Seven Three   -> 956
          NoPair Ace Jack Nine Seven Four    -> 957
          NoPair Ace Jack Nine Seven Five    -> 958
          NoPair Ace Jack Nine Seven Six     -> 959
          NoPair Ace Jack Nine Eight Two     -> 960
          NoPair Ace Jack Nine Eight Three   -> 961
          NoPair Ace Jack Nine Eight Four    -> 962
          NoPair Ace Jack Nine Eight Five    -> 963
          NoPair Ace Jack Nine Eight Six     -> 964
          NoPair Ace Jack Nine Eight Seven   -> 965
          NoPair Ace Jack Ten Three Two      -> 966
          NoPair Ace Jack Ten Four Two       -> 967
          NoPair Ace Jack Ten Four Three     -> 968
          NoPair Ace Jack Ten Five Two       -> 969
          NoPair Ace Jack Ten Five Three     -> 970
          NoPair Ace Jack Ten Five Four      -> 971
          NoPair Ace Jack Ten Six Two        -> 972
          NoPair Ace Jack Ten Six Three      -> 973
          NoPair Ace Jack Ten Six Four       -> 974
          NoPair Ace Jack Ten Six Five       -> 975
          NoPair Ace Jack Ten Seven Two      -> 976
          NoPair Ace Jack Ten Seven Three    -> 977
          NoPair Ace Jack Ten Seven Four     -> 978
          NoPair Ace Jack Ten Seven Five     -> 979
          NoPair Ace Jack Ten Seven Six      -> 980
          NoPair Ace Jack Ten Eight Two      -> 981
          NoPair Ace Jack Ten Eight Three    -> 982
          NoPair Ace Jack Ten Eight Four     -> 983
          NoPair Ace Jack Ten Eight Five     -> 984
          NoPair Ace Jack Ten Eight Six      -> 985
          NoPair Ace Jack Ten Eight Seven    -> 986
          NoPair Ace Jack Ten Nine Two       -> 987
          NoPair Ace Jack Ten Nine Three     -> 988
          NoPair Ace Jack Ten Nine Four      -> 989
          NoPair Ace Jack Ten Nine Five      -> 990
          NoPair Ace Jack Ten Nine Six       -> 991
          NoPair Ace Jack Ten Nine Seven     -> 992
          NoPair Ace Jack Ten Nine Eight     -> 993
          NoPair Ace Queen Four Three Two    -> 994
          NoPair Ace Queen Five Three Two    -> 995
          NoPair Ace Queen Five Four Two     -> 996
          NoPair Ace Queen Five Four Three   -> 997
          NoPair Ace Queen Six Three Two     -> 998
          NoPair Ace Queen Six Four Two      -> 999
          NoPair Ace Queen Six Four Three    -> 1000
          NoPair Ace Queen Six Five Two      -> 1001
          NoPair Ace Queen Six Five Three    -> 1002
          NoPair Ace Queen Six Five Four     -> 1003
          NoPair Ace Queen Seven Three Two   -> 1004
          NoPair Ace Queen Seven Four Two    -> 1005
          NoPair Ace Queen Seven Four Three  -> 1006
          NoPair Ace Queen Seven Five Two    -> 1007
          NoPair Ace Queen Seven Five Three  -> 1008
          NoPair Ace Queen Seven Five Four   -> 1009
          NoPair Ace Queen Seven Six Two     -> 1010
          NoPair Ace Queen Seven Six Three   -> 1011
          NoPair Ace Queen Seven Six Four    -> 1012
          NoPair Ace Queen Seven Six Five    -> 1013
          NoPair Ace Queen Eight Three Two   -> 1014
          NoPair Ace Queen Eight Four Two    -> 1015
          NoPair Ace Queen Eight Four Three  -> 1016
          NoPair Ace Queen Eight Five Two    -> 1017
          NoPair Ace Queen Eight Five Three  -> 1018
          NoPair Ace Queen Eight Five Four   -> 1019
          NoPair Ace Queen Eight Six Two     -> 1020
          NoPair Ace Queen Eight Six Three   -> 1021
          NoPair Ace Queen Eight Six Four    -> 1022
          NoPair Ace Queen Eight Six Five    -> 1023
          NoPair Ace Queen Eight Seven Two   -> 1024
          NoPair Ace Queen Eight Seven Three -> 1025
          NoPair Ace Queen Eight Seven Four  -> 1026
          NoPair Ace Queen Eight Seven Five  -> 1027
          NoPair Ace Queen Eight Seven Six   -> 1028
          NoPair Ace Queen Nine Three Two    -> 1029
          NoPair Ace Queen Nine Four Two     -> 1030
          NoPair Ace Queen Nine Four Three   -> 1031
          NoPair Ace Queen Nine Five Two     -> 1032
          NoPair Ace Queen Nine Five Three   -> 1033
          NoPair Ace Queen Nine Five Four    -> 1034
          NoPair Ace Queen Nine Six Two      -> 1035
          NoPair Ace Queen Nine Six Three    -> 1036
          NoPair Ace Queen Nine Six Four     -> 1037
          NoPair Ace Queen Nine Six Five     -> 1038
          NoPair Ace Queen Nine Seven Two    -> 1039
          NoPair Ace Queen Nine Seven Three  -> 1040
          NoPair Ace Queen Nine Seven Four   -> 1041
          NoPair Ace Queen Nine Seven Five   -> 1042
          NoPair Ace Queen Nine Seven Six    -> 1043
          NoPair Ace Queen Nine Eight Two    -> 1044
          NoPair Ace Queen Nine Eight Three  -> 1045
          NoPair Ace Queen Nine Eight Four   -> 1046
          NoPair Ace Queen Nine Eight Five   -> 1047
          NoPair Ace Queen Nine Eight Six    -> 1048
          NoPair Ace Queen Nine Eight Seven  -> 1049
          NoPair Ace Queen Ten Three Two     -> 1050
          NoPair Ace Queen Ten Four Two      -> 1051
          NoPair Ace Queen Ten Four Three    -> 1052
          NoPair Ace Queen Ten Five Two      -> 1053
          NoPair Ace Queen Ten Five Three    -> 1054
          NoPair Ace Queen Ten Five Four     -> 1055
          NoPair Ace Queen Ten Six Two       -> 1056
          NoPair Ace Queen Ten Six Three     -> 1057
          NoPair Ace Queen Ten Six Four      -> 1058
          NoPair Ace Queen Ten Six Five      -> 1059
          NoPair Ace Queen Ten Seven Two     -> 1060
          NoPair Ace Queen Ten Seven Three   -> 1061
          NoPair Ace Queen Ten Seven Four    -> 1062
          NoPair Ace Queen Ten Seven Five    -> 1063
          NoPair Ace Queen Ten Seven Six     -> 1064
          NoPair Ace Queen Ten Eight Two     -> 1065
          NoPair Ace Queen Ten Eight Three   -> 1066
          NoPair Ace Queen Ten Eight Four    -> 1067
          NoPair Ace Queen Ten Eight Five    -> 1068
          NoPair Ace Queen Ten Eight Six     -> 1069
          NoPair Ace Queen Ten Eight Seven   -> 1070
          NoPair Ace Queen Ten Nine Two      -> 1071
          NoPair Ace Queen Ten Nine Three    -> 1072
          NoPair Ace Queen Ten Nine Four     -> 1073
          NoPair Ace Queen Ten Nine Five     -> 1074
          NoPair Ace Queen Ten Nine Six      -> 1075
          NoPair Ace Queen Ten Nine Seven    -> 1076
          NoPair Ace Queen Ten Nine Eight    -> 1077
          NoPair Ace Queen Jack Three Two    -> 1078
          NoPair Ace Queen Jack Four Two     -> 1079
          NoPair Ace Queen Jack Four Three   -> 1080
          NoPair Ace Queen Jack Five Two     -> 1081
          NoPair Ace Queen Jack Five Three   -> 1082
          NoPair Ace Queen Jack Five Four    -> 1083
          NoPair Ace Queen Jack Six Two      -> 1084
          NoPair Ace Queen Jack Six Three    -> 1085
          NoPair Ace Queen Jack Six Four     -> 1086
          NoPair Ace Queen Jack Six Five     -> 1087
          NoPair Ace Queen Jack Seven Two    -> 1088
          NoPair Ace Queen Jack Seven Three  -> 1089
          NoPair Ace Queen Jack Seven Four   -> 1090
          NoPair Ace Queen Jack Seven Five   -> 1091
          NoPair Ace Queen Jack Seven Six    -> 1092
          NoPair Ace Queen Jack Eight Two    -> 1093
          NoPair Ace Queen Jack Eight Three  -> 1094
          NoPair Ace Queen Jack Eight Four   -> 1095
          NoPair Ace Queen Jack Eight Five   -> 1096
          NoPair Ace Queen Jack Eight Six    -> 1097
          NoPair Ace Queen Jack Eight Seven  -> 1098
          NoPair Ace Queen Jack Nine Two     -> 1099
          NoPair Ace Queen Jack Nine Three   -> 1100
          NoPair Ace Queen Jack Nine Four    -> 1101
          NoPair Ace Queen Jack Nine Five    -> 1102
          NoPair Ace Queen Jack Nine Six     -> 1103
          NoPair Ace Queen Jack Nine Seven   -> 1104
          NoPair Ace Queen Jack Nine Eight   -> 1105
          NoPair Ace Queen Jack Ten Two      -> 1106
          NoPair Ace Queen Jack Ten Three    -> 1107
          NoPair Ace Queen Jack Ten Four     -> 1108
          NoPair Ace Queen Jack Ten Five     -> 1109
          NoPair Ace Queen Jack Ten Six      -> 1110
          NoPair Ace Queen Jack Ten Seven    -> 1111
          NoPair Ace Queen Jack Ten Eight    -> 1112
          NoPair Ace Queen Jack Ten Nine     -> 1113
          NoPair Ace King Four Three Two     -> 1114
          NoPair Ace King Five Three Two     -> 1115
          NoPair Ace King Five Four Two      -> 1116
          NoPair Ace King Five Four Three    -> 1117
          NoPair Ace King Six Three Two      -> 1118
          NoPair Ace King Six Four Two       -> 1119
          NoPair Ace King Six Four Three     -> 1120
          NoPair Ace King Six Five Two       -> 1121
          NoPair Ace King Six Five Three     -> 1122
          NoPair Ace King Six Five Four      -> 1123
          NoPair Ace King Seven Three Two    -> 1124
          NoPair Ace King Seven Four Two     -> 1125
          NoPair Ace King Seven Four Three   -> 1126
          NoPair Ace King Seven Five Two     -> 1127
          NoPair Ace King Seven Five Three   -> 1128
          NoPair Ace King Seven Five Four    -> 1129
          NoPair Ace King Seven Six Two      -> 1130
          NoPair Ace King Seven Six Three    -> 1131
          NoPair Ace King Seven Six Four     -> 1132
          NoPair Ace King Seven Six Five     -> 1133
          NoPair Ace King Eight Three Two    -> 1134
          NoPair Ace King Eight Four Two     -> 1135
          NoPair Ace King Eight Four Three   -> 1136
          NoPair Ace King Eight Five Two     -> 1137
          NoPair Ace King Eight Five Three   -> 1138
          NoPair Ace King Eight Five Four    -> 1139
          NoPair Ace King Eight Six Two      -> 1140
          NoPair Ace King Eight Six Three    -> 1141
          NoPair Ace King Eight Six Four     -> 1142
          NoPair Ace King Eight Six Five     -> 1143
          NoPair Ace King Eight Seven Two    -> 1144
          NoPair Ace King Eight Seven Three  -> 1145
          NoPair Ace King Eight Seven Four   -> 1146
          NoPair Ace King Eight Seven Five   -> 1147
          NoPair Ace King Eight Seven Six    -> 1148
          NoPair Ace King Nine Three Two     -> 1149
          NoPair Ace King Nine Four Two      -> 1150
          NoPair Ace King Nine Four Three    -> 1151
          NoPair Ace King Nine Five Two      -> 1152
          NoPair Ace King Nine Five Three    -> 1153
          NoPair Ace King Nine Five Four     -> 1154
          NoPair Ace King Nine Six Two       -> 1155
          NoPair Ace King Nine Six Three     -> 1156
          NoPair Ace King Nine Six Four      -> 1157
          NoPair Ace King Nine Six Five      -> 1158
          NoPair Ace King Nine Seven Two     -> 1159
          NoPair Ace King Nine Seven Three   -> 1160
          NoPair Ace King Nine Seven Four    -> 1161
          NoPair Ace King Nine Seven Five    -> 1162
          NoPair Ace King Nine Seven Six     -> 1163
          NoPair Ace King Nine Eight Two     -> 1164
          NoPair Ace King Nine Eight Three   -> 1165
          NoPair Ace King Nine Eight Four    -> 1166
          NoPair Ace King Nine Eight Five    -> 1167
          NoPair Ace King Nine Eight Six     -> 1168
          NoPair Ace King Nine Eight Seven   -> 1169
          NoPair Ace King Ten Three Two      -> 1170
          NoPair Ace King Ten Four Two       -> 1171
          NoPair Ace King Ten Four Three     -> 1172
          NoPair Ace King Ten Five Two       -> 1173
          NoPair Ace King Ten Five Three     -> 1174
          NoPair Ace King Ten Five Four      -> 1175
          NoPair Ace King Ten Six Two        -> 1176
          NoPair Ace King Ten Six Three      -> 1177
          NoPair Ace King Ten Six Four       -> 1178
          NoPair Ace King Ten Six Five       -> 1179
          NoPair Ace King Ten Seven Two      -> 1180
          NoPair Ace King Ten Seven Three    -> 1181
          NoPair Ace King Ten Seven Four     -> 1182
          NoPair Ace King Ten Seven Five     -> 1183
          NoPair Ace King Ten Seven Six      -> 1184
          NoPair Ace King Ten Eight Two      -> 1185
          NoPair Ace King Ten Eight Three    -> 1186
          NoPair Ace King Ten Eight Four     -> 1187
          NoPair Ace King Ten Eight Five     -> 1188
          NoPair Ace King Ten Eight Six      -> 1189
          NoPair Ace King Ten Eight Seven    -> 1190
          NoPair Ace King Ten Nine Two       -> 1191
          NoPair Ace King Ten Nine Three     -> 1192
          NoPair Ace King Ten Nine Four      -> 1193
          NoPair Ace King Ten Nine Five      -> 1194
          NoPair Ace King Ten Nine Six       -> 1195
          NoPair Ace King Ten Nine Seven     -> 1196
          NoPair Ace King Ten Nine Eight     -> 1197
          NoPair Ace King Jack Three Two     -> 1198
          NoPair Ace King Jack Four Two      -> 1199
          NoPair Ace King Jack Four Three    -> 1200
          NoPair Ace King Jack Five Two      -> 1201
          NoPair Ace King Jack Five Three    -> 1202
          NoPair Ace King Jack Five Four     -> 1203
          NoPair Ace King Jack Six Two       -> 1204
          NoPair Ace King Jack Six Three     -> 1205
          NoPair Ace King Jack Six Four      -> 1206
          NoPair Ace King Jack Six Five      -> 1207
          NoPair Ace King Jack Seven Two     -> 1208
          NoPair Ace King Jack Seven Three   -> 1209
          NoPair Ace King Jack Seven Four    -> 1210
          NoPair Ace King Jack Seven Five    -> 1211
          NoPair Ace King Jack Seven Six     -> 1212
          NoPair Ace King Jack Eight Two     -> 1213
          NoPair Ace King Jack Eight Three   -> 1214
          NoPair Ace King Jack Eight Four    -> 1215
          NoPair Ace King Jack Eight Five    -> 1216
          NoPair Ace King Jack Eight Six     -> 1217
          NoPair Ace King Jack Eight Seven   -> 1218
          NoPair Ace King Jack Nine Two      -> 1219
          NoPair Ace King Jack Nine Three    -> 1220
          NoPair Ace King Jack Nine Four     -> 1221
          NoPair Ace King Jack Nine Five     -> 1222
          NoPair Ace King Jack Nine Six      -> 1223
          NoPair Ace King Jack Nine Seven    -> 1224
          NoPair Ace King Jack Nine Eight    -> 1225
          NoPair Ace King Jack Ten Two       -> 1226
          NoPair Ace King Jack Ten Three     -> 1227
          NoPair Ace King Jack Ten Four      -> 1228
          NoPair Ace King Jack Ten Five      -> 1229
          NoPair Ace King Jack Ten Six       -> 1230
          NoPair Ace King Jack Ten Seven     -> 1231
          NoPair Ace King Jack Ten Eight     -> 1232
          NoPair Ace King Jack Ten Nine      -> 1233
          NoPair Ace King Queen Three Two    -> 1234
          NoPair Ace King Queen Four Two     -> 1235
          NoPair Ace King Queen Four Three   -> 1236
          NoPair Ace King Queen Five Two     -> 1237
          NoPair Ace King Queen Five Three   -> 1238
          NoPair Ace King Queen Five Four    -> 1239
          NoPair Ace King Queen Six Two      -> 1240
          NoPair Ace King Queen Six Three    -> 1241
          NoPair Ace King Queen Six Four     -> 1242
          NoPair Ace King Queen Six Five     -> 1243
          NoPair Ace King Queen Seven Two    -> 1244
          NoPair Ace King Queen Seven Three  -> 1245
          NoPair Ace King Queen Seven Four   -> 1246
          NoPair Ace King Queen Seven Five   -> 1247
          NoPair Ace King Queen Seven Six    -> 1248
          NoPair Ace King Queen Eight Two    -> 1249
          NoPair Ace King Queen Eight Three  -> 1250
          NoPair Ace King Queen Eight Four   -> 1251
          NoPair Ace King Queen Eight Five   -> 1252
          NoPair Ace King Queen Eight Six    -> 1253
          NoPair Ace King Queen Eight Seven  -> 1254
          NoPair Ace King Queen Nine Two     -> 1255
          NoPair Ace King Queen Nine Three   -> 1256
          NoPair Ace King Queen Nine Four    -> 1257
          NoPair Ace King Queen Nine Five    -> 1258
          NoPair Ace King Queen Nine Six     -> 1259
          NoPair Ace King Queen Nine Seven   -> 1260
          NoPair Ace King Queen Nine Eight   -> 1261
          NoPair Ace King Queen Ten Two      -> 1262
          NoPair Ace King Queen Ten Three    -> 1263
          NoPair Ace King Queen Ten Four     -> 1264
          NoPair Ace King Queen Ten Five     -> 1265
          NoPair Ace King Queen Ten Six      -> 1266
          NoPair Ace King Queen Ten Seven    -> 1267
          NoPair Ace King Queen Ten Eight    -> 1268
          NoPair Ace King Queen Ten Nine     -> 1269
          NoPair Ace King Queen Jack Two     -> 1270
          NoPair Ace King Queen Jack Three   -> 1271
          NoPair Ace King Queen Jack Four    -> 1272
          NoPair Ace King Queen Jack Five    -> 1273
          NoPair Ace King Queen Jack Six     -> 1274
          NoPair Ace King Queen Jack Seven   -> 1275
          NoPair Ace King Queen Jack Eight   -> 1276
          NoPair Ace King Queen Jack Nine    -> 1277
          OnePair Two Five Four Three        -> 1278
          OnePair Two Six Four Three         -> 1279
          OnePair Two Six Five Three         -> 1280
          OnePair Two Six Five Four          -> 1281
          OnePair Two Seven Four Three       -> 1282
          OnePair Two Seven Five Three       -> 1283
          OnePair Two Seven Five Four        -> 1284
          OnePair Two Seven Six Three        -> 1285
          OnePair Two Seven Six Four         -> 1286
          OnePair Two Seven Six Five         -> 1287
          OnePair Two Eight Four Three       -> 1288
          OnePair Two Eight Five Three       -> 1289
          OnePair Two Eight Five Four        -> 1290
          OnePair Two Eight Six Three        -> 1291
          OnePair Two Eight Six Four         -> 1292
          OnePair Two Eight Six Five         -> 1293
          OnePair Two Eight Seven Three      -> 1294
          OnePair Two Eight Seven Four       -> 1295
          OnePair Two Eight Seven Five       -> 1296
          OnePair Two Eight Seven Six        -> 1297
          OnePair Two Nine Four Three        -> 1298
          OnePair Two Nine Five Three        -> 1299
          OnePair Two Nine Five Four         -> 1300
          OnePair Two Nine Six Three         -> 1301
          OnePair Two Nine Six Four          -> 1302
          OnePair Two Nine Six Five          -> 1303
          OnePair Two Nine Seven Three       -> 1304
          OnePair Two Nine Seven Four        -> 1305
          OnePair Two Nine Seven Five        -> 1306
          OnePair Two Nine Seven Six         -> 1307
          OnePair Two Nine Eight Three       -> 1308
          OnePair Two Nine Eight Four        -> 1309
          OnePair Two Nine Eight Five        -> 1310
          OnePair Two Nine Eight Six         -> 1311
          OnePair Two Nine Eight Seven       -> 1312
          OnePair Two Ten Four Three         -> 1313
          OnePair Two Ten Five Three         -> 1314
          OnePair Two Ten Five Four          -> 1315
          OnePair Two Ten Six Three          -> 1316
          OnePair Two Ten Six Four           -> 1317
          OnePair Two Ten Six Five           -> 1318
          OnePair Two Ten Seven Three        -> 1319
          OnePair Two Ten Seven Four         -> 1320
          OnePair Two Ten Seven Five         -> 1321
          OnePair Two Ten Seven Six          -> 1322
          OnePair Two Ten Eight Three        -> 1323
          OnePair Two Ten Eight Four         -> 1324
          OnePair Two Ten Eight Five         -> 1325
          OnePair Two Ten Eight Six          -> 1326
          OnePair Two Ten Eight Seven        -> 1327
          OnePair Two Ten Nine Three         -> 1328
          OnePair Two Ten Nine Four          -> 1329
          OnePair Two Ten Nine Five          -> 1330
          OnePair Two Ten Nine Six           -> 1331
          OnePair Two Ten Nine Seven         -> 1332
          OnePair Two Ten Nine Eight         -> 1333
          OnePair Two Jack Four Three        -> 1334
          OnePair Two Jack Five Three        -> 1335
          OnePair Two Jack Five Four         -> 1336
          OnePair Two Jack Six Three         -> 1337
          OnePair Two Jack Six Four          -> 1338
          OnePair Two Jack Six Five          -> 1339
          OnePair Two Jack Seven Three       -> 1340
          OnePair Two Jack Seven Four        -> 1341
          OnePair Two Jack Seven Five        -> 1342
          OnePair Two Jack Seven Six         -> 1343
          OnePair Two Jack Eight Three       -> 1344
          OnePair Two Jack Eight Four        -> 1345
          OnePair Two Jack Eight Five        -> 1346
          OnePair Two Jack Eight Six         -> 1347
          OnePair Two Jack Eight Seven       -> 1348
          OnePair Two Jack Nine Three        -> 1349
          OnePair Two Jack Nine Four         -> 1350
          OnePair Two Jack Nine Five         -> 1351
          OnePair Two Jack Nine Six          -> 1352
          OnePair Two Jack Nine Seven        -> 1353
          OnePair Two Jack Nine Eight        -> 1354
          OnePair Two Jack Ten Three         -> 1355
          OnePair Two Jack Ten Four          -> 1356
          OnePair Two Jack Ten Five          -> 1357
          OnePair Two Jack Ten Six           -> 1358
          OnePair Two Jack Ten Seven         -> 1359
          OnePair Two Jack Ten Eight         -> 1360
          OnePair Two Jack Ten Nine          -> 1361
          OnePair Two Queen Four Three       -> 1362
          OnePair Two Queen Five Three       -> 1363
          OnePair Two Queen Five Four        -> 1364
          OnePair Two Queen Six Three        -> 1365
          OnePair Two Queen Six Four         -> 1366
          OnePair Two Queen Six Five         -> 1367
          OnePair Two Queen Seven Three      -> 1368
          OnePair Two Queen Seven Four       -> 1369
          OnePair Two Queen Seven Five       -> 1370
          OnePair Two Queen Seven Six        -> 1371
          OnePair Two Queen Eight Three      -> 1372
          OnePair Two Queen Eight Four       -> 1373
          OnePair Two Queen Eight Five       -> 1374
          OnePair Two Queen Eight Six        -> 1375
          OnePair Two Queen Eight Seven      -> 1376
          OnePair Two Queen Nine Three       -> 1377
          OnePair Two Queen Nine Four        -> 1378
          OnePair Two Queen Nine Five        -> 1379
          OnePair Two Queen Nine Six         -> 1380
          OnePair Two Queen Nine Seven       -> 1381
          OnePair Two Queen Nine Eight       -> 1382
          OnePair Two Queen Ten Three        -> 1383
          OnePair Two Queen Ten Four         -> 1384
          OnePair Two Queen Ten Five         -> 1385
          OnePair Two Queen Ten Six          -> 1386
          OnePair Two Queen Ten Seven        -> 1387
          OnePair Two Queen Ten Eight        -> 1388
          OnePair Two Queen Ten Nine         -> 1389
          OnePair Two Queen Jack Three       -> 1390
          OnePair Two Queen Jack Four        -> 1391
          OnePair Two Queen Jack Five        -> 1392
          OnePair Two Queen Jack Six         -> 1393
          OnePair Two Queen Jack Seven       -> 1394
          OnePair Two Queen Jack Eight       -> 1395
          OnePair Two Queen Jack Nine        -> 1396
          OnePair Two Queen Jack Ten         -> 1397
          OnePair Two King Four Three        -> 1398
          OnePair Two King Five Three        -> 1399
          OnePair Two King Five Four         -> 1400
          OnePair Two King Six Three         -> 1401
          OnePair Two King Six Four          -> 1402
          OnePair Two King Six Five          -> 1403
          OnePair Two King Seven Three       -> 1404
          OnePair Two King Seven Four        -> 1405
          OnePair Two King Seven Five        -> 1406
          OnePair Two King Seven Six         -> 1407
          OnePair Two King Eight Three       -> 1408
          OnePair Two King Eight Four        -> 1409
          OnePair Two King Eight Five        -> 1410
          OnePair Two King Eight Six         -> 1411
          OnePair Two King Eight Seven       -> 1412
          OnePair Two King Nine Three        -> 1413
          OnePair Two King Nine Four         -> 1414
          OnePair Two King Nine Five         -> 1415
          OnePair Two King Nine Six          -> 1416
          OnePair Two King Nine Seven        -> 1417
          OnePair Two King Nine Eight        -> 1418
          OnePair Two King Ten Three         -> 1419
          OnePair Two King Ten Four          -> 1420
          OnePair Two King Ten Five          -> 1421
          OnePair Two King Ten Six           -> 1422
          OnePair Two King Ten Seven         -> 1423
          OnePair Two King Ten Eight         -> 1424
          OnePair Two King Ten Nine          -> 1425
          OnePair Two King Jack Three        -> 1426
          OnePair Two King Jack Four         -> 1427
          OnePair Two King Jack Five         -> 1428
          OnePair Two King Jack Six          -> 1429
          OnePair Two King Jack Seven        -> 1430
          OnePair Two King Jack Eight        -> 1431
          OnePair Two King Jack Nine         -> 1432
          OnePair Two King Jack Ten          -> 1433
          OnePair Two King Queen Three       -> 1434
          OnePair Two King Queen Four        -> 1435
          OnePair Two King Queen Five        -> 1436
          OnePair Two King Queen Six         -> 1437
          OnePair Two King Queen Seven       -> 1438
          OnePair Two King Queen Eight       -> 1439
          OnePair Two King Queen Nine        -> 1440
          OnePair Two King Queen Ten         -> 1441
          OnePair Two King Queen Jack        -> 1442
          OnePair Two Ace Four Three         -> 1443
          OnePair Two Ace Five Three         -> 1444
          OnePair Two Ace Five Four          -> 1445
          OnePair Two Ace Six Three          -> 1446
          OnePair Two Ace Six Four           -> 1447
          OnePair Two Ace Six Five           -> 1448
          OnePair Two Ace Seven Three        -> 1449
          OnePair Two Ace Seven Four         -> 1450
          OnePair Two Ace Seven Five         -> 1451
          OnePair Two Ace Seven Six          -> 1452
          OnePair Two Ace Eight Three        -> 1453
          OnePair Two Ace Eight Four         -> 1454
          OnePair Two Ace Eight Five         -> 1455
          OnePair Two Ace Eight Six          -> 1456
          OnePair Two Ace Eight Seven        -> 1457
          OnePair Two Ace Nine Three         -> 1458
          OnePair Two Ace Nine Four          -> 1459
          OnePair Two Ace Nine Five          -> 1460
          OnePair Two Ace Nine Six           -> 1461
          OnePair Two Ace Nine Seven         -> 1462
          OnePair Two Ace Nine Eight         -> 1463
          OnePair Two Ace Ten Three          -> 1464
          OnePair Two Ace Ten Four           -> 1465
          OnePair Two Ace Ten Five           -> 1466
          OnePair Two Ace Ten Six            -> 1467
          OnePair Two Ace Ten Seven          -> 1468
          OnePair Two Ace Ten Eight          -> 1469
          OnePair Two Ace Ten Nine           -> 1470
          OnePair Two Ace Jack Three         -> 1471
          OnePair Two Ace Jack Four          -> 1472
          OnePair Two Ace Jack Five          -> 1473
          OnePair Two Ace Jack Six           -> 1474
          OnePair Two Ace Jack Seven         -> 1475
          OnePair Two Ace Jack Eight         -> 1476
          OnePair Two Ace Jack Nine          -> 1477
          OnePair Two Ace Jack Ten           -> 1478
          OnePair Two Ace Queen Three        -> 1479
          OnePair Two Ace Queen Four         -> 1480
          OnePair Two Ace Queen Five         -> 1481
          OnePair Two Ace Queen Six          -> 1482
          OnePair Two Ace Queen Seven        -> 1483
          OnePair Two Ace Queen Eight        -> 1484
          OnePair Two Ace Queen Nine         -> 1485
          OnePair Two Ace Queen Ten          -> 1486
          OnePair Two Ace Queen Jack         -> 1487
          OnePair Two Ace King Three         -> 1488
          OnePair Two Ace King Four          -> 1489
          OnePair Two Ace King Five          -> 1490
          OnePair Two Ace King Six           -> 1491
          OnePair Two Ace King Seven         -> 1492
          OnePair Two Ace King Eight         -> 1493
          OnePair Two Ace King Nine          -> 1494
          OnePair Two Ace King Ten           -> 1495
          OnePair Two Ace King Jack          -> 1496
          OnePair Two Ace King Queen         -> 1497
          OnePair Three Five Four Two        -> 1498
          OnePair Three Six Four Two         -> 1499
          OnePair Three Six Five Two         -> 1500
          OnePair Three Six Five Four        -> 1501
          OnePair Three Seven Four Two       -> 1502
          OnePair Three Seven Five Two       -> 1503
          OnePair Three Seven Five Four      -> 1504
          OnePair Three Seven Six Two        -> 1505
          OnePair Three Seven Six Four       -> 1506
          OnePair Three Seven Six Five       -> 1507
          OnePair Three Eight Four Two       -> 1508
          OnePair Three Eight Five Two       -> 1509
          OnePair Three Eight Five Four      -> 1510
          OnePair Three Eight Six Two        -> 1511
          OnePair Three Eight Six Four       -> 1512
          OnePair Three Eight Six Five       -> 1513
          OnePair Three Eight Seven Two      -> 1514
          OnePair Three Eight Seven Four     -> 1515
          OnePair Three Eight Seven Five     -> 1516
          OnePair Three Eight Seven Six      -> 1517
          OnePair Three Nine Four Two        -> 1518
          OnePair Three Nine Five Two        -> 1519
          OnePair Three Nine Five Four       -> 1520
          OnePair Three Nine Six Two         -> 1521
          OnePair Three Nine Six Four        -> 1522
          OnePair Three Nine Six Five        -> 1523
          OnePair Three Nine Seven Two       -> 1524
          OnePair Three Nine Seven Four      -> 1525
          OnePair Three Nine Seven Five      -> 1526
          OnePair Three Nine Seven Six       -> 1527
          OnePair Three Nine Eight Two       -> 1528
          OnePair Three Nine Eight Four      -> 1529
          OnePair Three Nine Eight Five      -> 1530
          OnePair Three Nine Eight Six       -> 1531
          OnePair Three Nine Eight Seven     -> 1532
          OnePair Three Ten Four Two         -> 1533
          OnePair Three Ten Five Two         -> 1534
          OnePair Three Ten Five Four        -> 1535
          OnePair Three Ten Six Two          -> 1536
          OnePair Three Ten Six Four         -> 1537
          OnePair Three Ten Six Five         -> 1538
          OnePair Three Ten Seven Two        -> 1539
          OnePair Three Ten Seven Four       -> 1540
          OnePair Three Ten Seven Five       -> 1541
          OnePair Three Ten Seven Six        -> 1542
          OnePair Three Ten Eight Two        -> 1543
          OnePair Three Ten Eight Four       -> 1544
          OnePair Three Ten Eight Five       -> 1545
          OnePair Three Ten Eight Six        -> 1546
          OnePair Three Ten Eight Seven      -> 1547
          OnePair Three Ten Nine Two         -> 1548
          OnePair Three Ten Nine Four        -> 1549
          OnePair Three Ten Nine Five        -> 1550
          OnePair Three Ten Nine Six         -> 1551
          OnePair Three Ten Nine Seven       -> 1552
          OnePair Three Ten Nine Eight       -> 1553
          OnePair Three Jack Four Two        -> 1554
          OnePair Three Jack Five Two        -> 1555
          OnePair Three Jack Five Four       -> 1556
          OnePair Three Jack Six Two         -> 1557
          OnePair Three Jack Six Four        -> 1558
          OnePair Three Jack Six Five        -> 1559
          OnePair Three Jack Seven Two       -> 1560
          OnePair Three Jack Seven Four      -> 1561
          OnePair Three Jack Seven Five      -> 1562
          OnePair Three Jack Seven Six       -> 1563
          OnePair Three Jack Eight Two       -> 1564
          OnePair Three Jack Eight Four      -> 1565
          OnePair Three Jack Eight Five      -> 1566
          OnePair Three Jack Eight Six       -> 1567
          OnePair Three Jack Eight Seven     -> 1568
          OnePair Three Jack Nine Two        -> 1569
          OnePair Three Jack Nine Four       -> 1570
          OnePair Three Jack Nine Five       -> 1571
          OnePair Three Jack Nine Six        -> 1572
          OnePair Three Jack Nine Seven      -> 1573
          OnePair Three Jack Nine Eight      -> 1574
          OnePair Three Jack Ten Two         -> 1575
          OnePair Three Jack Ten Four        -> 1576
          OnePair Three Jack Ten Five        -> 1577
          OnePair Three Jack Ten Six         -> 1578
          OnePair Three Jack Ten Seven       -> 1579
          OnePair Three Jack Ten Eight       -> 1580
          OnePair Three Jack Ten Nine        -> 1581
          OnePair Three Queen Four Two       -> 1582
          OnePair Three Queen Five Two       -> 1583
          OnePair Three Queen Five Four      -> 1584
          OnePair Three Queen Six Two        -> 1585
          OnePair Three Queen Six Four       -> 1586
          OnePair Three Queen Six Five       -> 1587
          OnePair Three Queen Seven Two      -> 1588
          OnePair Three Queen Seven Four     -> 1589
          OnePair Three Queen Seven Five     -> 1590
          OnePair Three Queen Seven Six      -> 1591
          OnePair Three Queen Eight Two      -> 1592
          OnePair Three Queen Eight Four     -> 1593
          OnePair Three Queen Eight Five     -> 1594
          OnePair Three Queen Eight Six      -> 1595
          OnePair Three Queen Eight Seven    -> 1596
          OnePair Three Queen Nine Two       -> 1597
          OnePair Three Queen Nine Four      -> 1598
          OnePair Three Queen Nine Five      -> 1599
          OnePair Three Queen Nine Six       -> 1600
          OnePair Three Queen Nine Seven     -> 1601
          OnePair Three Queen Nine Eight     -> 1602
          OnePair Three Queen Ten Two        -> 1603
          OnePair Three Queen Ten Four       -> 1604
          OnePair Three Queen Ten Five       -> 1605
          OnePair Three Queen Ten Six        -> 1606
          OnePair Three Queen Ten Seven      -> 1607
          OnePair Three Queen Ten Eight      -> 1608
          OnePair Three Queen Ten Nine       -> 1609
          OnePair Three Queen Jack Two       -> 1610
          OnePair Three Queen Jack Four      -> 1611
          OnePair Three Queen Jack Five      -> 1612
          OnePair Three Queen Jack Six       -> 1613
          OnePair Three Queen Jack Seven     -> 1614
          OnePair Three Queen Jack Eight     -> 1615
          OnePair Three Queen Jack Nine      -> 1616
          OnePair Three Queen Jack Ten       -> 1617
          OnePair Three King Four Two        -> 1618
          OnePair Three King Five Two        -> 1619
          OnePair Three King Five Four       -> 1620
          OnePair Three King Six Two         -> 1621
          OnePair Three King Six Four        -> 1622
          OnePair Three King Six Five        -> 1623
          OnePair Three King Seven Two       -> 1624
          OnePair Three King Seven Four      -> 1625
          OnePair Three King Seven Five      -> 1626
          OnePair Three King Seven Six       -> 1627
          OnePair Three King Eight Two       -> 1628
          OnePair Three King Eight Four      -> 1629
          OnePair Three King Eight Five      -> 1630
          OnePair Three King Eight Six       -> 1631
          OnePair Three King Eight Seven     -> 1632
          OnePair Three King Nine Two        -> 1633
          OnePair Three King Nine Four       -> 1634
          OnePair Three King Nine Five       -> 1635
          OnePair Three King Nine Six        -> 1636
          OnePair Three King Nine Seven      -> 1637
          OnePair Three King Nine Eight      -> 1638
          OnePair Three King Ten Two         -> 1639
          OnePair Three King Ten Four        -> 1640
          OnePair Three King Ten Five        -> 1641
          OnePair Three King Ten Six         -> 1642
          OnePair Three King Ten Seven       -> 1643
          OnePair Three King Ten Eight       -> 1644
          OnePair Three King Ten Nine        -> 1645
          OnePair Three King Jack Two        -> 1646
          OnePair Three King Jack Four       -> 1647
          OnePair Three King Jack Five       -> 1648
          OnePair Three King Jack Six        -> 1649
          OnePair Three King Jack Seven      -> 1650
          OnePair Three King Jack Eight      -> 1651
          OnePair Three King Jack Nine       -> 1652
          OnePair Three King Jack Ten        -> 1653
          OnePair Three King Queen Two       -> 1654
          OnePair Three King Queen Four      -> 1655
          OnePair Three King Queen Five      -> 1656
          OnePair Three King Queen Six       -> 1657
          OnePair Three King Queen Seven     -> 1658
          OnePair Three King Queen Eight     -> 1659
          OnePair Three King Queen Nine      -> 1660
          OnePair Three King Queen Ten       -> 1661
          OnePair Three King Queen Jack      -> 1662
          OnePair Three Ace Four Two         -> 1663
          OnePair Three Ace Five Two         -> 1664
          OnePair Three Ace Five Four        -> 1665
          OnePair Three Ace Six Two          -> 1666
          OnePair Three Ace Six Four         -> 1667
          OnePair Three Ace Six Five         -> 1668
          OnePair Three Ace Seven Two        -> 1669
          OnePair Three Ace Seven Four       -> 1670
          OnePair Three Ace Seven Five       -> 1671
          OnePair Three Ace Seven Six        -> 1672
          OnePair Three Ace Eight Two        -> 1673
          OnePair Three Ace Eight Four       -> 1674
          OnePair Three Ace Eight Five       -> 1675
          OnePair Three Ace Eight Six        -> 1676
          OnePair Three Ace Eight Seven      -> 1677
          OnePair Three Ace Nine Two         -> 1678
          OnePair Three Ace Nine Four        -> 1679
          OnePair Three Ace Nine Five        -> 1680
          OnePair Three Ace Nine Six         -> 1681
          OnePair Three Ace Nine Seven       -> 1682
          OnePair Three Ace Nine Eight       -> 1683
          OnePair Three Ace Ten Two          -> 1684
          OnePair Three Ace Ten Four         -> 1685
          OnePair Three Ace Ten Five         -> 1686
          OnePair Three Ace Ten Six          -> 1687
          OnePair Three Ace Ten Seven        -> 1688
          OnePair Three Ace Ten Eight        -> 1689
          OnePair Three Ace Ten Nine         -> 1690
          OnePair Three Ace Jack Two         -> 1691
          OnePair Three Ace Jack Four        -> 1692
          OnePair Three Ace Jack Five        -> 1693
          OnePair Three Ace Jack Six         -> 1694
          OnePair Three Ace Jack Seven       -> 1695
          OnePair Three Ace Jack Eight       -> 1696
          OnePair Three Ace Jack Nine        -> 1697
          OnePair Three Ace Jack Ten         -> 1698
          OnePair Three Ace Queen Two        -> 1699
          OnePair Three Ace Queen Four       -> 1700
          OnePair Three Ace Queen Five       -> 1701
          OnePair Three Ace Queen Six        -> 1702
          OnePair Three Ace Queen Seven      -> 1703
          OnePair Three Ace Queen Eight      -> 1704
          OnePair Three Ace Queen Nine       -> 1705
          OnePair Three Ace Queen Ten        -> 1706
          OnePair Three Ace Queen Jack       -> 1707
          OnePair Three Ace King Two         -> 1708
          OnePair Three Ace King Four        -> 1709
          OnePair Three Ace King Five        -> 1710
          OnePair Three Ace King Six         -> 1711
          OnePair Three Ace King Seven       -> 1712
          OnePair Three Ace King Eight       -> 1713
          OnePair Three Ace King Nine        -> 1714
          OnePair Three Ace King Ten         -> 1715
          OnePair Three Ace King Jack        -> 1716
          OnePair Three Ace King Queen       -> 1717
          OnePair Four Five Three Two        -> 1718
          OnePair Four Six Three Two         -> 1719
          OnePair Four Six Five Two          -> 1720
          OnePair Four Six Five Three        -> 1721
          OnePair Four Seven Three Two       -> 1722
          OnePair Four Seven Five Two        -> 1723
          OnePair Four Seven Five Three      -> 1724
          OnePair Four Seven Six Two         -> 1725
          OnePair Four Seven Six Three       -> 1726
          OnePair Four Seven Six Five        -> 1727
          OnePair Four Eight Three Two       -> 1728
          OnePair Four Eight Five Two        -> 1729
          OnePair Four Eight Five Three      -> 1730
          OnePair Four Eight Six Two         -> 1731
          OnePair Four Eight Six Three       -> 1732
          OnePair Four Eight Six Five        -> 1733
          OnePair Four Eight Seven Two       -> 1734
          OnePair Four Eight Seven Three     -> 1735
          OnePair Four Eight Seven Five      -> 1736
          OnePair Four Eight Seven Six       -> 1737
          OnePair Four Nine Three Two        -> 1738
          OnePair Four Nine Five Two         -> 1739
          OnePair Four Nine Five Three       -> 1740
          OnePair Four Nine Six Two          -> 1741
          OnePair Four Nine Six Three        -> 1742
          OnePair Four Nine Six Five         -> 1743
          OnePair Four Nine Seven Two        -> 1744
          OnePair Four Nine Seven Three      -> 1745
          OnePair Four Nine Seven Five       -> 1746
          OnePair Four Nine Seven Six        -> 1747
          OnePair Four Nine Eight Two        -> 1748
          OnePair Four Nine Eight Three      -> 1749
          OnePair Four Nine Eight Five       -> 1750
          OnePair Four Nine Eight Six        -> 1751
          OnePair Four Nine Eight Seven      -> 1752
          OnePair Four Ten Three Two         -> 1753
          OnePair Four Ten Five Two          -> 1754
          OnePair Four Ten Five Three        -> 1755
          OnePair Four Ten Six Two           -> 1756
          OnePair Four Ten Six Three         -> 1757
          OnePair Four Ten Six Five          -> 1758
          OnePair Four Ten Seven Two         -> 1759
          OnePair Four Ten Seven Three       -> 1760
          OnePair Four Ten Seven Five        -> 1761
          OnePair Four Ten Seven Six         -> 1762
          OnePair Four Ten Eight Two         -> 1763
          OnePair Four Ten Eight Three       -> 1764
          OnePair Four Ten Eight Five        -> 1765
          OnePair Four Ten Eight Six         -> 1766
          OnePair Four Ten Eight Seven       -> 1767
          OnePair Four Ten Nine Two          -> 1768
          OnePair Four Ten Nine Three        -> 1769
          OnePair Four Ten Nine Five         -> 1770
          OnePair Four Ten Nine Six          -> 1771
          OnePair Four Ten Nine Seven        -> 1772
          OnePair Four Ten Nine Eight        -> 1773
          OnePair Four Jack Three Two        -> 1774
          OnePair Four Jack Five Two         -> 1775
          OnePair Four Jack Five Three       -> 1776
          OnePair Four Jack Six Two          -> 1777
          OnePair Four Jack Six Three        -> 1778
          OnePair Four Jack Six Five         -> 1779
          OnePair Four Jack Seven Two        -> 1780
          OnePair Four Jack Seven Three      -> 1781
          OnePair Four Jack Seven Five       -> 1782
          OnePair Four Jack Seven Six        -> 1783
          OnePair Four Jack Eight Two        -> 1784
          OnePair Four Jack Eight Three      -> 1785
          OnePair Four Jack Eight Five       -> 1786
          OnePair Four Jack Eight Six        -> 1787
          OnePair Four Jack Eight Seven      -> 1788
          OnePair Four Jack Nine Two         -> 1789
          OnePair Four Jack Nine Three       -> 1790
          OnePair Four Jack Nine Five        -> 1791
          OnePair Four Jack Nine Six         -> 1792
          OnePair Four Jack Nine Seven       -> 1793
          OnePair Four Jack Nine Eight       -> 1794
          OnePair Four Jack Ten Two          -> 1795
          OnePair Four Jack Ten Three        -> 1796
          OnePair Four Jack Ten Five         -> 1797
          OnePair Four Jack Ten Six          -> 1798
          OnePair Four Jack Ten Seven        -> 1799
          OnePair Four Jack Ten Eight        -> 1800
          OnePair Four Jack Ten Nine         -> 1801
          OnePair Four Queen Three Two       -> 1802
          OnePair Four Queen Five Two        -> 1803
          OnePair Four Queen Five Three      -> 1804
          OnePair Four Queen Six Two         -> 1805
          OnePair Four Queen Six Three       -> 1806
          OnePair Four Queen Six Five        -> 1807
          OnePair Four Queen Seven Two       -> 1808
          OnePair Four Queen Seven Three     -> 1809
          OnePair Four Queen Seven Five      -> 1810
          OnePair Four Queen Seven Six       -> 1811
          OnePair Four Queen Eight Two       -> 1812
          OnePair Four Queen Eight Three     -> 1813
          OnePair Four Queen Eight Five      -> 1814
          OnePair Four Queen Eight Six       -> 1815
          OnePair Four Queen Eight Seven     -> 1816
          OnePair Four Queen Nine Two        -> 1817
          OnePair Four Queen Nine Three      -> 1818
          OnePair Four Queen Nine Five       -> 1819
          OnePair Four Queen Nine Six        -> 1820
          OnePair Four Queen Nine Seven      -> 1821
          OnePair Four Queen Nine Eight      -> 1822
          OnePair Four Queen Ten Two         -> 1823
          OnePair Four Queen Ten Three       -> 1824
          OnePair Four Queen Ten Five        -> 1825
          OnePair Four Queen Ten Six         -> 1826
          OnePair Four Queen Ten Seven       -> 1827
          OnePair Four Queen Ten Eight       -> 1828
          OnePair Four Queen Ten Nine        -> 1829
          OnePair Four Queen Jack Two        -> 1830
          OnePair Four Queen Jack Three      -> 1831
          OnePair Four Queen Jack Five       -> 1832
          OnePair Four Queen Jack Six        -> 1833
          OnePair Four Queen Jack Seven      -> 1834
          OnePair Four Queen Jack Eight      -> 1835
          OnePair Four Queen Jack Nine       -> 1836
          OnePair Four Queen Jack Ten        -> 1837
          OnePair Four King Three Two        -> 1838
          OnePair Four King Five Two         -> 1839
          OnePair Four King Five Three       -> 1840
          OnePair Four King Six Two          -> 1841
          OnePair Four King Six Three        -> 1842
          OnePair Four King Six Five         -> 1843
          OnePair Four King Seven Two        -> 1844
          OnePair Four King Seven Three      -> 1845
          OnePair Four King Seven Five       -> 1846
          OnePair Four King Seven Six        -> 1847
          OnePair Four King Eight Two        -> 1848
          OnePair Four King Eight Three      -> 1849
          OnePair Four King Eight Five       -> 1850
          OnePair Four King Eight Six        -> 1851
          OnePair Four King Eight Seven      -> 1852
          OnePair Four King Nine Two         -> 1853
          OnePair Four King Nine Three       -> 1854
          OnePair Four King Nine Five        -> 1855
          OnePair Four King Nine Six         -> 1856
          OnePair Four King Nine Seven       -> 1857
          OnePair Four King Nine Eight       -> 1858
          OnePair Four King Ten Two          -> 1859
          OnePair Four King Ten Three        -> 1860
          OnePair Four King Ten Five         -> 1861
          OnePair Four King Ten Six          -> 1862
          OnePair Four King Ten Seven        -> 1863
          OnePair Four King Ten Eight        -> 1864
          OnePair Four King Ten Nine         -> 1865
          OnePair Four King Jack Two         -> 1866
          OnePair Four King Jack Three       -> 1867
          OnePair Four King Jack Five        -> 1868
          OnePair Four King Jack Six         -> 1869
          OnePair Four King Jack Seven       -> 1870
          OnePair Four King Jack Eight       -> 1871
          OnePair Four King Jack Nine        -> 1872
          OnePair Four King Jack Ten         -> 1873
          OnePair Four King Queen Two        -> 1874
          OnePair Four King Queen Three      -> 1875
          OnePair Four King Queen Five       -> 1876
          OnePair Four King Queen Six        -> 1877
          OnePair Four King Queen Seven      -> 1878
          OnePair Four King Queen Eight      -> 1879
          OnePair Four King Queen Nine       -> 1880
          OnePair Four King Queen Ten        -> 1881
          OnePair Four King Queen Jack       -> 1882
          OnePair Four Ace Three Two         -> 1883
          OnePair Four Ace Five Two          -> 1884
          OnePair Four Ace Five Three        -> 1885
          OnePair Four Ace Six Two           -> 1886
          OnePair Four Ace Six Three         -> 1887
          OnePair Four Ace Six Five          -> 1888
          OnePair Four Ace Seven Two         -> 1889
          OnePair Four Ace Seven Three       -> 1890
          OnePair Four Ace Seven Five        -> 1891
          OnePair Four Ace Seven Six         -> 1892
          OnePair Four Ace Eight Two         -> 1893
          OnePair Four Ace Eight Three       -> 1894
          OnePair Four Ace Eight Five        -> 1895
          OnePair Four Ace Eight Six         -> 1896
          OnePair Four Ace Eight Seven       -> 1897
          OnePair Four Ace Nine Two          -> 1898
          OnePair Four Ace Nine Three        -> 1899
          OnePair Four Ace Nine Five         -> 1900
          OnePair Four Ace Nine Six          -> 1901
          OnePair Four Ace Nine Seven        -> 1902
          OnePair Four Ace Nine Eight        -> 1903
          OnePair Four Ace Ten Two           -> 1904
          OnePair Four Ace Ten Three         -> 1905
          OnePair Four Ace Ten Five          -> 1906
          OnePair Four Ace Ten Six           -> 1907
          OnePair Four Ace Ten Seven         -> 1908
          OnePair Four Ace Ten Eight         -> 1909
          OnePair Four Ace Ten Nine          -> 1910
          OnePair Four Ace Jack Two          -> 1911
          OnePair Four Ace Jack Three        -> 1912
          OnePair Four Ace Jack Five         -> 1913
          OnePair Four Ace Jack Six          -> 1914
          OnePair Four Ace Jack Seven        -> 1915
          OnePair Four Ace Jack Eight        -> 1916
          OnePair Four Ace Jack Nine         -> 1917
          OnePair Four Ace Jack Ten          -> 1918
          OnePair Four Ace Queen Two         -> 1919
          OnePair Four Ace Queen Three       -> 1920
          OnePair Four Ace Queen Five        -> 1921
          OnePair Four Ace Queen Six         -> 1922
          OnePair Four Ace Queen Seven       -> 1923
          OnePair Four Ace Queen Eight       -> 1924
          OnePair Four Ace Queen Nine        -> 1925
          OnePair Four Ace Queen Ten         -> 1926
          OnePair Four Ace Queen Jack        -> 1927
          OnePair Four Ace King Two          -> 1928
          OnePair Four Ace King Three        -> 1929
          OnePair Four Ace King Five         -> 1930
          OnePair Four Ace King Six          -> 1931
          OnePair Four Ace King Seven        -> 1932
          OnePair Four Ace King Eight        -> 1933
          OnePair Four Ace King Nine         -> 1934
          OnePair Four Ace King Ten          -> 1935
          OnePair Four Ace King Jack         -> 1936
          OnePair Four Ace King Queen        -> 1937
          OnePair Five Four Three Two        -> 1938
          OnePair Five Six Three Two         -> 1939
          OnePair Five Six Four Two          -> 1940
          OnePair Five Six Four Three        -> 1941
          OnePair Five Seven Three Two       -> 1942
          OnePair Five Seven Four Two        -> 1943
          OnePair Five Seven Four Three      -> 1944
          OnePair Five Seven Six Two         -> 1945
          OnePair Five Seven Six Three       -> 1946
          OnePair Five Seven Six Four        -> 1947
          OnePair Five Eight Three Two       -> 1948
          OnePair Five Eight Four Two        -> 1949
          OnePair Five Eight Four Three      -> 1950
          OnePair Five Eight Six Two         -> 1951
          OnePair Five Eight Six Three       -> 1952
          OnePair Five Eight Six Four        -> 1953
          OnePair Five Eight Seven Two       -> 1954
          OnePair Five Eight Seven Three     -> 1955
          OnePair Five Eight Seven Four      -> 1956
          OnePair Five Eight Seven Six       -> 1957
          OnePair Five Nine Three Two        -> 1958
          OnePair Five Nine Four Two         -> 1959
          OnePair Five Nine Four Three       -> 1960
          OnePair Five Nine Six Two          -> 1961
          OnePair Five Nine Six Three        -> 1962
          OnePair Five Nine Six Four         -> 1963
          OnePair Five Nine Seven Two        -> 1964
          OnePair Five Nine Seven Three      -> 1965
          OnePair Five Nine Seven Four       -> 1966
          OnePair Five Nine Seven Six        -> 1967
          OnePair Five Nine Eight Two        -> 1968
          OnePair Five Nine Eight Three      -> 1969
          OnePair Five Nine Eight Four       -> 1970
          OnePair Five Nine Eight Six        -> 1971
          OnePair Five Nine Eight Seven      -> 1972
          OnePair Five Ten Three Two         -> 1973
          OnePair Five Ten Four Two          -> 1974
          OnePair Five Ten Four Three        -> 1975
          OnePair Five Ten Six Two           -> 1976
          OnePair Five Ten Six Three         -> 1977
          OnePair Five Ten Six Four          -> 1978
          OnePair Five Ten Seven Two         -> 1979
          OnePair Five Ten Seven Three       -> 1980
          OnePair Five Ten Seven Four        -> 1981
          OnePair Five Ten Seven Six         -> 1982
          OnePair Five Ten Eight Two         -> 1983
          OnePair Five Ten Eight Three       -> 1984
          OnePair Five Ten Eight Four        -> 1985
          OnePair Five Ten Eight Six         -> 1986
          OnePair Five Ten Eight Seven       -> 1987
          OnePair Five Ten Nine Two          -> 1988
          OnePair Five Ten Nine Three        -> 1989
          OnePair Five Ten Nine Four         -> 1990
          OnePair Five Ten Nine Six          -> 1991
          OnePair Five Ten Nine Seven        -> 1992
          OnePair Five Ten Nine Eight        -> 1993
          OnePair Five Jack Three Two        -> 1994
          OnePair Five Jack Four Two         -> 1995
          OnePair Five Jack Four Three       -> 1996
          OnePair Five Jack Six Two          -> 1997
          OnePair Five Jack Six Three        -> 1998
          OnePair Five Jack Six Four         -> 1999
          OnePair Five Jack Seven Two        -> 2000
          OnePair Five Jack Seven Three      -> 2001
          OnePair Five Jack Seven Four       -> 2002
          OnePair Five Jack Seven Six        -> 2003
          OnePair Five Jack Eight Two        -> 2004
          OnePair Five Jack Eight Three      -> 2005
          OnePair Five Jack Eight Four       -> 2006
          OnePair Five Jack Eight Six        -> 2007
          OnePair Five Jack Eight Seven      -> 2008
          OnePair Five Jack Nine Two         -> 2009
          OnePair Five Jack Nine Three       -> 2010
          OnePair Five Jack Nine Four        -> 2011
          OnePair Five Jack Nine Six         -> 2012
          OnePair Five Jack Nine Seven       -> 2013
          OnePair Five Jack Nine Eight       -> 2014
          OnePair Five Jack Ten Two          -> 2015
          OnePair Five Jack Ten Three        -> 2016
          OnePair Five Jack Ten Four         -> 2017
          OnePair Five Jack Ten Six          -> 2018
          OnePair Five Jack Ten Seven        -> 2019
          OnePair Five Jack Ten Eight        -> 2020
          OnePair Five Jack Ten Nine         -> 2021
          OnePair Five Queen Three Two       -> 2022
          OnePair Five Queen Four Two        -> 2023
          OnePair Five Queen Four Three      -> 2024
          OnePair Five Queen Six Two         -> 2025
          OnePair Five Queen Six Three       -> 2026
          OnePair Five Queen Six Four        -> 2027
          OnePair Five Queen Seven Two       -> 2028
          OnePair Five Queen Seven Three     -> 2029
          OnePair Five Queen Seven Four      -> 2030
          OnePair Five Queen Seven Six       -> 2031
          OnePair Five Queen Eight Two       -> 2032
          OnePair Five Queen Eight Three     -> 2033
          OnePair Five Queen Eight Four      -> 2034
          OnePair Five Queen Eight Six       -> 2035
          OnePair Five Queen Eight Seven     -> 2036
          OnePair Five Queen Nine Two        -> 2037
          OnePair Five Queen Nine Three      -> 2038
          OnePair Five Queen Nine Four       -> 2039
          OnePair Five Queen Nine Six        -> 2040
          OnePair Five Queen Nine Seven      -> 2041
          OnePair Five Queen Nine Eight      -> 2042
          OnePair Five Queen Ten Two         -> 2043
          OnePair Five Queen Ten Three       -> 2044
          OnePair Five Queen Ten Four        -> 2045
          OnePair Five Queen Ten Six         -> 2046
          OnePair Five Queen Ten Seven       -> 2047
          OnePair Five Queen Ten Eight       -> 2048
          OnePair Five Queen Ten Nine        -> 2049
          OnePair Five Queen Jack Two        -> 2050
          OnePair Five Queen Jack Three      -> 2051
          OnePair Five Queen Jack Four       -> 2052
          OnePair Five Queen Jack Six        -> 2053
          OnePair Five Queen Jack Seven      -> 2054
          OnePair Five Queen Jack Eight      -> 2055
          OnePair Five Queen Jack Nine       -> 2056
          OnePair Five Queen Jack Ten        -> 2057
          OnePair Five King Three Two        -> 2058
          OnePair Five King Four Two         -> 2059
          OnePair Five King Four Three       -> 2060
          OnePair Five King Six Two          -> 2061
          OnePair Five King Six Three        -> 2062
          OnePair Five King Six Four         -> 2063
          OnePair Five King Seven Two        -> 2064
          OnePair Five King Seven Three      -> 2065
          OnePair Five King Seven Four       -> 2066
          OnePair Five King Seven Six        -> 2067
          OnePair Five King Eight Two        -> 2068
          OnePair Five King Eight Three      -> 2069
          OnePair Five King Eight Four       -> 2070
          OnePair Five King Eight Six        -> 2071
          OnePair Five King Eight Seven      -> 2072
          OnePair Five King Nine Two         -> 2073
          OnePair Five King Nine Three       -> 2074
          OnePair Five King Nine Four        -> 2075
          OnePair Five King Nine Six         -> 2076
          OnePair Five King Nine Seven       -> 2077
          OnePair Five King Nine Eight       -> 2078
          OnePair Five King Ten Two          -> 2079
          OnePair Five King Ten Three        -> 2080
          OnePair Five King Ten Four         -> 2081
          OnePair Five King Ten Six          -> 2082
          OnePair Five King Ten Seven        -> 2083
          OnePair Five King Ten Eight        -> 2084
          OnePair Five King Ten Nine         -> 2085
          OnePair Five King Jack Two         -> 2086
          OnePair Five King Jack Three       -> 2087
          OnePair Five King Jack Four        -> 2088
          OnePair Five King Jack Six         -> 2089
          OnePair Five King Jack Seven       -> 2090
          OnePair Five King Jack Eight       -> 2091
          OnePair Five King Jack Nine        -> 2092
          OnePair Five King Jack Ten         -> 2093
          OnePair Five King Queen Two        -> 2094
          OnePair Five King Queen Three      -> 2095
          OnePair Five King Queen Four       -> 2096
          OnePair Five King Queen Six        -> 2097
          OnePair Five King Queen Seven      -> 2098
          OnePair Five King Queen Eight      -> 2099
          OnePair Five King Queen Nine       -> 2100
          OnePair Five King Queen Ten        -> 2101
          OnePair Five King Queen Jack       -> 2102
          OnePair Five Ace Three Two         -> 2103
          OnePair Five Ace Four Two          -> 2104
          OnePair Five Ace Four Three        -> 2105
          OnePair Five Ace Six Two           -> 2106
          OnePair Five Ace Six Three         -> 2107
          OnePair Five Ace Six Four          -> 2108
          OnePair Five Ace Seven Two         -> 2109
          OnePair Five Ace Seven Three       -> 2110
          OnePair Five Ace Seven Four        -> 2111
          OnePair Five Ace Seven Six         -> 2112
          OnePair Five Ace Eight Two         -> 2113
          OnePair Five Ace Eight Three       -> 2114
          OnePair Five Ace Eight Four        -> 2115
          OnePair Five Ace Eight Six         -> 2116
          OnePair Five Ace Eight Seven       -> 2117
          OnePair Five Ace Nine Two          -> 2118
          OnePair Five Ace Nine Three        -> 2119
          OnePair Five Ace Nine Four         -> 2120
          OnePair Five Ace Nine Six          -> 2121
          OnePair Five Ace Nine Seven        -> 2122
          OnePair Five Ace Nine Eight        -> 2123
          OnePair Five Ace Ten Two           -> 2124
          OnePair Five Ace Ten Three         -> 2125
          OnePair Five Ace Ten Four          -> 2126
          OnePair Five Ace Ten Six           -> 2127
          OnePair Five Ace Ten Seven         -> 2128
          OnePair Five Ace Ten Eight         -> 2129
          OnePair Five Ace Ten Nine          -> 2130
          OnePair Five Ace Jack Two          -> 2131
          OnePair Five Ace Jack Three        -> 2132
          OnePair Five Ace Jack Four         -> 2133
          OnePair Five Ace Jack Six          -> 2134
          OnePair Five Ace Jack Seven        -> 2135
          OnePair Five Ace Jack Eight        -> 2136
          OnePair Five Ace Jack Nine         -> 2137
          OnePair Five Ace Jack Ten          -> 2138
          OnePair Five Ace Queen Two         -> 2139
          OnePair Five Ace Queen Three       -> 2140
          OnePair Five Ace Queen Four        -> 2141
          OnePair Five Ace Queen Six         -> 2142
          OnePair Five Ace Queen Seven       -> 2143
          OnePair Five Ace Queen Eight       -> 2144
          OnePair Five Ace Queen Nine        -> 2145
          OnePair Five Ace Queen Ten         -> 2146
          OnePair Five Ace Queen Jack        -> 2147
          OnePair Five Ace King Two          -> 2148
          OnePair Five Ace King Three        -> 2149
          OnePair Five Ace King Four         -> 2150
          OnePair Five Ace King Six          -> 2151
          OnePair Five Ace King Seven        -> 2152
          OnePair Five Ace King Eight        -> 2153
          OnePair Five Ace King Nine         -> 2154
          OnePair Five Ace King Ten          -> 2155
          OnePair Five Ace King Jack         -> 2156
          OnePair Five Ace King Queen        -> 2157
          OnePair Six Four Three Two         -> 2158
          OnePair Six Five Three Two         -> 2159
          OnePair Six Five Four Two          -> 2160
          OnePair Six Five Four Three        -> 2161
          OnePair Six Seven Three Two        -> 2162
          OnePair Six Seven Four Two         -> 2163
          OnePair Six Seven Four Three       -> 2164
          OnePair Six Seven Five Two         -> 2165
          OnePair Six Seven Five Three       -> 2166
          OnePair Six Seven Five Four        -> 2167
          OnePair Six Eight Three Two        -> 2168
          OnePair Six Eight Four Two         -> 2169
          OnePair Six Eight Four Three       -> 2170
          OnePair Six Eight Five Two         -> 2171
          OnePair Six Eight Five Three       -> 2172
          OnePair Six Eight Five Four        -> 2173
          OnePair Six Eight Seven Two        -> 2174
          OnePair Six Eight Seven Three      -> 2175
          OnePair Six Eight Seven Four       -> 2176
          OnePair Six Eight Seven Five       -> 2177
          OnePair Six Nine Three Two         -> 2178
          OnePair Six Nine Four Two          -> 2179
          OnePair Six Nine Four Three        -> 2180
          OnePair Six Nine Five Two          -> 2181
          OnePair Six Nine Five Three        -> 2182
          OnePair Six Nine Five Four         -> 2183
          OnePair Six Nine Seven Two         -> 2184
          OnePair Six Nine Seven Three       -> 2185
          OnePair Six Nine Seven Four        -> 2186
          OnePair Six Nine Seven Five        -> 2187
          OnePair Six Nine Eight Two         -> 2188
          OnePair Six Nine Eight Three       -> 2189
          OnePair Six Nine Eight Four        -> 2190
          OnePair Six Nine Eight Five        -> 2191
          OnePair Six Nine Eight Seven       -> 2192
          OnePair Six Ten Three Two          -> 2193
          OnePair Six Ten Four Two           -> 2194
          OnePair Six Ten Four Three         -> 2195
          OnePair Six Ten Five Two           -> 2196
          OnePair Six Ten Five Three         -> 2197
          OnePair Six Ten Five Four          -> 2198
          OnePair Six Ten Seven Two          -> 2199
          OnePair Six Ten Seven Three        -> 2200
          OnePair Six Ten Seven Four         -> 2201
          OnePair Six Ten Seven Five         -> 2202
          OnePair Six Ten Eight Two          -> 2203
          OnePair Six Ten Eight Three        -> 2204
          OnePair Six Ten Eight Four         -> 2205
          OnePair Six Ten Eight Five         -> 2206
          OnePair Six Ten Eight Seven        -> 2207
          OnePair Six Ten Nine Two           -> 2208
          OnePair Six Ten Nine Three         -> 2209
          OnePair Six Ten Nine Four          -> 2210
          OnePair Six Ten Nine Five          -> 2211
          OnePair Six Ten Nine Seven         -> 2212
          OnePair Six Ten Nine Eight         -> 2213
          OnePair Six Jack Three Two         -> 2214
          OnePair Six Jack Four Two          -> 2215
          OnePair Six Jack Four Three        -> 2216
          OnePair Six Jack Five Two          -> 2217
          OnePair Six Jack Five Three        -> 2218
          OnePair Six Jack Five Four         -> 2219
          OnePair Six Jack Seven Two         -> 2220
          OnePair Six Jack Seven Three       -> 2221
          OnePair Six Jack Seven Four        -> 2222
          OnePair Six Jack Seven Five        -> 2223
          OnePair Six Jack Eight Two         -> 2224
          OnePair Six Jack Eight Three       -> 2225
          OnePair Six Jack Eight Four        -> 2226
          OnePair Six Jack Eight Five        -> 2227
          OnePair Six Jack Eight Seven       -> 2228
          OnePair Six Jack Nine Two          -> 2229
          OnePair Six Jack Nine Three        -> 2230
          OnePair Six Jack Nine Four         -> 2231
          OnePair Six Jack Nine Five         -> 2232
          OnePair Six Jack Nine Seven        -> 2233
          OnePair Six Jack Nine Eight        -> 2234
          OnePair Six Jack Ten Two           -> 2235
          OnePair Six Jack Ten Three         -> 2236
          OnePair Six Jack Ten Four          -> 2237
          OnePair Six Jack Ten Five          -> 2238
          OnePair Six Jack Ten Seven         -> 2239
          OnePair Six Jack Ten Eight         -> 2240
          OnePair Six Jack Ten Nine          -> 2241
          OnePair Six Queen Three Two        -> 2242
          OnePair Six Queen Four Two         -> 2243
          OnePair Six Queen Four Three       -> 2244
          OnePair Six Queen Five Two         -> 2245
          OnePair Six Queen Five Three       -> 2246
          OnePair Six Queen Five Four        -> 2247
          OnePair Six Queen Seven Two        -> 2248
          OnePair Six Queen Seven Three      -> 2249
          OnePair Six Queen Seven Four       -> 2250
          OnePair Six Queen Seven Five       -> 2251
          OnePair Six Queen Eight Two        -> 2252
          OnePair Six Queen Eight Three      -> 2253
          OnePair Six Queen Eight Four       -> 2254
          OnePair Six Queen Eight Five       -> 2255
          OnePair Six Queen Eight Seven      -> 2256
          OnePair Six Queen Nine Two         -> 2257
          OnePair Six Queen Nine Three       -> 2258
          OnePair Six Queen Nine Four        -> 2259
          OnePair Six Queen Nine Five        -> 2260
          OnePair Six Queen Nine Seven       -> 2261
          OnePair Six Queen Nine Eight       -> 2262
          OnePair Six Queen Ten Two          -> 2263
          OnePair Six Queen Ten Three        -> 2264
          OnePair Six Queen Ten Four         -> 2265
          OnePair Six Queen Ten Five         -> 2266
          OnePair Six Queen Ten Seven        -> 2267
          OnePair Six Queen Ten Eight        -> 2268
          OnePair Six Queen Ten Nine         -> 2269
          OnePair Six Queen Jack Two         -> 2270
          OnePair Six Queen Jack Three       -> 2271
          OnePair Six Queen Jack Four        -> 2272
          OnePair Six Queen Jack Five        -> 2273
          OnePair Six Queen Jack Seven       -> 2274
          OnePair Six Queen Jack Eight       -> 2275
          OnePair Six Queen Jack Nine        -> 2276
          OnePair Six Queen Jack Ten         -> 2277
          OnePair Six King Three Two         -> 2278
          OnePair Six King Four Two          -> 2279
          OnePair Six King Four Three        -> 2280
          OnePair Six King Five Two          -> 2281
          OnePair Six King Five Three        -> 2282
          OnePair Six King Five Four         -> 2283
          OnePair Six King Seven Two         -> 2284
          OnePair Six King Seven Three       -> 2285
          OnePair Six King Seven Four        -> 2286
          OnePair Six King Seven Five        -> 2287
          OnePair Six King Eight Two         -> 2288
          OnePair Six King Eight Three       -> 2289
          OnePair Six King Eight Four        -> 2290
          OnePair Six King Eight Five        -> 2291
          OnePair Six King Eight Seven       -> 2292
          OnePair Six King Nine Two          -> 2293
          OnePair Six King Nine Three        -> 2294
          OnePair Six King Nine Four         -> 2295
          OnePair Six King Nine Five         -> 2296
          OnePair Six King Nine Seven        -> 2297
          OnePair Six King Nine Eight        -> 2298
          OnePair Six King Ten Two           -> 2299
          OnePair Six King Ten Three         -> 2300
          OnePair Six King Ten Four          -> 2301
          OnePair Six King Ten Five          -> 2302
          OnePair Six King Ten Seven         -> 2303
          OnePair Six King Ten Eight         -> 2304
          OnePair Six King Ten Nine          -> 2305
          OnePair Six King Jack Two          -> 2306
          OnePair Six King Jack Three        -> 2307
          OnePair Six King Jack Four         -> 2308
          OnePair Six King Jack Five         -> 2309
          OnePair Six King Jack Seven        -> 2310
          OnePair Six King Jack Eight        -> 2311
          OnePair Six King Jack Nine         -> 2312
          OnePair Six King Jack Ten          -> 2313
          OnePair Six King Queen Two         -> 2314
          OnePair Six King Queen Three       -> 2315
          OnePair Six King Queen Four        -> 2316
          OnePair Six King Queen Five        -> 2317
          OnePair Six King Queen Seven       -> 2318
          OnePair Six King Queen Eight       -> 2319
          OnePair Six King Queen Nine        -> 2320
          OnePair Six King Queen Ten         -> 2321
          OnePair Six King Queen Jack        -> 2322
          OnePair Six Ace Three Two          -> 2323
          OnePair Six Ace Four Two           -> 2324
          OnePair Six Ace Four Three         -> 2325
          OnePair Six Ace Five Two           -> 2326
          OnePair Six Ace Five Three         -> 2327
          OnePair Six Ace Five Four          -> 2328
          OnePair Six Ace Seven Two          -> 2329
          OnePair Six Ace Seven Three        -> 2330
          OnePair Six Ace Seven Four         -> 2331
          OnePair Six Ace Seven Five         -> 2332
          OnePair Six Ace Eight Two          -> 2333
          OnePair Six Ace Eight Three        -> 2334
          OnePair Six Ace Eight Four         -> 2335
          OnePair Six Ace Eight Five         -> 2336
          OnePair Six Ace Eight Seven        -> 2337
          OnePair Six Ace Nine Two           -> 2338
          OnePair Six Ace Nine Three         -> 2339
          OnePair Six Ace Nine Four          -> 2340
          OnePair Six Ace Nine Five          -> 2341
          OnePair Six Ace Nine Seven         -> 2342
          OnePair Six Ace Nine Eight         -> 2343
          OnePair Six Ace Ten Two            -> 2344
          OnePair Six Ace Ten Three          -> 2345
          OnePair Six Ace Ten Four           -> 2346
          OnePair Six Ace Ten Five           -> 2347
          OnePair Six Ace Ten Seven          -> 2348
          OnePair Six Ace Ten Eight          -> 2349
          OnePair Six Ace Ten Nine           -> 2350
          OnePair Six Ace Jack Two           -> 2351
          OnePair Six Ace Jack Three         -> 2352
          OnePair Six Ace Jack Four          -> 2353
          OnePair Six Ace Jack Five          -> 2354
          OnePair Six Ace Jack Seven         -> 2355
          OnePair Six Ace Jack Eight         -> 2356
          OnePair Six Ace Jack Nine          -> 2357
          OnePair Six Ace Jack Ten           -> 2358
          OnePair Six Ace Queen Two          -> 2359
          OnePair Six Ace Queen Three        -> 2360
          OnePair Six Ace Queen Four         -> 2361
          OnePair Six Ace Queen Five         -> 2362
          OnePair Six Ace Queen Seven        -> 2363
          OnePair Six Ace Queen Eight        -> 2364
          OnePair Six Ace Queen Nine         -> 2365
          OnePair Six Ace Queen Ten          -> 2366
          OnePair Six Ace Queen Jack         -> 2367
          OnePair Six Ace King Two           -> 2368
          OnePair Six Ace King Three         -> 2369
          OnePair Six Ace King Four          -> 2370
          OnePair Six Ace King Five          -> 2371
          OnePair Six Ace King Seven         -> 2372
          OnePair Six Ace King Eight         -> 2373
          OnePair Six Ace King Nine          -> 2374
          OnePair Six Ace King Ten           -> 2375
          OnePair Six Ace King Jack          -> 2376
          OnePair Six Ace King Queen         -> 2377
          OnePair Seven Four Three Two       -> 2378
          OnePair Seven Five Three Two       -> 2379
          OnePair Seven Five Four Two        -> 2380
          OnePair Seven Five Four Three      -> 2381
          OnePair Seven Six Three Two        -> 2382
          OnePair Seven Six Four Two         -> 2383
          OnePair Seven Six Four Three       -> 2384
          OnePair Seven Six Five Two         -> 2385
          OnePair Seven Six Five Three       -> 2386
          OnePair Seven Six Five Four        -> 2387
          OnePair Seven Eight Three Two      -> 2388
          OnePair Seven Eight Four Two       -> 2389
          OnePair Seven Eight Four Three     -> 2390
          OnePair Seven Eight Five Two       -> 2391
          OnePair Seven Eight Five Three     -> 2392
          OnePair Seven Eight Five Four      -> 2393
          OnePair Seven Eight Six Two        -> 2394
          OnePair Seven Eight Six Three      -> 2395
          OnePair Seven Eight Six Four       -> 2396
          OnePair Seven Eight Six Five       -> 2397
          OnePair Seven Nine Three Two       -> 2398
          OnePair Seven Nine Four Two        -> 2399
          OnePair Seven Nine Four Three      -> 2400
          OnePair Seven Nine Five Two        -> 2401
          OnePair Seven Nine Five Three      -> 2402
          OnePair Seven Nine Five Four       -> 2403
          OnePair Seven Nine Six Two         -> 2404
          OnePair Seven Nine Six Three       -> 2405
          OnePair Seven Nine Six Four        -> 2406
          OnePair Seven Nine Six Five        -> 2407
          OnePair Seven Nine Eight Two       -> 2408
          OnePair Seven Nine Eight Three     -> 2409
          OnePair Seven Nine Eight Four      -> 2410
          OnePair Seven Nine Eight Five      -> 2411
          OnePair Seven Nine Eight Six       -> 2412
          OnePair Seven Ten Three Two        -> 2413
          OnePair Seven Ten Four Two         -> 2414
          OnePair Seven Ten Four Three       -> 2415
          OnePair Seven Ten Five Two         -> 2416
          OnePair Seven Ten Five Three       -> 2417
          OnePair Seven Ten Five Four        -> 2418
          OnePair Seven Ten Six Two          -> 2419
          OnePair Seven Ten Six Three        -> 2420
          OnePair Seven Ten Six Four         -> 2421
          OnePair Seven Ten Six Five         -> 2422
          OnePair Seven Ten Eight Two        -> 2423
          OnePair Seven Ten Eight Three      -> 2424
          OnePair Seven Ten Eight Four       -> 2425
          OnePair Seven Ten Eight Five       -> 2426
          OnePair Seven Ten Eight Six        -> 2427
          OnePair Seven Ten Nine Two         -> 2428
          OnePair Seven Ten Nine Three       -> 2429
          OnePair Seven Ten Nine Four        -> 2430
          OnePair Seven Ten Nine Five        -> 2431
          OnePair Seven Ten Nine Six         -> 2432
          OnePair Seven Ten Nine Eight       -> 2433
          OnePair Seven Jack Three Two       -> 2434
          OnePair Seven Jack Four Two        -> 2435
          OnePair Seven Jack Four Three      -> 2436
          OnePair Seven Jack Five Two        -> 2437
          OnePair Seven Jack Five Three      -> 2438
          OnePair Seven Jack Five Four       -> 2439
          OnePair Seven Jack Six Two         -> 2440
          OnePair Seven Jack Six Three       -> 2441
          OnePair Seven Jack Six Four        -> 2442
          OnePair Seven Jack Six Five        -> 2443
          OnePair Seven Jack Eight Two       -> 2444
          OnePair Seven Jack Eight Three     -> 2445
          OnePair Seven Jack Eight Four      -> 2446
          OnePair Seven Jack Eight Five      -> 2447
          OnePair Seven Jack Eight Six       -> 2448
          OnePair Seven Jack Nine Two        -> 2449
          OnePair Seven Jack Nine Three      -> 2450
          OnePair Seven Jack Nine Four       -> 2451
          OnePair Seven Jack Nine Five       -> 2452
          OnePair Seven Jack Nine Six        -> 2453
          OnePair Seven Jack Nine Eight      -> 2454
          OnePair Seven Jack Ten Two         -> 2455
          OnePair Seven Jack Ten Three       -> 2456
          OnePair Seven Jack Ten Four        -> 2457
          OnePair Seven Jack Ten Five        -> 2458
          OnePair Seven Jack Ten Six         -> 2459
          OnePair Seven Jack Ten Eight       -> 2460
          OnePair Seven Jack Ten Nine        -> 2461
          OnePair Seven Queen Three Two      -> 2462
          OnePair Seven Queen Four Two       -> 2463
          OnePair Seven Queen Four Three     -> 2464
          OnePair Seven Queen Five Two       -> 2465
          OnePair Seven Queen Five Three     -> 2466
          OnePair Seven Queen Five Four      -> 2467
          OnePair Seven Queen Six Two        -> 2468
          OnePair Seven Queen Six Three      -> 2469
          OnePair Seven Queen Six Four       -> 2470
          OnePair Seven Queen Six Five       -> 2471
          OnePair Seven Queen Eight Two      -> 2472
          OnePair Seven Queen Eight Three    -> 2473
          OnePair Seven Queen Eight Four     -> 2474
          OnePair Seven Queen Eight Five     -> 2475
          OnePair Seven Queen Eight Six      -> 2476
          OnePair Seven Queen Nine Two       -> 2477
          OnePair Seven Queen Nine Three     -> 2478
          OnePair Seven Queen Nine Four      -> 2479
          OnePair Seven Queen Nine Five      -> 2480
          OnePair Seven Queen Nine Six       -> 2481
          OnePair Seven Queen Nine Eight     -> 2482
          OnePair Seven Queen Ten Two        -> 2483
          OnePair Seven Queen Ten Three      -> 2484
          OnePair Seven Queen Ten Four       -> 2485
          OnePair Seven Queen Ten Five       -> 2486
          OnePair Seven Queen Ten Six        -> 2487
          OnePair Seven Queen Ten Eight      -> 2488
          OnePair Seven Queen Ten Nine       -> 2489
          OnePair Seven Queen Jack Two       -> 2490
          OnePair Seven Queen Jack Three     -> 2491
          OnePair Seven Queen Jack Four      -> 2492
          OnePair Seven Queen Jack Five      -> 2493
          OnePair Seven Queen Jack Six       -> 2494
          OnePair Seven Queen Jack Eight     -> 2495
          OnePair Seven Queen Jack Nine      -> 2496
          OnePair Seven Queen Jack Ten       -> 2497
          OnePair Seven King Three Two       -> 2498
          OnePair Seven King Four Two        -> 2499
          OnePair Seven King Four Three      -> 2500
          OnePair Seven King Five Two        -> 2501
          OnePair Seven King Five Three      -> 2502
          OnePair Seven King Five Four       -> 2503
          OnePair Seven King Six Two         -> 2504
          OnePair Seven King Six Three       -> 2505
          OnePair Seven King Six Four        -> 2506
          OnePair Seven King Six Five        -> 2507
          OnePair Seven King Eight Two       -> 2508
          OnePair Seven King Eight Three     -> 2509
          OnePair Seven King Eight Four      -> 2510
          OnePair Seven King Eight Five      -> 2511
          OnePair Seven King Eight Six       -> 2512
          OnePair Seven King Nine Two        -> 2513
          OnePair Seven King Nine Three      -> 2514
          OnePair Seven King Nine Four       -> 2515
          OnePair Seven King Nine Five       -> 2516
          OnePair Seven King Nine Six        -> 2517
          OnePair Seven King Nine Eight      -> 2518
          OnePair Seven King Ten Two         -> 2519
          OnePair Seven King Ten Three       -> 2520
          OnePair Seven King Ten Four        -> 2521
          OnePair Seven King Ten Five        -> 2522
          OnePair Seven King Ten Six         -> 2523
          OnePair Seven King Ten Eight       -> 2524
          OnePair Seven King Ten Nine        -> 2525
          OnePair Seven King Jack Two        -> 2526
          OnePair Seven King Jack Three      -> 2527
          OnePair Seven King Jack Four       -> 2528
          OnePair Seven King Jack Five       -> 2529
          OnePair Seven King Jack Six        -> 2530
          OnePair Seven King Jack Eight      -> 2531
          OnePair Seven King Jack Nine       -> 2532
          OnePair Seven King Jack Ten        -> 2533
          OnePair Seven King Queen Two       -> 2534
          OnePair Seven King Queen Three     -> 2535
          OnePair Seven King Queen Four      -> 2536
          OnePair Seven King Queen Five      -> 2537
          OnePair Seven King Queen Six       -> 2538
          OnePair Seven King Queen Eight     -> 2539
          OnePair Seven King Queen Nine      -> 2540
          OnePair Seven King Queen Ten       -> 2541
          OnePair Seven King Queen Jack      -> 2542
          OnePair Seven Ace Three Two        -> 2543
          OnePair Seven Ace Four Two         -> 2544
          OnePair Seven Ace Four Three       -> 2545
          OnePair Seven Ace Five Two         -> 2546
          OnePair Seven Ace Five Three       -> 2547
          OnePair Seven Ace Five Four        -> 2548
          OnePair Seven Ace Six Two          -> 2549
          OnePair Seven Ace Six Three        -> 2550
          OnePair Seven Ace Six Four         -> 2551
          OnePair Seven Ace Six Five         -> 2552
          OnePair Seven Ace Eight Two        -> 2553
          OnePair Seven Ace Eight Three      -> 2554
          OnePair Seven Ace Eight Four       -> 2555
          OnePair Seven Ace Eight Five       -> 2556
          OnePair Seven Ace Eight Six        -> 2557
          OnePair Seven Ace Nine Two         -> 2558
          OnePair Seven Ace Nine Three       -> 2559
          OnePair Seven Ace Nine Four        -> 2560
          OnePair Seven Ace Nine Five        -> 2561
          OnePair Seven Ace Nine Six         -> 2562
          OnePair Seven Ace Nine Eight       -> 2563
          OnePair Seven Ace Ten Two          -> 2564
          OnePair Seven Ace Ten Three        -> 2565
          OnePair Seven Ace Ten Four         -> 2566
          OnePair Seven Ace Ten Five         -> 2567
          OnePair Seven Ace Ten Six          -> 2568
          OnePair Seven Ace Ten Eight        -> 2569
          OnePair Seven Ace Ten Nine         -> 2570
          OnePair Seven Ace Jack Two         -> 2571
          OnePair Seven Ace Jack Three       -> 2572
          OnePair Seven Ace Jack Four        -> 2573
          OnePair Seven Ace Jack Five        -> 2574
          OnePair Seven Ace Jack Six         -> 2575
          OnePair Seven Ace Jack Eight       -> 2576
          OnePair Seven Ace Jack Nine        -> 2577
          OnePair Seven Ace Jack Ten         -> 2578
          OnePair Seven Ace Queen Two        -> 2579
          OnePair Seven Ace Queen Three      -> 2580
          OnePair Seven Ace Queen Four       -> 2581
          OnePair Seven Ace Queen Five       -> 2582
          OnePair Seven Ace Queen Six        -> 2583
          OnePair Seven Ace Queen Eight      -> 2584
          OnePair Seven Ace Queen Nine       -> 2585
          OnePair Seven Ace Queen Ten        -> 2586
          OnePair Seven Ace Queen Jack       -> 2587
          OnePair Seven Ace King Two         -> 2588
          OnePair Seven Ace King Three       -> 2589
          OnePair Seven Ace King Four        -> 2590
          OnePair Seven Ace King Five        -> 2591
          OnePair Seven Ace King Six         -> 2592
          OnePair Seven Ace King Eight       -> 2593
          OnePair Seven Ace King Nine        -> 2594
          OnePair Seven Ace King Ten         -> 2595
          OnePair Seven Ace King Jack        -> 2596
          OnePair Seven Ace King Queen       -> 2597
          OnePair Eight Four Three Two       -> 2598
          OnePair Eight Five Three Two       -> 2599
          OnePair Eight Five Four Two        -> 2600
          OnePair Eight Five Four Three      -> 2601
          OnePair Eight Six Three Two        -> 2602
          OnePair Eight Six Four Two         -> 2603
          OnePair Eight Six Four Three       -> 2604
          OnePair Eight Six Five Two         -> 2605
          OnePair Eight Six Five Three       -> 2606
          OnePair Eight Six Five Four        -> 2607
          OnePair Eight Seven Three Two      -> 2608
          OnePair Eight Seven Four Two       -> 2609
          OnePair Eight Seven Four Three     -> 2610
          OnePair Eight Seven Five Two       -> 2611
          OnePair Eight Seven Five Three     -> 2612
          OnePair Eight Seven Five Four      -> 2613
          OnePair Eight Seven Six Two        -> 2614
          OnePair Eight Seven Six Three      -> 2615
          OnePair Eight Seven Six Four       -> 2616
          OnePair Eight Seven Six Five       -> 2617
          OnePair Eight Nine Three Two       -> 2618
          OnePair Eight Nine Four Two        -> 2619
          OnePair Eight Nine Four Three      -> 2620
          OnePair Eight Nine Five Two        -> 2621
          OnePair Eight Nine Five Three      -> 2622
          OnePair Eight Nine Five Four       -> 2623
          OnePair Eight Nine Six Two         -> 2624
          OnePair Eight Nine Six Three       -> 2625
          OnePair Eight Nine Six Four        -> 2626
          OnePair Eight Nine Six Five        -> 2627
          OnePair Eight Nine Seven Two       -> 2628
          OnePair Eight Nine Seven Three     -> 2629
          OnePair Eight Nine Seven Four      -> 2630
          OnePair Eight Nine Seven Five      -> 2631
          OnePair Eight Nine Seven Six       -> 2632
          OnePair Eight Ten Three Two        -> 2633
          OnePair Eight Ten Four Two         -> 2634
          OnePair Eight Ten Four Three       -> 2635
          OnePair Eight Ten Five Two         -> 2636
          OnePair Eight Ten Five Three       -> 2637
          OnePair Eight Ten Five Four        -> 2638
          OnePair Eight Ten Six Two          -> 2639
          OnePair Eight Ten Six Three        -> 2640
          OnePair Eight Ten Six Four         -> 2641
          OnePair Eight Ten Six Five         -> 2642
          OnePair Eight Ten Seven Two        -> 2643
          OnePair Eight Ten Seven Three      -> 2644
          OnePair Eight Ten Seven Four       -> 2645
          OnePair Eight Ten Seven Five       -> 2646
          OnePair Eight Ten Seven Six        -> 2647
          OnePair Eight Ten Nine Two         -> 2648
          OnePair Eight Ten Nine Three       -> 2649
          OnePair Eight Ten Nine Four        -> 2650
          OnePair Eight Ten Nine Five        -> 2651
          OnePair Eight Ten Nine Six         -> 2652
          OnePair Eight Ten Nine Seven       -> 2653
          OnePair Eight Jack Three Two       -> 2654
          OnePair Eight Jack Four Two        -> 2655
          OnePair Eight Jack Four Three      -> 2656
          OnePair Eight Jack Five Two        -> 2657
          OnePair Eight Jack Five Three      -> 2658
          OnePair Eight Jack Five Four       -> 2659
          OnePair Eight Jack Six Two         -> 2660
          OnePair Eight Jack Six Three       -> 2661
          OnePair Eight Jack Six Four        -> 2662
          OnePair Eight Jack Six Five        -> 2663
          OnePair Eight Jack Seven Two       -> 2664
          OnePair Eight Jack Seven Three     -> 2665
          OnePair Eight Jack Seven Four      -> 2666
          OnePair Eight Jack Seven Five      -> 2667
          OnePair Eight Jack Seven Six       -> 2668
          OnePair Eight Jack Nine Two        -> 2669
          OnePair Eight Jack Nine Three      -> 2670
          OnePair Eight Jack Nine Four       -> 2671
          OnePair Eight Jack Nine Five       -> 2672
          OnePair Eight Jack Nine Six        -> 2673
          OnePair Eight Jack Nine Seven      -> 2674
          OnePair Eight Jack Ten Two         -> 2675
          OnePair Eight Jack Ten Three       -> 2676
          OnePair Eight Jack Ten Four        -> 2677
          OnePair Eight Jack Ten Five        -> 2678
          OnePair Eight Jack Ten Six         -> 2679
          OnePair Eight Jack Ten Seven       -> 2680
          OnePair Eight Jack Ten Nine        -> 2681
          OnePair Eight Queen Three Two      -> 2682
          OnePair Eight Queen Four Two       -> 2683
          OnePair Eight Queen Four Three     -> 2684
          OnePair Eight Queen Five Two       -> 2685
          OnePair Eight Queen Five Three     -> 2686
          OnePair Eight Queen Five Four      -> 2687
          OnePair Eight Queen Six Two        -> 2688
          OnePair Eight Queen Six Three      -> 2689
          OnePair Eight Queen Six Four       -> 2690
          OnePair Eight Queen Six Five       -> 2691
          OnePair Eight Queen Seven Two      -> 2692
          OnePair Eight Queen Seven Three    -> 2693
          OnePair Eight Queen Seven Four     -> 2694
          OnePair Eight Queen Seven Five     -> 2695
          OnePair Eight Queen Seven Six      -> 2696
          OnePair Eight Queen Nine Two       -> 2697
          OnePair Eight Queen Nine Three     -> 2698
          OnePair Eight Queen Nine Four      -> 2699
          OnePair Eight Queen Nine Five      -> 2700
          OnePair Eight Queen Nine Six       -> 2701
          OnePair Eight Queen Nine Seven     -> 2702
          OnePair Eight Queen Ten Two        -> 2703
          OnePair Eight Queen Ten Three      -> 2704
          OnePair Eight Queen Ten Four       -> 2705
          OnePair Eight Queen Ten Five       -> 2706
          OnePair Eight Queen Ten Six        -> 2707
          OnePair Eight Queen Ten Seven      -> 2708
          OnePair Eight Queen Ten Nine       -> 2709
          OnePair Eight Queen Jack Two       -> 2710
          OnePair Eight Queen Jack Three     -> 2711
          OnePair Eight Queen Jack Four      -> 2712
          OnePair Eight Queen Jack Five      -> 2713
          OnePair Eight Queen Jack Six       -> 2714
          OnePair Eight Queen Jack Seven     -> 2715
          OnePair Eight Queen Jack Nine      -> 2716
          OnePair Eight Queen Jack Ten       -> 2717
          OnePair Eight King Three Two       -> 2718
          OnePair Eight King Four Two        -> 2719
          OnePair Eight King Four Three      -> 2720
          OnePair Eight King Five Two        -> 2721
          OnePair Eight King Five Three      -> 2722
          OnePair Eight King Five Four       -> 2723
          OnePair Eight King Six Two         -> 2724
          OnePair Eight King Six Three       -> 2725
          OnePair Eight King Six Four        -> 2726
          OnePair Eight King Six Five        -> 2727
          OnePair Eight King Seven Two       -> 2728
          OnePair Eight King Seven Three     -> 2729
          OnePair Eight King Seven Four      -> 2730
          OnePair Eight King Seven Five      -> 2731
          OnePair Eight King Seven Six       -> 2732
          OnePair Eight King Nine Two        -> 2733
          OnePair Eight King Nine Three      -> 2734
          OnePair Eight King Nine Four       -> 2735
          OnePair Eight King Nine Five       -> 2736
          OnePair Eight King Nine Six        -> 2737
          OnePair Eight King Nine Seven      -> 2738
          OnePair Eight King Ten Two         -> 2739
          OnePair Eight King Ten Three       -> 2740
          OnePair Eight King Ten Four        -> 2741
          OnePair Eight King Ten Five        -> 2742
          OnePair Eight King Ten Six         -> 2743
          OnePair Eight King Ten Seven       -> 2744
          OnePair Eight King Ten Nine        -> 2745
          OnePair Eight King Jack Two        -> 2746
          OnePair Eight King Jack Three      -> 2747
          OnePair Eight King Jack Four       -> 2748
          OnePair Eight King Jack Five       -> 2749
          OnePair Eight King Jack Six        -> 2750
          OnePair Eight King Jack Seven      -> 2751
          OnePair Eight King Jack Nine       -> 2752
          OnePair Eight King Jack Ten        -> 2753
          OnePair Eight King Queen Two       -> 2754
          OnePair Eight King Queen Three     -> 2755
          OnePair Eight King Queen Four      -> 2756
          OnePair Eight King Queen Five      -> 2757
          OnePair Eight King Queen Six       -> 2758
          OnePair Eight King Queen Seven     -> 2759
          OnePair Eight King Queen Nine      -> 2760
          OnePair Eight King Queen Ten       -> 2761
          OnePair Eight King Queen Jack      -> 2762
          OnePair Eight Ace Three Two        -> 2763
          OnePair Eight Ace Four Two         -> 2764
          OnePair Eight Ace Four Three       -> 2765
          OnePair Eight Ace Five Two         -> 2766
          OnePair Eight Ace Five Three       -> 2767
          OnePair Eight Ace Five Four        -> 2768
          OnePair Eight Ace Six Two          -> 2769
          OnePair Eight Ace Six Three        -> 2770
          OnePair Eight Ace Six Four         -> 2771
          OnePair Eight Ace Six Five         -> 2772
          OnePair Eight Ace Seven Two        -> 2773
          OnePair Eight Ace Seven Three      -> 2774
          OnePair Eight Ace Seven Four       -> 2775
          OnePair Eight Ace Seven Five       -> 2776
          OnePair Eight Ace Seven Six        -> 2777
          OnePair Eight Ace Nine Two         -> 2778
          OnePair Eight Ace Nine Three       -> 2779
          OnePair Eight Ace Nine Four        -> 2780
          OnePair Eight Ace Nine Five        -> 2781
          OnePair Eight Ace Nine Six         -> 2782
          OnePair Eight Ace Nine Seven       -> 2783
          OnePair Eight Ace Ten Two          -> 2784
          OnePair Eight Ace Ten Three        -> 2785
          OnePair Eight Ace Ten Four         -> 2786
          OnePair Eight Ace Ten Five         -> 2787
          OnePair Eight Ace Ten Six          -> 2788
          OnePair Eight Ace Ten Seven        -> 2789
          OnePair Eight Ace Ten Nine         -> 2790
          OnePair Eight Ace Jack Two         -> 2791
          OnePair Eight Ace Jack Three       -> 2792
          OnePair Eight Ace Jack Four        -> 2793
          OnePair Eight Ace Jack Five        -> 2794
          OnePair Eight Ace Jack Six         -> 2795
          OnePair Eight Ace Jack Seven       -> 2796
          OnePair Eight Ace Jack Nine        -> 2797
          OnePair Eight Ace Jack Ten         -> 2798
          OnePair Eight Ace Queen Two        -> 2799
          OnePair Eight Ace Queen Three      -> 2800
          OnePair Eight Ace Queen Four       -> 2801
          OnePair Eight Ace Queen Five       -> 2802
          OnePair Eight Ace Queen Six        -> 2803
          OnePair Eight Ace Queen Seven      -> 2804
          OnePair Eight Ace Queen Nine       -> 2805
          OnePair Eight Ace Queen Ten        -> 2806
          OnePair Eight Ace Queen Jack       -> 2807
          OnePair Eight Ace King Two         -> 2808
          OnePair Eight Ace King Three       -> 2809
          OnePair Eight Ace King Four        -> 2810
          OnePair Eight Ace King Five        -> 2811
          OnePair Eight Ace King Six         -> 2812
          OnePair Eight Ace King Seven       -> 2813
          OnePair Eight Ace King Nine        -> 2814
          OnePair Eight Ace King Ten         -> 2815
          OnePair Eight Ace King Jack        -> 2816
          OnePair Eight Ace King Queen       -> 2817
          OnePair Nine Four Three Two        -> 2818
          OnePair Nine Five Three Two        -> 2819
          OnePair Nine Five Four Two         -> 2820
          OnePair Nine Five Four Three       -> 2821
          OnePair Nine Six Three Two         -> 2822
          OnePair Nine Six Four Two          -> 2823
          OnePair Nine Six Four Three        -> 2824
          OnePair Nine Six Five Two          -> 2825
          OnePair Nine Six Five Three        -> 2826
          OnePair Nine Six Five Four         -> 2827
          OnePair Nine Seven Three Two       -> 2828
          OnePair Nine Seven Four Two        -> 2829
          OnePair Nine Seven Four Three      -> 2830
          OnePair Nine Seven Five Two        -> 2831
          OnePair Nine Seven Five Three      -> 2832
          OnePair Nine Seven Five Four       -> 2833
          OnePair Nine Seven Six Two         -> 2834
          OnePair Nine Seven Six Three       -> 2835
          OnePair Nine Seven Six Four        -> 2836
          OnePair Nine Seven Six Five        -> 2837
          OnePair Nine Eight Three Two       -> 2838
          OnePair Nine Eight Four Two        -> 2839
          OnePair Nine Eight Four Three      -> 2840
          OnePair Nine Eight Five Two        -> 2841
          OnePair Nine Eight Five Three      -> 2842
          OnePair Nine Eight Five Four       -> 2843
          OnePair Nine Eight Six Two         -> 2844
          OnePair Nine Eight Six Three       -> 2845
          OnePair Nine Eight Six Four        -> 2846
          OnePair Nine Eight Six Five        -> 2847
          OnePair Nine Eight Seven Two       -> 2848
          OnePair Nine Eight Seven Three     -> 2849
          OnePair Nine Eight Seven Four      -> 2850
          OnePair Nine Eight Seven Five      -> 2851
          OnePair Nine Eight Seven Six       -> 2852
          OnePair Nine Ten Three Two         -> 2853
          OnePair Nine Ten Four Two          -> 2854
          OnePair Nine Ten Four Three        -> 2855
          OnePair Nine Ten Five Two          -> 2856
          OnePair Nine Ten Five Three        -> 2857
          OnePair Nine Ten Five Four         -> 2858
          OnePair Nine Ten Six Two           -> 2859
          OnePair Nine Ten Six Three         -> 2860
          OnePair Nine Ten Six Four          -> 2861
          OnePair Nine Ten Six Five          -> 2862
          OnePair Nine Ten Seven Two         -> 2863
          OnePair Nine Ten Seven Three       -> 2864
          OnePair Nine Ten Seven Four        -> 2865
          OnePair Nine Ten Seven Five        -> 2866
          OnePair Nine Ten Seven Six         -> 2867
          OnePair Nine Ten Eight Two         -> 2868
          OnePair Nine Ten Eight Three       -> 2869
          OnePair Nine Ten Eight Four        -> 2870
          OnePair Nine Ten Eight Five        -> 2871
          OnePair Nine Ten Eight Six         -> 2872
          OnePair Nine Ten Eight Seven       -> 2873
          OnePair Nine Jack Three Two        -> 2874
          OnePair Nine Jack Four Two         -> 2875
          OnePair Nine Jack Four Three       -> 2876
          OnePair Nine Jack Five Two         -> 2877
          OnePair Nine Jack Five Three       -> 2878
          OnePair Nine Jack Five Four        -> 2879
          OnePair Nine Jack Six Two          -> 2880
          OnePair Nine Jack Six Three        -> 2881
          OnePair Nine Jack Six Four         -> 2882
          OnePair Nine Jack Six Five         -> 2883
          OnePair Nine Jack Seven Two        -> 2884
          OnePair Nine Jack Seven Three      -> 2885
          OnePair Nine Jack Seven Four       -> 2886
          OnePair Nine Jack Seven Five       -> 2887
          OnePair Nine Jack Seven Six        -> 2888
          OnePair Nine Jack Eight Two        -> 2889
          OnePair Nine Jack Eight Three      -> 2890
          OnePair Nine Jack Eight Four       -> 2891
          OnePair Nine Jack Eight Five       -> 2892
          OnePair Nine Jack Eight Six        -> 2893
          OnePair Nine Jack Eight Seven      -> 2894
          OnePair Nine Jack Ten Two          -> 2895
          OnePair Nine Jack Ten Three        -> 2896
          OnePair Nine Jack Ten Four         -> 2897
          OnePair Nine Jack Ten Five         -> 2898
          OnePair Nine Jack Ten Six          -> 2899
          OnePair Nine Jack Ten Seven        -> 2900
          OnePair Nine Jack Ten Eight        -> 2901
          OnePair Nine Queen Three Two       -> 2902
          OnePair Nine Queen Four Two        -> 2903
          OnePair Nine Queen Four Three      -> 2904
          OnePair Nine Queen Five Two        -> 2905
          OnePair Nine Queen Five Three      -> 2906
          OnePair Nine Queen Five Four       -> 2907
          OnePair Nine Queen Six Two         -> 2908
          OnePair Nine Queen Six Three       -> 2909
          OnePair Nine Queen Six Four        -> 2910
          OnePair Nine Queen Six Five        -> 2911
          OnePair Nine Queen Seven Two       -> 2912
          OnePair Nine Queen Seven Three     -> 2913
          OnePair Nine Queen Seven Four      -> 2914
          OnePair Nine Queen Seven Five      -> 2915
          OnePair Nine Queen Seven Six       -> 2916
          OnePair Nine Queen Eight Two       -> 2917
          OnePair Nine Queen Eight Three     -> 2918
          OnePair Nine Queen Eight Four      -> 2919
          OnePair Nine Queen Eight Five      -> 2920
          OnePair Nine Queen Eight Six       -> 2921
          OnePair Nine Queen Eight Seven     -> 2922
          OnePair Nine Queen Ten Two         -> 2923
          OnePair Nine Queen Ten Three       -> 2924
          OnePair Nine Queen Ten Four        -> 2925
          OnePair Nine Queen Ten Five        -> 2926
          OnePair Nine Queen Ten Six         -> 2927
          OnePair Nine Queen Ten Seven       -> 2928
          OnePair Nine Queen Ten Eight       -> 2929
          OnePair Nine Queen Jack Two        -> 2930
          OnePair Nine Queen Jack Three      -> 2931
          OnePair Nine Queen Jack Four       -> 2932
          OnePair Nine Queen Jack Five       -> 2933
          OnePair Nine Queen Jack Six        -> 2934
          OnePair Nine Queen Jack Seven      -> 2935
          OnePair Nine Queen Jack Eight      -> 2936
          OnePair Nine Queen Jack Ten        -> 2937
          OnePair Nine King Three Two        -> 2938
          OnePair Nine King Four Two         -> 2939
          OnePair Nine King Four Three       -> 2940
          OnePair Nine King Five Two         -> 2941
          OnePair Nine King Five Three       -> 2942
          OnePair Nine King Five Four        -> 2943
          OnePair Nine King Six Two          -> 2944
          OnePair Nine King Six Three        -> 2945
          OnePair Nine King Six Four         -> 2946
          OnePair Nine King Six Five         -> 2947
          OnePair Nine King Seven Two        -> 2948
          OnePair Nine King Seven Three      -> 2949
          OnePair Nine King Seven Four       -> 2950
          OnePair Nine King Seven Five       -> 2951
          OnePair Nine King Seven Six        -> 2952
          OnePair Nine King Eight Two        -> 2953
          OnePair Nine King Eight Three      -> 2954
          OnePair Nine King Eight Four       -> 2955
          OnePair Nine King Eight Five       -> 2956
          OnePair Nine King Eight Six        -> 2957
          OnePair Nine King Eight Seven      -> 2958
          OnePair Nine King Ten Two          -> 2959
          OnePair Nine King Ten Three        -> 2960
          OnePair Nine King Ten Four         -> 2961
          OnePair Nine King Ten Five         -> 2962
          OnePair Nine King Ten Six          -> 2963
          OnePair Nine King Ten Seven        -> 2964
          OnePair Nine King Ten Eight        -> 2965
          OnePair Nine King Jack Two         -> 2966
          OnePair Nine King Jack Three       -> 2967
          OnePair Nine King Jack Four        -> 2968
          OnePair Nine King Jack Five        -> 2969
          OnePair Nine King Jack Six         -> 2970
          OnePair Nine King Jack Seven       -> 2971
          OnePair Nine King Jack Eight       -> 2972
          OnePair Nine King Jack Ten         -> 2973
          OnePair Nine King Queen Two        -> 2974
          OnePair Nine King Queen Three      -> 2975
          OnePair Nine King Queen Four       -> 2976
          OnePair Nine King Queen Five       -> 2977
          OnePair Nine King Queen Six        -> 2978
          OnePair Nine King Queen Seven      -> 2979
          OnePair Nine King Queen Eight      -> 2980
          OnePair Nine King Queen Ten        -> 2981
          OnePair Nine King Queen Jack       -> 2982
          OnePair Nine Ace Three Two         -> 2983
          OnePair Nine Ace Four Two          -> 2984
          OnePair Nine Ace Four Three        -> 2985
          OnePair Nine Ace Five Two          -> 2986
          OnePair Nine Ace Five Three        -> 2987
          OnePair Nine Ace Five Four         -> 2988
          OnePair Nine Ace Six Two           -> 2989
          OnePair Nine Ace Six Three         -> 2990
          OnePair Nine Ace Six Four          -> 2991
          OnePair Nine Ace Six Five          -> 2992
          OnePair Nine Ace Seven Two         -> 2993
          OnePair Nine Ace Seven Three       -> 2994
          OnePair Nine Ace Seven Four        -> 2995
          OnePair Nine Ace Seven Five        -> 2996
          OnePair Nine Ace Seven Six         -> 2997
          OnePair Nine Ace Eight Two         -> 2998
          OnePair Nine Ace Eight Three       -> 2999
          OnePair Nine Ace Eight Four        -> 3000
          OnePair Nine Ace Eight Five        -> 3001
          OnePair Nine Ace Eight Six         -> 3002
          OnePair Nine Ace Eight Seven       -> 3003
          OnePair Nine Ace Ten Two           -> 3004
          OnePair Nine Ace Ten Three         -> 3005
          OnePair Nine Ace Ten Four          -> 3006
          OnePair Nine Ace Ten Five          -> 3007
          OnePair Nine Ace Ten Six           -> 3008
          OnePair Nine Ace Ten Seven         -> 3009
          OnePair Nine Ace Ten Eight         -> 3010
          OnePair Nine Ace Jack Two          -> 3011
          OnePair Nine Ace Jack Three        -> 3012
          OnePair Nine Ace Jack Four         -> 3013
          OnePair Nine Ace Jack Five         -> 3014
          OnePair Nine Ace Jack Six          -> 3015
          OnePair Nine Ace Jack Seven        -> 3016
          OnePair Nine Ace Jack Eight        -> 3017
          OnePair Nine Ace Jack Ten          -> 3018
          OnePair Nine Ace Queen Two         -> 3019
          OnePair Nine Ace Queen Three       -> 3020
          OnePair Nine Ace Queen Four        -> 3021
          OnePair Nine Ace Queen Five        -> 3022
          OnePair Nine Ace Queen Six         -> 3023
          OnePair Nine Ace Queen Seven       -> 3024
          OnePair Nine Ace Queen Eight       -> 3025
          OnePair Nine Ace Queen Ten         -> 3026
          OnePair Nine Ace Queen Jack        -> 3027
          OnePair Nine Ace King Two          -> 3028
          OnePair Nine Ace King Three        -> 3029
          OnePair Nine Ace King Four         -> 3030
          OnePair Nine Ace King Five         -> 3031
          OnePair Nine Ace King Six          -> 3032
          OnePair Nine Ace King Seven        -> 3033
          OnePair Nine Ace King Eight        -> 3034
          OnePair Nine Ace King Ten          -> 3035
          OnePair Nine Ace King Jack         -> 3036
          OnePair Nine Ace King Queen        -> 3037
          OnePair Ten Four Three Two         -> 3038
          OnePair Ten Five Three Two         -> 3039
          OnePair Ten Five Four Two          -> 3040
          OnePair Ten Five Four Three        -> 3041
          OnePair Ten Six Three Two          -> 3042
          OnePair Ten Six Four Two           -> 3043
          OnePair Ten Six Four Three         -> 3044
          OnePair Ten Six Five Two           -> 3045
          OnePair Ten Six Five Three         -> 3046
          OnePair Ten Six Five Four          -> 3047
          OnePair Ten Seven Three Two        -> 3048
          OnePair Ten Seven Four Two         -> 3049
          OnePair Ten Seven Four Three       -> 3050
          OnePair Ten Seven Five Two         -> 3051
          OnePair Ten Seven Five Three       -> 3052
          OnePair Ten Seven Five Four        -> 3053
          OnePair Ten Seven Six Two          -> 3054
          OnePair Ten Seven Six Three        -> 3055
          OnePair Ten Seven Six Four         -> 3056
          OnePair Ten Seven Six Five         -> 3057
          OnePair Ten Eight Three Two        -> 3058
          OnePair Ten Eight Four Two         -> 3059
          OnePair Ten Eight Four Three       -> 3060
          OnePair Ten Eight Five Two         -> 3061
          OnePair Ten Eight Five Three       -> 3062
          OnePair Ten Eight Five Four        -> 3063
          OnePair Ten Eight Six Two          -> 3064
          OnePair Ten Eight Six Three        -> 3065
          OnePair Ten Eight Six Four         -> 3066
          OnePair Ten Eight Six Five         -> 3067
          OnePair Ten Eight Seven Two        -> 3068
          OnePair Ten Eight Seven Three      -> 3069
          OnePair Ten Eight Seven Four       -> 3070
          OnePair Ten Eight Seven Five       -> 3071
          OnePair Ten Eight Seven Six        -> 3072
          OnePair Ten Nine Three Two         -> 3073
          OnePair Ten Nine Four Two          -> 3074
          OnePair Ten Nine Four Three        -> 3075
          OnePair Ten Nine Five Two          -> 3076
          OnePair Ten Nine Five Three        -> 3077
          OnePair Ten Nine Five Four         -> 3078
          OnePair Ten Nine Six Two           -> 3079
          OnePair Ten Nine Six Three         -> 3080
          OnePair Ten Nine Six Four          -> 3081
          OnePair Ten Nine Six Five          -> 3082
          OnePair Ten Nine Seven Two         -> 3083
          OnePair Ten Nine Seven Three       -> 3084
          OnePair Ten Nine Seven Four        -> 3085
          OnePair Ten Nine Seven Five        -> 3086
          OnePair Ten Nine Seven Six         -> 3087
          OnePair Ten Nine Eight Two         -> 3088
          OnePair Ten Nine Eight Three       -> 3089
          OnePair Ten Nine Eight Four        -> 3090
          OnePair Ten Nine Eight Five        -> 3091
          OnePair Ten Nine Eight Six         -> 3092
          OnePair Ten Nine Eight Seven       -> 3093
          OnePair Ten Jack Three Two         -> 3094
          OnePair Ten Jack Four Two          -> 3095
          OnePair Ten Jack Four Three        -> 3096
          OnePair Ten Jack Five Two          -> 3097
          OnePair Ten Jack Five Three        -> 3098
          OnePair Ten Jack Five Four         -> 3099
          OnePair Ten Jack Six Two           -> 3100
          OnePair Ten Jack Six Three         -> 3101
          OnePair Ten Jack Six Four          -> 3102
          OnePair Ten Jack Six Five          -> 3103
          OnePair Ten Jack Seven Two         -> 3104
          OnePair Ten Jack Seven Three       -> 3105
          OnePair Ten Jack Seven Four        -> 3106
          OnePair Ten Jack Seven Five        -> 3107
          OnePair Ten Jack Seven Six         -> 3108
          OnePair Ten Jack Eight Two         -> 3109
          OnePair Ten Jack Eight Three       -> 3110
          OnePair Ten Jack Eight Four        -> 3111
          OnePair Ten Jack Eight Five        -> 3112
          OnePair Ten Jack Eight Six         -> 3113
          OnePair Ten Jack Eight Seven       -> 3114
          OnePair Ten Jack Nine Two          -> 3115
          OnePair Ten Jack Nine Three        -> 3116
          OnePair Ten Jack Nine Four         -> 3117
          OnePair Ten Jack Nine Five         -> 3118
          OnePair Ten Jack Nine Six          -> 3119
          OnePair Ten Jack Nine Seven        -> 3120
          OnePair Ten Jack Nine Eight        -> 3121
          OnePair Ten Queen Three Two        -> 3122
          OnePair Ten Queen Four Two         -> 3123
          OnePair Ten Queen Four Three       -> 3124
          OnePair Ten Queen Five Two         -> 3125
          OnePair Ten Queen Five Three       -> 3126
          OnePair Ten Queen Five Four        -> 3127
          OnePair Ten Queen Six Two          -> 3128
          OnePair Ten Queen Six Three        -> 3129
          OnePair Ten Queen Six Four         -> 3130
          OnePair Ten Queen Six Five         -> 3131
          OnePair Ten Queen Seven Two        -> 3132
          OnePair Ten Queen Seven Three      -> 3133
          OnePair Ten Queen Seven Four       -> 3134
          OnePair Ten Queen Seven Five       -> 3135
          OnePair Ten Queen Seven Six        -> 3136
          OnePair Ten Queen Eight Two        -> 3137
          OnePair Ten Queen Eight Three      -> 3138
          OnePair Ten Queen Eight Four       -> 3139
          OnePair Ten Queen Eight Five       -> 3140
          OnePair Ten Queen Eight Six        -> 3141
          OnePair Ten Queen Eight Seven      -> 3142
          OnePair Ten Queen Nine Two         -> 3143
          OnePair Ten Queen Nine Three       -> 3144
          OnePair Ten Queen Nine Four        -> 3145
          OnePair Ten Queen Nine Five        -> 3146
          OnePair Ten Queen Nine Six         -> 3147
          OnePair Ten Queen Nine Seven       -> 3148
          OnePair Ten Queen Nine Eight       -> 3149
          OnePair Ten Queen Jack Two         -> 3150
          OnePair Ten Queen Jack Three       -> 3151
          OnePair Ten Queen Jack Four        -> 3152
          OnePair Ten Queen Jack Five        -> 3153
          OnePair Ten Queen Jack Six         -> 3154
          OnePair Ten Queen Jack Seven       -> 3155
          OnePair Ten Queen Jack Eight       -> 3156
          OnePair Ten Queen Jack Nine        -> 3157
          OnePair Ten King Three Two         -> 3158
          OnePair Ten King Four Two          -> 3159
          OnePair Ten King Four Three        -> 3160
          OnePair Ten King Five Two          -> 3161
          OnePair Ten King Five Three        -> 3162
          OnePair Ten King Five Four         -> 3163
          OnePair Ten King Six Two           -> 3164
          OnePair Ten King Six Three         -> 3165
          OnePair Ten King Six Four          -> 3166
          OnePair Ten King Six Five          -> 3167
          OnePair Ten King Seven Two         -> 3168
          OnePair Ten King Seven Three       -> 3169
          OnePair Ten King Seven Four        -> 3170
          OnePair Ten King Seven Five        -> 3171
          OnePair Ten King Seven Six         -> 3172
          OnePair Ten King Eight Two         -> 3173
          OnePair Ten King Eight Three       -> 3174
          OnePair Ten King Eight Four        -> 3175
          OnePair Ten King Eight Five        -> 3176
          OnePair Ten King Eight Six         -> 3177
          OnePair Ten King Eight Seven       -> 3178
          OnePair Ten King Nine Two          -> 3179
          OnePair Ten King Nine Three        -> 3180
          OnePair Ten King Nine Four         -> 3181
          OnePair Ten King Nine Five         -> 3182
          OnePair Ten King Nine Six          -> 3183
          OnePair Ten King Nine Seven        -> 3184
          OnePair Ten King Nine Eight        -> 3185
          OnePair Ten King Jack Two          -> 3186
          OnePair Ten King Jack Three        -> 3187
          OnePair Ten King Jack Four         -> 3188
          OnePair Ten King Jack Five         -> 3189
          OnePair Ten King Jack Six          -> 3190
          OnePair Ten King Jack Seven        -> 3191
          OnePair Ten King Jack Eight        -> 3192
          OnePair Ten King Jack Nine         -> 3193
          OnePair Ten King Queen Two         -> 3194
          OnePair Ten King Queen Three       -> 3195
          OnePair Ten King Queen Four        -> 3196
          OnePair Ten King Queen Five        -> 3197
          OnePair Ten King Queen Six         -> 3198
          OnePair Ten King Queen Seven       -> 3199
          OnePair Ten King Queen Eight       -> 3200
          OnePair Ten King Queen Nine        -> 3201
          OnePair Ten King Queen Jack        -> 3202
          OnePair Ten Ace Three Two          -> 3203
          OnePair Ten Ace Four Two           -> 3204
          OnePair Ten Ace Four Three         -> 3205
          OnePair Ten Ace Five Two           -> 3206
          OnePair Ten Ace Five Three         -> 3207
          OnePair Ten Ace Five Four          -> 3208
          OnePair Ten Ace Six Two            -> 3209
          OnePair Ten Ace Six Three          -> 3210
          OnePair Ten Ace Six Four           -> 3211
          OnePair Ten Ace Six Five           -> 3212
          OnePair Ten Ace Seven Two          -> 3213
          OnePair Ten Ace Seven Three        -> 3214
          OnePair Ten Ace Seven Four         -> 3215
          OnePair Ten Ace Seven Five         -> 3216
          OnePair Ten Ace Seven Six          -> 3217
          OnePair Ten Ace Eight Two          -> 3218
          OnePair Ten Ace Eight Three        -> 3219
          OnePair Ten Ace Eight Four         -> 3220
          OnePair Ten Ace Eight Five         -> 3221
          OnePair Ten Ace Eight Six          -> 3222
          OnePair Ten Ace Eight Seven        -> 3223
          OnePair Ten Ace Nine Two           -> 3224
          OnePair Ten Ace Nine Three         -> 3225
          OnePair Ten Ace Nine Four          -> 3226
          OnePair Ten Ace Nine Five          -> 3227
          OnePair Ten Ace Nine Six           -> 3228
          OnePair Ten Ace Nine Seven         -> 3229
          OnePair Ten Ace Nine Eight         -> 3230
          OnePair Ten Ace Jack Two           -> 3231
          OnePair Ten Ace Jack Three         -> 3232
          OnePair Ten Ace Jack Four          -> 3233
          OnePair Ten Ace Jack Five          -> 3234
          OnePair Ten Ace Jack Six           -> 3235
          OnePair Ten Ace Jack Seven         -> 3236
          OnePair Ten Ace Jack Eight         -> 3237
          OnePair Ten Ace Jack Nine          -> 3238
          OnePair Ten Ace Queen Two          -> 3239
          OnePair Ten Ace Queen Three        -> 3240
          OnePair Ten Ace Queen Four         -> 3241
          OnePair Ten Ace Queen Five         -> 3242
          OnePair Ten Ace Queen Six          -> 3243
          OnePair Ten Ace Queen Seven        -> 3244
          OnePair Ten Ace Queen Eight        -> 3245
          OnePair Ten Ace Queen Nine         -> 3246
          OnePair Ten Ace Queen Jack         -> 3247
          OnePair Ten Ace King Two           -> 3248
          OnePair Ten Ace King Three         -> 3249
          OnePair Ten Ace King Four          -> 3250
          OnePair Ten Ace King Five          -> 3251
          OnePair Ten Ace King Six           -> 3252
          OnePair Ten Ace King Seven         -> 3253
          OnePair Ten Ace King Eight         -> 3254
          OnePair Ten Ace King Nine          -> 3255
          OnePair Ten Ace King Jack          -> 3256
          OnePair Ten Ace King Queen         -> 3257
          OnePair Jack Four Three Two        -> 3258
          OnePair Jack Five Three Two        -> 3259
          OnePair Jack Five Four Two         -> 3260
          OnePair Jack Five Four Three       -> 3261
          OnePair Jack Six Three Two         -> 3262
          OnePair Jack Six Four Two          -> 3263
          OnePair Jack Six Four Three        -> 3264
          OnePair Jack Six Five Two          -> 3265
          OnePair Jack Six Five Three        -> 3266
          OnePair Jack Six Five Four         -> 3267
          OnePair Jack Seven Three Two       -> 3268
          OnePair Jack Seven Four Two        -> 3269
          OnePair Jack Seven Four Three      -> 3270
          OnePair Jack Seven Five Two        -> 3271
          OnePair Jack Seven Five Three      -> 3272
          OnePair Jack Seven Five Four       -> 3273
          OnePair Jack Seven Six Two         -> 3274
          OnePair Jack Seven Six Three       -> 3275
          OnePair Jack Seven Six Four        -> 3276
          OnePair Jack Seven Six Five        -> 3277
          OnePair Jack Eight Three Two       -> 3278
          OnePair Jack Eight Four Two        -> 3279
          OnePair Jack Eight Four Three      -> 3280
          OnePair Jack Eight Five Two        -> 3281
          OnePair Jack Eight Five Three      -> 3282
          OnePair Jack Eight Five Four       -> 3283
          OnePair Jack Eight Six Two         -> 3284
          OnePair Jack Eight Six Three       -> 3285
          OnePair Jack Eight Six Four        -> 3286
          OnePair Jack Eight Six Five        -> 3287
          OnePair Jack Eight Seven Two       -> 3288
          OnePair Jack Eight Seven Three     -> 3289
          OnePair Jack Eight Seven Four      -> 3290
          OnePair Jack Eight Seven Five      -> 3291
          OnePair Jack Eight Seven Six       -> 3292
          OnePair Jack Nine Three Two        -> 3293
          OnePair Jack Nine Four Two         -> 3294
          OnePair Jack Nine Four Three       -> 3295
          OnePair Jack Nine Five Two         -> 3296
          OnePair Jack Nine Five Three       -> 3297
          OnePair Jack Nine Five Four        -> 3298
          OnePair Jack Nine Six Two          -> 3299
          OnePair Jack Nine Six Three        -> 3300
          OnePair Jack Nine Six Four         -> 3301
          OnePair Jack Nine Six Five         -> 3302
          OnePair Jack Nine Seven Two        -> 3303
          OnePair Jack Nine Seven Three      -> 3304
          OnePair Jack Nine Seven Four       -> 3305
          OnePair Jack Nine Seven Five       -> 3306
          OnePair Jack Nine Seven Six        -> 3307
          OnePair Jack Nine Eight Two        -> 3308
          OnePair Jack Nine Eight Three      -> 3309
          OnePair Jack Nine Eight Four       -> 3310
          OnePair Jack Nine Eight Five       -> 3311
          OnePair Jack Nine Eight Six        -> 3312
          OnePair Jack Nine Eight Seven      -> 3313
          OnePair Jack Ten Three Two         -> 3314
          OnePair Jack Ten Four Two          -> 3315
          OnePair Jack Ten Four Three        -> 3316
          OnePair Jack Ten Five Two          -> 3317
          OnePair Jack Ten Five Three        -> 3318
          OnePair Jack Ten Five Four         -> 3319
          OnePair Jack Ten Six Two           -> 3320
          OnePair Jack Ten Six Three         -> 3321
          OnePair Jack Ten Six Four          -> 3322
          OnePair Jack Ten Six Five          -> 3323
          OnePair Jack Ten Seven Two         -> 3324
          OnePair Jack Ten Seven Three       -> 3325
          OnePair Jack Ten Seven Four        -> 3326
          OnePair Jack Ten Seven Five        -> 3327
          OnePair Jack Ten Seven Six         -> 3328
          OnePair Jack Ten Eight Two         -> 3329
          OnePair Jack Ten Eight Three       -> 3330
          OnePair Jack Ten Eight Four        -> 3331
          OnePair Jack Ten Eight Five        -> 3332
          OnePair Jack Ten Eight Six         -> 3333
          OnePair Jack Ten Eight Seven       -> 3334
          OnePair Jack Ten Nine Two          -> 3335
          OnePair Jack Ten Nine Three        -> 3336
          OnePair Jack Ten Nine Four         -> 3337
          OnePair Jack Ten Nine Five         -> 3338
          OnePair Jack Ten Nine Six          -> 3339
          OnePair Jack Ten Nine Seven        -> 3340
          OnePair Jack Ten Nine Eight        -> 3341
          OnePair Jack Queen Three Two       -> 3342
          OnePair Jack Queen Four Two        -> 3343
          OnePair Jack Queen Four Three      -> 3344
          OnePair Jack Queen Five Two        -> 3345
          OnePair Jack Queen Five Three      -> 3346
          OnePair Jack Queen Five Four       -> 3347
          OnePair Jack Queen Six Two         -> 3348
          OnePair Jack Queen Six Three       -> 3349
          OnePair Jack Queen Six Four        -> 3350
          OnePair Jack Queen Six Five        -> 3351
          OnePair Jack Queen Seven Two       -> 3352
          OnePair Jack Queen Seven Three     -> 3353
          OnePair Jack Queen Seven Four      -> 3354
          OnePair Jack Queen Seven Five      -> 3355
          OnePair Jack Queen Seven Six       -> 3356
          OnePair Jack Queen Eight Two       -> 3357
          OnePair Jack Queen Eight Three     -> 3358
          OnePair Jack Queen Eight Four      -> 3359
          OnePair Jack Queen Eight Five      -> 3360
          OnePair Jack Queen Eight Six       -> 3361
          OnePair Jack Queen Eight Seven     -> 3362
          OnePair Jack Queen Nine Two        -> 3363
          OnePair Jack Queen Nine Three      -> 3364
          OnePair Jack Queen Nine Four       -> 3365
          OnePair Jack Queen Nine Five       -> 3366
          OnePair Jack Queen Nine Six        -> 3367
          OnePair Jack Queen Nine Seven      -> 3368
          OnePair Jack Queen Nine Eight      -> 3369
          OnePair Jack Queen Ten Two         -> 3370
          OnePair Jack Queen Ten Three       -> 3371
          OnePair Jack Queen Ten Four        -> 3372
          OnePair Jack Queen Ten Five        -> 3373
          OnePair Jack Queen Ten Six         -> 3374
          OnePair Jack Queen Ten Seven       -> 3375
          OnePair Jack Queen Ten Eight       -> 3376
          OnePair Jack Queen Ten Nine        -> 3377
          OnePair Jack King Three Two        -> 3378
          OnePair Jack King Four Two         -> 3379
          OnePair Jack King Four Three       -> 3380
          OnePair Jack King Five Two         -> 3381
          OnePair Jack King Five Three       -> 3382
          OnePair Jack King Five Four        -> 3383
          OnePair Jack King Six Two          -> 3384
          OnePair Jack King Six Three        -> 3385
          OnePair Jack King Six Four         -> 3386
          OnePair Jack King Six Five         -> 3387
          OnePair Jack King Seven Two        -> 3388
          OnePair Jack King Seven Three      -> 3389
          OnePair Jack King Seven Four       -> 3390
          OnePair Jack King Seven Five       -> 3391
          OnePair Jack King Seven Six        -> 3392
          OnePair Jack King Eight Two        -> 3393
          OnePair Jack King Eight Three      -> 3394
          OnePair Jack King Eight Four       -> 3395
          OnePair Jack King Eight Five       -> 3396
          OnePair Jack King Eight Six        -> 3397
          OnePair Jack King Eight Seven      -> 3398
          OnePair Jack King Nine Two         -> 3399
          OnePair Jack King Nine Three       -> 3400
          OnePair Jack King Nine Four        -> 3401
          OnePair Jack King Nine Five        -> 3402
          OnePair Jack King Nine Six         -> 3403
          OnePair Jack King Nine Seven       -> 3404
          OnePair Jack King Nine Eight       -> 3405
          OnePair Jack King Ten Two          -> 3406
          OnePair Jack King Ten Three        -> 3407
          OnePair Jack King Ten Four         -> 3408
          OnePair Jack King Ten Five         -> 3409
          OnePair Jack King Ten Six          -> 3410
          OnePair Jack King Ten Seven        -> 3411
          OnePair Jack King Ten Eight        -> 3412
          OnePair Jack King Ten Nine         -> 3413
          OnePair Jack King Queen Two        -> 3414
          OnePair Jack King Queen Three      -> 3415
          OnePair Jack King Queen Four       -> 3416
          OnePair Jack King Queen Five       -> 3417
          OnePair Jack King Queen Six        -> 3418
          OnePair Jack King Queen Seven      -> 3419
          OnePair Jack King Queen Eight      -> 3420
          OnePair Jack King Queen Nine       -> 3421
          OnePair Jack King Queen Ten        -> 3422
          OnePair Jack Ace Three Two         -> 3423
          OnePair Jack Ace Four Two          -> 3424
          OnePair Jack Ace Four Three        -> 3425
          OnePair Jack Ace Five Two          -> 3426
          OnePair Jack Ace Five Three        -> 3427
          OnePair Jack Ace Five Four         -> 3428
          OnePair Jack Ace Six Two           -> 3429
          OnePair Jack Ace Six Three         -> 3430
          OnePair Jack Ace Six Four          -> 3431
          OnePair Jack Ace Six Five          -> 3432
          OnePair Jack Ace Seven Two         -> 3433
          OnePair Jack Ace Seven Three       -> 3434
          OnePair Jack Ace Seven Four        -> 3435
          OnePair Jack Ace Seven Five        -> 3436
          OnePair Jack Ace Seven Six         -> 3437
          OnePair Jack Ace Eight Two         -> 3438
          OnePair Jack Ace Eight Three       -> 3439
          OnePair Jack Ace Eight Four        -> 3440
          OnePair Jack Ace Eight Five        -> 3441
          OnePair Jack Ace Eight Six         -> 3442
          OnePair Jack Ace Eight Seven       -> 3443
          OnePair Jack Ace Nine Two          -> 3444
          OnePair Jack Ace Nine Three        -> 3445
          OnePair Jack Ace Nine Four         -> 3446
          OnePair Jack Ace Nine Five         -> 3447
          OnePair Jack Ace Nine Six          -> 3448
          OnePair Jack Ace Nine Seven        -> 3449
          OnePair Jack Ace Nine Eight        -> 3450
          OnePair Jack Ace Ten Two           -> 3451
          OnePair Jack Ace Ten Three         -> 3452
          OnePair Jack Ace Ten Four          -> 3453
          OnePair Jack Ace Ten Five          -> 3454
          OnePair Jack Ace Ten Six           -> 3455
          OnePair Jack Ace Ten Seven         -> 3456
          OnePair Jack Ace Ten Eight         -> 3457
          OnePair Jack Ace Ten Nine          -> 3458
          OnePair Jack Ace Queen Two         -> 3459
          OnePair Jack Ace Queen Three       -> 3460
          OnePair Jack Ace Queen Four        -> 3461
          OnePair Jack Ace Queen Five        -> 3462
          OnePair Jack Ace Queen Six         -> 3463
          OnePair Jack Ace Queen Seven       -> 3464
          OnePair Jack Ace Queen Eight       -> 3465
          OnePair Jack Ace Queen Nine        -> 3466
          OnePair Jack Ace Queen Ten         -> 3467
          OnePair Jack Ace King Two          -> 3468
          OnePair Jack Ace King Three        -> 3469
          OnePair Jack Ace King Four         -> 3470
          OnePair Jack Ace King Five         -> 3471
          OnePair Jack Ace King Six          -> 3472
          OnePair Jack Ace King Seven        -> 3473
          OnePair Jack Ace King Eight        -> 3474
          OnePair Jack Ace King Nine         -> 3475
          OnePair Jack Ace King Ten          -> 3476
          OnePair Jack Ace King Queen        -> 3477
          OnePair Queen Four Three Two       -> 3478
          OnePair Queen Five Three Two       -> 3479
          OnePair Queen Five Four Two        -> 3480
          OnePair Queen Five Four Three      -> 3481
          OnePair Queen Six Three Two        -> 3482
          OnePair Queen Six Four Two         -> 3483
          OnePair Queen Six Four Three       -> 3484
          OnePair Queen Six Five Two         -> 3485
          OnePair Queen Six Five Three       -> 3486
          OnePair Queen Six Five Four        -> 3487
          OnePair Queen Seven Three Two      -> 3488
          OnePair Queen Seven Four Two       -> 3489
          OnePair Queen Seven Four Three     -> 3490
          OnePair Queen Seven Five Two       -> 3491
          OnePair Queen Seven Five Three     -> 3492
          OnePair Queen Seven Five Four      -> 3493
          OnePair Queen Seven Six Two        -> 3494
          OnePair Queen Seven Six Three      -> 3495
          OnePair Queen Seven Six Four       -> 3496
          OnePair Queen Seven Six Five       -> 3497
          OnePair Queen Eight Three Two      -> 3498
          OnePair Queen Eight Four Two       -> 3499
          OnePair Queen Eight Four Three     -> 3500
          OnePair Queen Eight Five Two       -> 3501
          OnePair Queen Eight Five Three     -> 3502
          OnePair Queen Eight Five Four      -> 3503
          OnePair Queen Eight Six Two        -> 3504
          OnePair Queen Eight Six Three      -> 3505
          OnePair Queen Eight Six Four       -> 3506
          OnePair Queen Eight Six Five       -> 3507
          OnePair Queen Eight Seven Two      -> 3508
          OnePair Queen Eight Seven Three    -> 3509
          OnePair Queen Eight Seven Four     -> 3510
          OnePair Queen Eight Seven Five     -> 3511
          OnePair Queen Eight Seven Six      -> 3512
          OnePair Queen Nine Three Two       -> 3513
          OnePair Queen Nine Four Two        -> 3514
          OnePair Queen Nine Four Three      -> 3515
          OnePair Queen Nine Five Two        -> 3516
          OnePair Queen Nine Five Three      -> 3517
          OnePair Queen Nine Five Four       -> 3518
          OnePair Queen Nine Six Two         -> 3519
          OnePair Queen Nine Six Three       -> 3520
          OnePair Queen Nine Six Four        -> 3521
          OnePair Queen Nine Six Five        -> 3522
          OnePair Queen Nine Seven Two       -> 3523
          OnePair Queen Nine Seven Three     -> 3524
          OnePair Queen Nine Seven Four      -> 3525
          OnePair Queen Nine Seven Five      -> 3526
          OnePair Queen Nine Seven Six       -> 3527
          OnePair Queen Nine Eight Two       -> 3528
          OnePair Queen Nine Eight Three     -> 3529
          OnePair Queen Nine Eight Four      -> 3530
          OnePair Queen Nine Eight Five      -> 3531
          OnePair Queen Nine Eight Six       -> 3532
          OnePair Queen Nine Eight Seven     -> 3533
          OnePair Queen Ten Three Two        -> 3534
          OnePair Queen Ten Four Two         -> 3535
          OnePair Queen Ten Four Three       -> 3536
          OnePair Queen Ten Five Two         -> 3537
          OnePair Queen Ten Five Three       -> 3538
          OnePair Queen Ten Five Four        -> 3539
          OnePair Queen Ten Six Two          -> 3540
          OnePair Queen Ten Six Three        -> 3541
          OnePair Queen Ten Six Four         -> 3542
          OnePair Queen Ten Six Five         -> 3543
          OnePair Queen Ten Seven Two        -> 3544
          OnePair Queen Ten Seven Three      -> 3545
          OnePair Queen Ten Seven Four       -> 3546
          OnePair Queen Ten Seven Five       -> 3547
          OnePair Queen Ten Seven Six        -> 3548
          OnePair Queen Ten Eight Two        -> 3549
          OnePair Queen Ten Eight Three      -> 3550
          OnePair Queen Ten Eight Four       -> 3551
          OnePair Queen Ten Eight Five       -> 3552
          OnePair Queen Ten Eight Six        -> 3553
          OnePair Queen Ten Eight Seven      -> 3554
          OnePair Queen Ten Nine Two         -> 3555
          OnePair Queen Ten Nine Three       -> 3556
          OnePair Queen Ten Nine Four        -> 3557
          OnePair Queen Ten Nine Five        -> 3558
          OnePair Queen Ten Nine Six         -> 3559
          OnePair Queen Ten Nine Seven       -> 3560
          OnePair Queen Ten Nine Eight       -> 3561
          OnePair Queen Jack Three Two       -> 3562
          OnePair Queen Jack Four Two        -> 3563
          OnePair Queen Jack Four Three      -> 3564
          OnePair Queen Jack Five Two        -> 3565
          OnePair Queen Jack Five Three      -> 3566
          OnePair Queen Jack Five Four       -> 3567
          OnePair Queen Jack Six Two         -> 3568
          OnePair Queen Jack Six Three       -> 3569
          OnePair Queen Jack Six Four        -> 3570
          OnePair Queen Jack Six Five        -> 3571
          OnePair Queen Jack Seven Two       -> 3572
          OnePair Queen Jack Seven Three     -> 3573
          OnePair Queen Jack Seven Four      -> 3574
          OnePair Queen Jack Seven Five      -> 3575
          OnePair Queen Jack Seven Six       -> 3576
          OnePair Queen Jack Eight Two       -> 3577
          OnePair Queen Jack Eight Three     -> 3578
          OnePair Queen Jack Eight Four      -> 3579
          OnePair Queen Jack Eight Five      -> 3580
          OnePair Queen Jack Eight Six       -> 3581
          OnePair Queen Jack Eight Seven     -> 3582
          OnePair Queen Jack Nine Two        -> 3583
          OnePair Queen Jack Nine Three      -> 3584
          OnePair Queen Jack Nine Four       -> 3585
          OnePair Queen Jack Nine Five       -> 3586
          OnePair Queen Jack Nine Six        -> 3587
          OnePair Queen Jack Nine Seven      -> 3588
          OnePair Queen Jack Nine Eight      -> 3589
          OnePair Queen Jack Ten Two         -> 3590
          OnePair Queen Jack Ten Three       -> 3591
          OnePair Queen Jack Ten Four        -> 3592
          OnePair Queen Jack Ten Five        -> 3593
          OnePair Queen Jack Ten Six         -> 3594
          OnePair Queen Jack Ten Seven       -> 3595
          OnePair Queen Jack Ten Eight       -> 3596
          OnePair Queen Jack Ten Nine        -> 3597
          OnePair Queen King Three Two       -> 3598
          OnePair Queen King Four Two        -> 3599
          OnePair Queen King Four Three      -> 3600
          OnePair Queen King Five Two        -> 3601
          OnePair Queen King Five Three      -> 3602
          OnePair Queen King Five Four       -> 3603
          OnePair Queen King Six Two         -> 3604
          OnePair Queen King Six Three       -> 3605
          OnePair Queen King Six Four        -> 3606
          OnePair Queen King Six Five        -> 3607
          OnePair Queen King Seven Two       -> 3608
          OnePair Queen King Seven Three     -> 3609
          OnePair Queen King Seven Four      -> 3610
          OnePair Queen King Seven Five      -> 3611
          OnePair Queen King Seven Six       -> 3612
          OnePair Queen King Eight Two       -> 3613
          OnePair Queen King Eight Three     -> 3614
          OnePair Queen King Eight Four      -> 3615
          OnePair Queen King Eight Five      -> 3616
          OnePair Queen King Eight Six       -> 3617
          OnePair Queen King Eight Seven     -> 3618
          OnePair Queen King Nine Two        -> 3619
          OnePair Queen King Nine Three      -> 3620
          OnePair Queen King Nine Four       -> 3621
          OnePair Queen King Nine Five       -> 3622
          OnePair Queen King Nine Six        -> 3623
          OnePair Queen King Nine Seven      -> 3624
          OnePair Queen King Nine Eight      -> 3625
          OnePair Queen King Ten Two         -> 3626
          OnePair Queen King Ten Three       -> 3627
          OnePair Queen King Ten Four        -> 3628
          OnePair Queen King Ten Five        -> 3629
          OnePair Queen King Ten Six         -> 3630
          OnePair Queen King Ten Seven       -> 3631
          OnePair Queen King Ten Eight       -> 3632
          OnePair Queen King Ten Nine        -> 3633
          OnePair Queen King Jack Two        -> 3634
          OnePair Queen King Jack Three      -> 3635
          OnePair Queen King Jack Four       -> 3636
          OnePair Queen King Jack Five       -> 3637
          OnePair Queen King Jack Six        -> 3638
          OnePair Queen King Jack Seven      -> 3639
          OnePair Queen King Jack Eight      -> 3640
          OnePair Queen King Jack Nine       -> 3641
          OnePair Queen King Jack Ten        -> 3642
          OnePair Queen Ace Three Two        -> 3643
          OnePair Queen Ace Four Two         -> 3644
          OnePair Queen Ace Four Three       -> 3645
          OnePair Queen Ace Five Two         -> 3646
          OnePair Queen Ace Five Three       -> 3647
          OnePair Queen Ace Five Four        -> 3648
          OnePair Queen Ace Six Two          -> 3649
          OnePair Queen Ace Six Three        -> 3650
          OnePair Queen Ace Six Four         -> 3651
          OnePair Queen Ace Six Five         -> 3652
          OnePair Queen Ace Seven Two        -> 3653
          OnePair Queen Ace Seven Three      -> 3654
          OnePair Queen Ace Seven Four       -> 3655
          OnePair Queen Ace Seven Five       -> 3656
          OnePair Queen Ace Seven Six        -> 3657
          OnePair Queen Ace Eight Two        -> 3658
          OnePair Queen Ace Eight Three      -> 3659
          OnePair Queen Ace Eight Four       -> 3660
          OnePair Queen Ace Eight Five       -> 3661
          OnePair Queen Ace Eight Six        -> 3662
          OnePair Queen Ace Eight Seven      -> 3663
          OnePair Queen Ace Nine Two         -> 3664
          OnePair Queen Ace Nine Three       -> 3665
          OnePair Queen Ace Nine Four        -> 3666
          OnePair Queen Ace Nine Five        -> 3667
          OnePair Queen Ace Nine Six         -> 3668
          OnePair Queen Ace Nine Seven       -> 3669
          OnePair Queen Ace Nine Eight       -> 3670
          OnePair Queen Ace Ten Two          -> 3671
          OnePair Queen Ace Ten Three        -> 3672
          OnePair Queen Ace Ten Four         -> 3673
          OnePair Queen Ace Ten Five         -> 3674
          OnePair Queen Ace Ten Six          -> 3675
          OnePair Queen Ace Ten Seven        -> 3676
          OnePair Queen Ace Ten Eight        -> 3677
          OnePair Queen Ace Ten Nine         -> 3678
          OnePair Queen Ace Jack Two         -> 3679
          OnePair Queen Ace Jack Three       -> 3680
          OnePair Queen Ace Jack Four        -> 3681
          OnePair Queen Ace Jack Five        -> 3682
          OnePair Queen Ace Jack Six         -> 3683
          OnePair Queen Ace Jack Seven       -> 3684
          OnePair Queen Ace Jack Eight       -> 3685
          OnePair Queen Ace Jack Nine        -> 3686
          OnePair Queen Ace Jack Ten         -> 3687
          OnePair Queen Ace King Two         -> 3688
          OnePair Queen Ace King Three       -> 3689
          OnePair Queen Ace King Four        -> 3690
          OnePair Queen Ace King Five        -> 3691
          OnePair Queen Ace King Six         -> 3692
          OnePair Queen Ace King Seven       -> 3693
          OnePair Queen Ace King Eight       -> 3694
          OnePair Queen Ace King Nine        -> 3695
          OnePair Queen Ace King Ten         -> 3696
          OnePair Queen Ace King Jack        -> 3697
          OnePair King Four Three Two        -> 3698
          OnePair King Five Three Two        -> 3699
          OnePair King Five Four Two         -> 3700
          OnePair King Five Four Three       -> 3701
          OnePair King Six Three Two         -> 3702
          OnePair King Six Four Two          -> 3703
          OnePair King Six Four Three        -> 3704
          OnePair King Six Five Two          -> 3705
          OnePair King Six Five Three        -> 3706
          OnePair King Six Five Four         -> 3707
          OnePair King Seven Three Two       -> 3708
          OnePair King Seven Four Two        -> 3709
          OnePair King Seven Four Three      -> 3710
          OnePair King Seven Five Two        -> 3711
          OnePair King Seven Five Three      -> 3712
          OnePair King Seven Five Four       -> 3713
          OnePair King Seven Six Two         -> 3714
          OnePair King Seven Six Three       -> 3715
          OnePair King Seven Six Four        -> 3716
          OnePair King Seven Six Five        -> 3717
          OnePair King Eight Three Two       -> 3718
          OnePair King Eight Four Two        -> 3719
          OnePair King Eight Four Three      -> 3720
          OnePair King Eight Five Two        -> 3721
          OnePair King Eight Five Three      -> 3722
          OnePair King Eight Five Four       -> 3723
          OnePair King Eight Six Two         -> 3724
          OnePair King Eight Six Three       -> 3725
          OnePair King Eight Six Four        -> 3726
          OnePair King Eight Six Five        -> 3727
          OnePair King Eight Seven Two       -> 3728
          OnePair King Eight Seven Three     -> 3729
          OnePair King Eight Seven Four      -> 3730
          OnePair King Eight Seven Five      -> 3731
          OnePair King Eight Seven Six       -> 3732
          OnePair King Nine Three Two        -> 3733
          OnePair King Nine Four Two         -> 3734
          OnePair King Nine Four Three       -> 3735
          OnePair King Nine Five Two         -> 3736
          OnePair King Nine Five Three       -> 3737
          OnePair King Nine Five Four        -> 3738
          OnePair King Nine Six Two          -> 3739
          OnePair King Nine Six Three        -> 3740
          OnePair King Nine Six Four         -> 3741
          OnePair King Nine Six Five         -> 3742
          OnePair King Nine Seven Two        -> 3743
          OnePair King Nine Seven Three      -> 3744
          OnePair King Nine Seven Four       -> 3745
          OnePair King Nine Seven Five       -> 3746
          OnePair King Nine Seven Six        -> 3747
          OnePair King Nine Eight Two        -> 3748
          OnePair King Nine Eight Three      -> 3749
          OnePair King Nine Eight Four       -> 3750
          OnePair King Nine Eight Five       -> 3751
          OnePair King Nine Eight Six        -> 3752
          OnePair King Nine Eight Seven      -> 3753
          OnePair King Ten Three Two         -> 3754
          OnePair King Ten Four Two          -> 3755
          OnePair King Ten Four Three        -> 3756
          OnePair King Ten Five Two          -> 3757
          OnePair King Ten Five Three        -> 3758
          OnePair King Ten Five Four         -> 3759
          OnePair King Ten Six Two           -> 3760
          OnePair King Ten Six Three         -> 3761
          OnePair King Ten Six Four          -> 3762
          OnePair King Ten Six Five          -> 3763
          OnePair King Ten Seven Two         -> 3764
          OnePair King Ten Seven Three       -> 3765
          OnePair King Ten Seven Four        -> 3766
          OnePair King Ten Seven Five        -> 3767
          OnePair King Ten Seven Six         -> 3768
          OnePair King Ten Eight Two         -> 3769
          OnePair King Ten Eight Three       -> 3770
          OnePair King Ten Eight Four        -> 3771
          OnePair King Ten Eight Five        -> 3772
          OnePair King Ten Eight Six         -> 3773
          OnePair King Ten Eight Seven       -> 3774
          OnePair King Ten Nine Two          -> 3775
          OnePair King Ten Nine Three        -> 3776
          OnePair King Ten Nine Four         -> 3777
          OnePair King Ten Nine Five         -> 3778
          OnePair King Ten Nine Six          -> 3779
          OnePair King Ten Nine Seven        -> 3780
          OnePair King Ten Nine Eight        -> 3781
          OnePair King Jack Three Two        -> 3782
          OnePair King Jack Four Two         -> 3783
          OnePair King Jack Four Three       -> 3784
          OnePair King Jack Five Two         -> 3785
          OnePair King Jack Five Three       -> 3786
          OnePair King Jack Five Four        -> 3787
          OnePair King Jack Six Two          -> 3788
          OnePair King Jack Six Three        -> 3789
          OnePair King Jack Six Four         -> 3790
          OnePair King Jack Six Five         -> 3791
          OnePair King Jack Seven Two        -> 3792
          OnePair King Jack Seven Three      -> 3793
          OnePair King Jack Seven Four       -> 3794
          OnePair King Jack Seven Five       -> 3795
          OnePair King Jack Seven Six        -> 3796
          OnePair King Jack Eight Two        -> 3797
          OnePair King Jack Eight Three      -> 3798
          OnePair King Jack Eight Four       -> 3799
          OnePair King Jack Eight Five       -> 3800
          OnePair King Jack Eight Six        -> 3801
          OnePair King Jack Eight Seven      -> 3802
          OnePair King Jack Nine Two         -> 3803
          OnePair King Jack Nine Three       -> 3804
          OnePair King Jack Nine Four        -> 3805
          OnePair King Jack Nine Five        -> 3806
          OnePair King Jack Nine Six         -> 3807
          OnePair King Jack Nine Seven       -> 3808
          OnePair King Jack Nine Eight       -> 3809
          OnePair King Jack Ten Two          -> 3810
          OnePair King Jack Ten Three        -> 3811
          OnePair King Jack Ten Four         -> 3812
          OnePair King Jack Ten Five         -> 3813
          OnePair King Jack Ten Six          -> 3814
          OnePair King Jack Ten Seven        -> 3815
          OnePair King Jack Ten Eight        -> 3816
          OnePair King Jack Ten Nine         -> 3817
          OnePair King Queen Three Two       -> 3818
          OnePair King Queen Four Two        -> 3819
          OnePair King Queen Four Three      -> 3820
          OnePair King Queen Five Two        -> 3821
          OnePair King Queen Five Three      -> 3822
          OnePair King Queen Five Four       -> 3823
          OnePair King Queen Six Two         -> 3824
          OnePair King Queen Six Three       -> 3825
          OnePair King Queen Six Four        -> 3826
          OnePair King Queen Six Five        -> 3827
          OnePair King Queen Seven Two       -> 3828
          OnePair King Queen Seven Three     -> 3829
          OnePair King Queen Seven Four      -> 3830
          OnePair King Queen Seven Five      -> 3831
          OnePair King Queen Seven Six       -> 3832
          OnePair King Queen Eight Two       -> 3833
          OnePair King Queen Eight Three     -> 3834
          OnePair King Queen Eight Four      -> 3835
          OnePair King Queen Eight Five      -> 3836
          OnePair King Queen Eight Six       -> 3837
          OnePair King Queen Eight Seven     -> 3838
          OnePair King Queen Nine Two        -> 3839
          OnePair King Queen Nine Three      -> 3840
          OnePair King Queen Nine Four       -> 3841
          OnePair King Queen Nine Five       -> 3842
          OnePair King Queen Nine Six        -> 3843
          OnePair King Queen Nine Seven      -> 3844
          OnePair King Queen Nine Eight      -> 3845
          OnePair King Queen Ten Two         -> 3846
          OnePair King Queen Ten Three       -> 3847
          OnePair King Queen Ten Four        -> 3848
          OnePair King Queen Ten Five        -> 3849
          OnePair King Queen Ten Six         -> 3850
          OnePair King Queen Ten Seven       -> 3851
          OnePair King Queen Ten Eight       -> 3852
          OnePair King Queen Ten Nine        -> 3853
          OnePair King Queen Jack Two        -> 3854
          OnePair King Queen Jack Three      -> 3855
          OnePair King Queen Jack Four       -> 3856
          OnePair King Queen Jack Five       -> 3857
          OnePair King Queen Jack Six        -> 3858
          OnePair King Queen Jack Seven      -> 3859
          OnePair King Queen Jack Eight      -> 3860
          OnePair King Queen Jack Nine       -> 3861
          OnePair King Queen Jack Ten        -> 3862
          OnePair King Ace Three Two         -> 3863
          OnePair King Ace Four Two          -> 3864
          OnePair King Ace Four Three        -> 3865
          OnePair King Ace Five Two          -> 3866
          OnePair King Ace Five Three        -> 3867
          OnePair King Ace Five Four         -> 3868
          OnePair King Ace Six Two           -> 3869
          OnePair King Ace Six Three         -> 3870
          OnePair King Ace Six Four          -> 3871
          OnePair King Ace Six Five          -> 3872
          OnePair King Ace Seven Two         -> 3873
          OnePair King Ace Seven Three       -> 3874
          OnePair King Ace Seven Four        -> 3875
          OnePair King Ace Seven Five        -> 3876
          OnePair King Ace Seven Six         -> 3877
          OnePair King Ace Eight Two         -> 3878
          OnePair King Ace Eight Three       -> 3879
          OnePair King Ace Eight Four        -> 3880
          OnePair King Ace Eight Five        -> 3881
          OnePair King Ace Eight Six         -> 3882
          OnePair King Ace Eight Seven       -> 3883
          OnePair King Ace Nine Two          -> 3884
          OnePair King Ace Nine Three        -> 3885
          OnePair King Ace Nine Four         -> 3886
          OnePair King Ace Nine Five         -> 3887
          OnePair King Ace Nine Six          -> 3888
          OnePair King Ace Nine Seven        -> 3889
          OnePair King Ace Nine Eight        -> 3890
          OnePair King Ace Ten Two           -> 3891
          OnePair King Ace Ten Three         -> 3892
          OnePair King Ace Ten Four          -> 3893
          OnePair King Ace Ten Five          -> 3894
          OnePair King Ace Ten Six           -> 3895
          OnePair King Ace Ten Seven         -> 3896
          OnePair King Ace Ten Eight         -> 3897
          OnePair King Ace Ten Nine          -> 3898
          OnePair King Ace Jack Two          -> 3899
          OnePair King Ace Jack Three        -> 3900
          OnePair King Ace Jack Four         -> 3901
          OnePair King Ace Jack Five         -> 3902
          OnePair King Ace Jack Six          -> 3903
          OnePair King Ace Jack Seven        -> 3904
          OnePair King Ace Jack Eight        -> 3905
          OnePair King Ace Jack Nine         -> 3906
          OnePair King Ace Jack Ten          -> 3907
          OnePair King Ace Queen Two         -> 3908
          OnePair King Ace Queen Three       -> 3909
          OnePair King Ace Queen Four        -> 3910
          OnePair King Ace Queen Five        -> 3911
          OnePair King Ace Queen Six         -> 3912
          OnePair King Ace Queen Seven       -> 3913
          OnePair King Ace Queen Eight       -> 3914
          OnePair King Ace Queen Nine        -> 3915
          OnePair King Ace Queen Ten         -> 3916
          OnePair King Ace Queen Jack        -> 3917
          OnePair Ace Four Three Two         -> 3918
          OnePair Ace Five Three Two         -> 3919
          OnePair Ace Five Four Two          -> 3920
          OnePair Ace Five Four Three        -> 3921
          OnePair Ace Six Three Two          -> 3922
          OnePair Ace Six Four Two           -> 3923
          OnePair Ace Six Four Three         -> 3924
          OnePair Ace Six Five Two           -> 3925
          OnePair Ace Six Five Three         -> 3926
          OnePair Ace Six Five Four          -> 3927
          OnePair Ace Seven Three Two        -> 3928
          OnePair Ace Seven Four Two         -> 3929
          OnePair Ace Seven Four Three       -> 3930
          OnePair Ace Seven Five Two         -> 3931
          OnePair Ace Seven Five Three       -> 3932
          OnePair Ace Seven Five Four        -> 3933
          OnePair Ace Seven Six Two          -> 3934
          OnePair Ace Seven Six Three        -> 3935
          OnePair Ace Seven Six Four         -> 3936
          OnePair Ace Seven Six Five         -> 3937
          OnePair Ace Eight Three Two        -> 3938
          OnePair Ace Eight Four Two         -> 3939
          OnePair Ace Eight Four Three       -> 3940
          OnePair Ace Eight Five Two         -> 3941
          OnePair Ace Eight Five Three       -> 3942
          OnePair Ace Eight Five Four        -> 3943
          OnePair Ace Eight Six Two          -> 3944
          OnePair Ace Eight Six Three        -> 3945
          OnePair Ace Eight Six Four         -> 3946
          OnePair Ace Eight Six Five         -> 3947
          OnePair Ace Eight Seven Two        -> 3948
          OnePair Ace Eight Seven Three      -> 3949
          OnePair Ace Eight Seven Four       -> 3950
          OnePair Ace Eight Seven Five       -> 3951
          OnePair Ace Eight Seven Six        -> 3952
          OnePair Ace Nine Three Two         -> 3953
          OnePair Ace Nine Four Two          -> 3954
          OnePair Ace Nine Four Three        -> 3955
          OnePair Ace Nine Five Two          -> 3956
          OnePair Ace Nine Five Three        -> 3957
          OnePair Ace Nine Five Four         -> 3958
          OnePair Ace Nine Six Two           -> 3959
          OnePair Ace Nine Six Three         -> 3960
          OnePair Ace Nine Six Four          -> 3961
          OnePair Ace Nine Six Five          -> 3962
          OnePair Ace Nine Seven Two         -> 3963
          OnePair Ace Nine Seven Three       -> 3964
          OnePair Ace Nine Seven Four        -> 3965
          OnePair Ace Nine Seven Five        -> 3966
          OnePair Ace Nine Seven Six         -> 3967
          OnePair Ace Nine Eight Two         -> 3968
          OnePair Ace Nine Eight Three       -> 3969
          OnePair Ace Nine Eight Four        -> 3970
          OnePair Ace Nine Eight Five        -> 3971
          OnePair Ace Nine Eight Six         -> 3972
          OnePair Ace Nine Eight Seven       -> 3973
          OnePair Ace Ten Three Two          -> 3974
          OnePair Ace Ten Four Two           -> 3975
          OnePair Ace Ten Four Three         -> 3976
          OnePair Ace Ten Five Two           -> 3977
          OnePair Ace Ten Five Three         -> 3978
          OnePair Ace Ten Five Four          -> 3979
          OnePair Ace Ten Six Two            -> 3980
          OnePair Ace Ten Six Three          -> 3981
          OnePair Ace Ten Six Four           -> 3982
          OnePair Ace Ten Six Five           -> 3983
          OnePair Ace Ten Seven Two          -> 3984
          OnePair Ace Ten Seven Three        -> 3985
          OnePair Ace Ten Seven Four         -> 3986
          OnePair Ace Ten Seven Five         -> 3987
          OnePair Ace Ten Seven Six          -> 3988
          OnePair Ace Ten Eight Two          -> 3989
          OnePair Ace Ten Eight Three        -> 3990
          OnePair Ace Ten Eight Four         -> 3991
          OnePair Ace Ten Eight Five         -> 3992
          OnePair Ace Ten Eight Six          -> 3993
          OnePair Ace Ten Eight Seven        -> 3994
          OnePair Ace Ten Nine Two           -> 3995
          OnePair Ace Ten Nine Three         -> 3996
          OnePair Ace Ten Nine Four          -> 3997
          OnePair Ace Ten Nine Five          -> 3998
          OnePair Ace Ten Nine Six           -> 3999
          OnePair Ace Ten Nine Seven         -> 4000
          OnePair Ace Ten Nine Eight         -> 4001
          OnePair Ace Jack Three Two         -> 4002
          OnePair Ace Jack Four Two          -> 4003
          OnePair Ace Jack Four Three        -> 4004
          OnePair Ace Jack Five Two          -> 4005
          OnePair Ace Jack Five Three        -> 4006
          OnePair Ace Jack Five Four         -> 4007
          OnePair Ace Jack Six Two           -> 4008
          OnePair Ace Jack Six Three         -> 4009
          OnePair Ace Jack Six Four          -> 4010
          OnePair Ace Jack Six Five          -> 4011
          OnePair Ace Jack Seven Two         -> 4012
          OnePair Ace Jack Seven Three       -> 4013
          OnePair Ace Jack Seven Four        -> 4014
          OnePair Ace Jack Seven Five        -> 4015
          OnePair Ace Jack Seven Six         -> 4016
          OnePair Ace Jack Eight Two         -> 4017
          OnePair Ace Jack Eight Three       -> 4018
          OnePair Ace Jack Eight Four        -> 4019
          OnePair Ace Jack Eight Five        -> 4020
          OnePair Ace Jack Eight Six         -> 4021
          OnePair Ace Jack Eight Seven       -> 4022
          OnePair Ace Jack Nine Two          -> 4023
          OnePair Ace Jack Nine Three        -> 4024
          OnePair Ace Jack Nine Four         -> 4025
          OnePair Ace Jack Nine Five         -> 4026
          OnePair Ace Jack Nine Six          -> 4027
          OnePair Ace Jack Nine Seven        -> 4028
          OnePair Ace Jack Nine Eight        -> 4029
          OnePair Ace Jack Ten Two           -> 4030
          OnePair Ace Jack Ten Three         -> 4031
          OnePair Ace Jack Ten Four          -> 4032
          OnePair Ace Jack Ten Five          -> 4033
          OnePair Ace Jack Ten Six           -> 4034
          OnePair Ace Jack Ten Seven         -> 4035
          OnePair Ace Jack Ten Eight         -> 4036
          OnePair Ace Jack Ten Nine          -> 4037
          OnePair Ace Queen Three Two        -> 4038
          OnePair Ace Queen Four Two         -> 4039
          OnePair Ace Queen Four Three       -> 4040
          OnePair Ace Queen Five Two         -> 4041
          OnePair Ace Queen Five Three       -> 4042
          OnePair Ace Queen Five Four        -> 4043
          OnePair Ace Queen Six Two          -> 4044
          OnePair Ace Queen Six Three        -> 4045
          OnePair Ace Queen Six Four         -> 4046
          OnePair Ace Queen Six Five         -> 4047
          OnePair Ace Queen Seven Two        -> 4048
          OnePair Ace Queen Seven Three      -> 4049
          OnePair Ace Queen Seven Four       -> 4050
          OnePair Ace Queen Seven Five       -> 4051
          OnePair Ace Queen Seven Six        -> 4052
          OnePair Ace Queen Eight Two        -> 4053
          OnePair Ace Queen Eight Three      -> 4054
          OnePair Ace Queen Eight Four       -> 4055
          OnePair Ace Queen Eight Five       -> 4056
          OnePair Ace Queen Eight Six        -> 4057
          OnePair Ace Queen Eight Seven      -> 4058
          OnePair Ace Queen Nine Two         -> 4059
          OnePair Ace Queen Nine Three       -> 4060
          OnePair Ace Queen Nine Four        -> 4061
          OnePair Ace Queen Nine Five        -> 4062
          OnePair Ace Queen Nine Six         -> 4063
          OnePair Ace Queen Nine Seven       -> 4064
          OnePair Ace Queen Nine Eight       -> 4065
          OnePair Ace Queen Ten Two          -> 4066
          OnePair Ace Queen Ten Three        -> 4067
          OnePair Ace Queen Ten Four         -> 4068
          OnePair Ace Queen Ten Five         -> 4069
          OnePair Ace Queen Ten Six          -> 4070
          OnePair Ace Queen Ten Seven        -> 4071
          OnePair Ace Queen Ten Eight        -> 4072
          OnePair Ace Queen Ten Nine         -> 4073
          OnePair Ace Queen Jack Two         -> 4074
          OnePair Ace Queen Jack Three       -> 4075
          OnePair Ace Queen Jack Four        -> 4076
          OnePair Ace Queen Jack Five        -> 4077
          OnePair Ace Queen Jack Six         -> 4078
          OnePair Ace Queen Jack Seven       -> 4079
          OnePair Ace Queen Jack Eight       -> 4080
          OnePair Ace Queen Jack Nine        -> 4081
          OnePair Ace Queen Jack Ten         -> 4082
          OnePair Ace King Three Two         -> 4083
          OnePair Ace King Four Two          -> 4084
          OnePair Ace King Four Three        -> 4085
          OnePair Ace King Five Two          -> 4086
          OnePair Ace King Five Three        -> 4087
          OnePair Ace King Five Four         -> 4088
          OnePair Ace King Six Two           -> 4089
          OnePair Ace King Six Three         -> 4090
          OnePair Ace King Six Four          -> 4091
          OnePair Ace King Six Five          -> 4092
          OnePair Ace King Seven Two         -> 4093
          OnePair Ace King Seven Three       -> 4094
          OnePair Ace King Seven Four        -> 4095
          OnePair Ace King Seven Five        -> 4096
          OnePair Ace King Seven Six         -> 4097
          OnePair Ace King Eight Two         -> 4098
          OnePair Ace King Eight Three       -> 4099
          OnePair Ace King Eight Four        -> 4100
          OnePair Ace King Eight Five        -> 4101
          OnePair Ace King Eight Six         -> 4102
          OnePair Ace King Eight Seven       -> 4103
          OnePair Ace King Nine Two          -> 4104
          OnePair Ace King Nine Three        -> 4105
          OnePair Ace King Nine Four         -> 4106
          OnePair Ace King Nine Five         -> 4107
          OnePair Ace King Nine Six          -> 4108
          OnePair Ace King Nine Seven        -> 4109
          OnePair Ace King Nine Eight        -> 4110
          OnePair Ace King Ten Two           -> 4111
          OnePair Ace King Ten Three         -> 4112
          OnePair Ace King Ten Four          -> 4113
          OnePair Ace King Ten Five          -> 4114
          OnePair Ace King Ten Six           -> 4115
          OnePair Ace King Ten Seven         -> 4116
          OnePair Ace King Ten Eight         -> 4117
          OnePair Ace King Ten Nine          -> 4118
          OnePair Ace King Jack Two          -> 4119
          OnePair Ace King Jack Three        -> 4120
          OnePair Ace King Jack Four         -> 4121
          OnePair Ace King Jack Five         -> 4122
          OnePair Ace King Jack Six          -> 4123
          OnePair Ace King Jack Seven        -> 4124
          OnePair Ace King Jack Eight        -> 4125
          OnePair Ace King Jack Nine         -> 4126
          OnePair Ace King Jack Ten          -> 4127
          OnePair Ace King Queen Two         -> 4128
          OnePair Ace King Queen Three       -> 4129
          OnePair Ace King Queen Four        -> 4130
          OnePair Ace King Queen Five        -> 4131
          OnePair Ace King Queen Six         -> 4132
          OnePair Ace King Queen Seven       -> 4133
          OnePair Ace King Queen Eight       -> 4134
          OnePair Ace King Queen Nine        -> 4135
          OnePair Ace King Queen Ten         -> 4136
          OnePair Ace King Queen Jack        -> 4137
          TwoPair Three Two Four             -> 4138
          TwoPair Three Two Five             -> 4139
          TwoPair Three Two Six              -> 4140
          TwoPair Three Two Seven            -> 4141
          TwoPair Three Two Eight            -> 4142
          TwoPair Three Two Nine             -> 4143
          TwoPair Three Two Ten              -> 4144
          TwoPair Three Two Jack             -> 4145
          TwoPair Three Two Queen            -> 4146
          TwoPair Three Two King             -> 4147
          TwoPair Three Two Ace              -> 4148
          TwoPair Four Two Three             -> 4149
          TwoPair Four Two Five              -> 4150
          TwoPair Four Two Six               -> 4151
          TwoPair Four Two Seven             -> 4152
          TwoPair Four Two Eight             -> 4153
          TwoPair Four Two Nine              -> 4154
          TwoPair Four Two Ten               -> 4155
          TwoPair Four Two Jack              -> 4156
          TwoPair Four Two Queen             -> 4157
          TwoPair Four Two King              -> 4158
          TwoPair Four Two Ace               -> 4159
          TwoPair Four Three Two             -> 4160
          TwoPair Four Three Five            -> 4161
          TwoPair Four Three Six             -> 4162
          TwoPair Four Three Seven           -> 4163
          TwoPair Four Three Eight           -> 4164
          TwoPair Four Three Nine            -> 4165
          TwoPair Four Three Ten             -> 4166
          TwoPair Four Three Jack            -> 4167
          TwoPair Four Three Queen           -> 4168
          TwoPair Four Three King            -> 4169
          TwoPair Four Three Ace             -> 4170
          TwoPair Five Two Three             -> 4171
          TwoPair Five Two Four              -> 4172
          TwoPair Five Two Six               -> 4173
          TwoPair Five Two Seven             -> 4174
          TwoPair Five Two Eight             -> 4175
          TwoPair Five Two Nine              -> 4176
          TwoPair Five Two Ten               -> 4177
          TwoPair Five Two Jack              -> 4178
          TwoPair Five Two Queen             -> 4179
          TwoPair Five Two King              -> 4180
          TwoPair Five Two Ace               -> 4181
          TwoPair Five Three Two             -> 4182
          TwoPair Five Three Four            -> 4183
          TwoPair Five Three Six             -> 4184
          TwoPair Five Three Seven           -> 4185
          TwoPair Five Three Eight           -> 4186
          TwoPair Five Three Nine            -> 4187
          TwoPair Five Three Ten             -> 4188
          TwoPair Five Three Jack            -> 4189
          TwoPair Five Three Queen           -> 4190
          TwoPair Five Three King            -> 4191
          TwoPair Five Three Ace             -> 4192
          TwoPair Five Four Two              -> 4193
          TwoPair Five Four Three            -> 4194
          TwoPair Five Four Six              -> 4195
          TwoPair Five Four Seven            -> 4196
          TwoPair Five Four Eight            -> 4197
          TwoPair Five Four Nine             -> 4198
          TwoPair Five Four Ten              -> 4199
          TwoPair Five Four Jack             -> 4200
          TwoPair Five Four Queen            -> 4201
          TwoPair Five Four King             -> 4202
          TwoPair Five Four Ace              -> 4203
          TwoPair Six Two Three              -> 4204
          TwoPair Six Two Four               -> 4205
          TwoPair Six Two Five               -> 4206
          TwoPair Six Two Seven              -> 4207
          TwoPair Six Two Eight              -> 4208
          TwoPair Six Two Nine               -> 4209
          TwoPair Six Two Ten                -> 4210
          TwoPair Six Two Jack               -> 4211
          TwoPair Six Two Queen              -> 4212
          TwoPair Six Two King               -> 4213
          TwoPair Six Two Ace                -> 4214
          TwoPair Six Three Two              -> 4215
          TwoPair Six Three Four             -> 4216
          TwoPair Six Three Five             -> 4217
          TwoPair Six Three Seven            -> 4218
          TwoPair Six Three Eight            -> 4219
          TwoPair Six Three Nine             -> 4220
          TwoPair Six Three Ten              -> 4221
          TwoPair Six Three Jack             -> 4222
          TwoPair Six Three Queen            -> 4223
          TwoPair Six Three King             -> 4224
          TwoPair Six Three Ace              -> 4225
          TwoPair Six Four Two               -> 4226
          TwoPair Six Four Three             -> 4227
          TwoPair Six Four Five              -> 4228
          TwoPair Six Four Seven             -> 4229
          TwoPair Six Four Eight             -> 4230
          TwoPair Six Four Nine              -> 4231
          TwoPair Six Four Ten               -> 4232
          TwoPair Six Four Jack              -> 4233
          TwoPair Six Four Queen             -> 4234
          TwoPair Six Four King              -> 4235
          TwoPair Six Four Ace               -> 4236
          TwoPair Six Five Two               -> 4237
          TwoPair Six Five Three             -> 4238
          TwoPair Six Five Four              -> 4239
          TwoPair Six Five Seven             -> 4240
          TwoPair Six Five Eight             -> 4241
          TwoPair Six Five Nine              -> 4242
          TwoPair Six Five Ten               -> 4243
          TwoPair Six Five Jack              -> 4244
          TwoPair Six Five Queen             -> 4245
          TwoPair Six Five King              -> 4246
          TwoPair Six Five Ace               -> 4247
          TwoPair Seven Two Three            -> 4248
          TwoPair Seven Two Four             -> 4249
          TwoPair Seven Two Five             -> 4250
          TwoPair Seven Two Six              -> 4251
          TwoPair Seven Two Eight            -> 4252
          TwoPair Seven Two Nine             -> 4253
          TwoPair Seven Two Ten              -> 4254
          TwoPair Seven Two Jack             -> 4255
          TwoPair Seven Two Queen            -> 4256
          TwoPair Seven Two King             -> 4257
          TwoPair Seven Two Ace              -> 4258
          TwoPair Seven Three Two            -> 4259
          TwoPair Seven Three Four           -> 4260
          TwoPair Seven Three Five           -> 4261
          TwoPair Seven Three Six            -> 4262
          TwoPair Seven Three Eight          -> 4263
          TwoPair Seven Three Nine           -> 4264
          TwoPair Seven Three Ten            -> 4265
          TwoPair Seven Three Jack           -> 4266
          TwoPair Seven Three Queen          -> 4267
          TwoPair Seven Three King           -> 4268
          TwoPair Seven Three Ace            -> 4269
          TwoPair Seven Four Two             -> 4270
          TwoPair Seven Four Three           -> 4271
          TwoPair Seven Four Five            -> 4272
          TwoPair Seven Four Six             -> 4273
          TwoPair Seven Four Eight           -> 4274
          TwoPair Seven Four Nine            -> 4275
          TwoPair Seven Four Ten             -> 4276
          TwoPair Seven Four Jack            -> 4277
          TwoPair Seven Four Queen           -> 4278
          TwoPair Seven Four King            -> 4279
          TwoPair Seven Four Ace             -> 4280
          TwoPair Seven Five Two             -> 4281
          TwoPair Seven Five Three           -> 4282
          TwoPair Seven Five Four            -> 4283
          TwoPair Seven Five Six             -> 4284
          TwoPair Seven Five Eight           -> 4285
          TwoPair Seven Five Nine            -> 4286
          TwoPair Seven Five Ten             -> 4287
          TwoPair Seven Five Jack            -> 4288
          TwoPair Seven Five Queen           -> 4289
          TwoPair Seven Five King            -> 4290
          TwoPair Seven Five Ace             -> 4291
          TwoPair Seven Six Two              -> 4292
          TwoPair Seven Six Three            -> 4293
          TwoPair Seven Six Four             -> 4294
          TwoPair Seven Six Five             -> 4295
          TwoPair Seven Six Eight            -> 4296
          TwoPair Seven Six Nine             -> 4297
          TwoPair Seven Six Ten              -> 4298
          TwoPair Seven Six Jack             -> 4299
          TwoPair Seven Six Queen            -> 4300
          TwoPair Seven Six King             -> 4301
          TwoPair Seven Six Ace              -> 4302
          TwoPair Eight Two Three            -> 4303
          TwoPair Eight Two Four             -> 4304
          TwoPair Eight Two Five             -> 4305
          TwoPair Eight Two Six              -> 4306
          TwoPair Eight Two Seven            -> 4307
          TwoPair Eight Two Nine             -> 4308
          TwoPair Eight Two Ten              -> 4309
          TwoPair Eight Two Jack             -> 4310
          TwoPair Eight Two Queen            -> 4311
          TwoPair Eight Two King             -> 4312
          TwoPair Eight Two Ace              -> 4313
          TwoPair Eight Three Two            -> 4314
          TwoPair Eight Three Four           -> 4315
          TwoPair Eight Three Five           -> 4316
          TwoPair Eight Three Six            -> 4317
          TwoPair Eight Three Seven          -> 4318
          TwoPair Eight Three Nine           -> 4319
          TwoPair Eight Three Ten            -> 4320
          TwoPair Eight Three Jack           -> 4321
          TwoPair Eight Three Queen          -> 4322
          TwoPair Eight Three King           -> 4323
          TwoPair Eight Three Ace            -> 4324
          TwoPair Eight Four Two             -> 4325
          TwoPair Eight Four Three           -> 4326
          TwoPair Eight Four Five            -> 4327
          TwoPair Eight Four Six             -> 4328
          TwoPair Eight Four Seven           -> 4329
          TwoPair Eight Four Nine            -> 4330
          TwoPair Eight Four Ten             -> 4331
          TwoPair Eight Four Jack            -> 4332
          TwoPair Eight Four Queen           -> 4333
          TwoPair Eight Four King            -> 4334
          TwoPair Eight Four Ace             -> 4335
          TwoPair Eight Five Two             -> 4336
          TwoPair Eight Five Three           -> 4337
          TwoPair Eight Five Four            -> 4338
          TwoPair Eight Five Six             -> 4339
          TwoPair Eight Five Seven           -> 4340
          TwoPair Eight Five Nine            -> 4341
          TwoPair Eight Five Ten             -> 4342
          TwoPair Eight Five Jack            -> 4343
          TwoPair Eight Five Queen           -> 4344
          TwoPair Eight Five King            -> 4345
          TwoPair Eight Five Ace             -> 4346
          TwoPair Eight Six Two              -> 4347
          TwoPair Eight Six Three            -> 4348
          TwoPair Eight Six Four             -> 4349
          TwoPair Eight Six Five             -> 4350
          TwoPair Eight Six Seven            -> 4351
          TwoPair Eight Six Nine             -> 4352
          TwoPair Eight Six Ten              -> 4353
          TwoPair Eight Six Jack             -> 4354
          TwoPair Eight Six Queen            -> 4355
          TwoPair Eight Six King             -> 4356
          TwoPair Eight Six Ace              -> 4357
          TwoPair Eight Seven Two            -> 4358
          TwoPair Eight Seven Three          -> 4359
          TwoPair Eight Seven Four           -> 4360
          TwoPair Eight Seven Five           -> 4361
          TwoPair Eight Seven Six            -> 4362
          TwoPair Eight Seven Nine           -> 4363
          TwoPair Eight Seven Ten            -> 4364
          TwoPair Eight Seven Jack           -> 4365
          TwoPair Eight Seven Queen          -> 4366
          TwoPair Eight Seven King           -> 4367
          TwoPair Eight Seven Ace            -> 4368
          TwoPair Nine Two Three             -> 4369
          TwoPair Nine Two Four              -> 4370
          TwoPair Nine Two Five              -> 4371
          TwoPair Nine Two Six               -> 4372
          TwoPair Nine Two Seven             -> 4373
          TwoPair Nine Two Eight             -> 4374
          TwoPair Nine Two Ten               -> 4375
          TwoPair Nine Two Jack              -> 4376
          TwoPair Nine Two Queen             -> 4377
          TwoPair Nine Two King              -> 4378
          TwoPair Nine Two Ace               -> 4379
          TwoPair Nine Three Two             -> 4380
          TwoPair Nine Three Four            -> 4381
          TwoPair Nine Three Five            -> 4382
          TwoPair Nine Three Six             -> 4383
          TwoPair Nine Three Seven           -> 4384
          TwoPair Nine Three Eight           -> 4385
          TwoPair Nine Three Ten             -> 4386
          TwoPair Nine Three Jack            -> 4387
          TwoPair Nine Three Queen           -> 4388
          TwoPair Nine Three King            -> 4389
          TwoPair Nine Three Ace             -> 4390
          TwoPair Nine Four Two              -> 4391
          TwoPair Nine Four Three            -> 4392
          TwoPair Nine Four Five             -> 4393
          TwoPair Nine Four Six              -> 4394
          TwoPair Nine Four Seven            -> 4395
          TwoPair Nine Four Eight            -> 4396
          TwoPair Nine Four Ten              -> 4397
          TwoPair Nine Four Jack             -> 4398
          TwoPair Nine Four Queen            -> 4399
          TwoPair Nine Four King             -> 4400
          TwoPair Nine Four Ace              -> 4401
          TwoPair Nine Five Two              -> 4402
          TwoPair Nine Five Three            -> 4403
          TwoPair Nine Five Four             -> 4404
          TwoPair Nine Five Six              -> 4405
          TwoPair Nine Five Seven            -> 4406
          TwoPair Nine Five Eight            -> 4407
          TwoPair Nine Five Ten              -> 4408
          TwoPair Nine Five Jack             -> 4409
          TwoPair Nine Five Queen            -> 4410
          TwoPair Nine Five King             -> 4411
          TwoPair Nine Five Ace              -> 4412
          TwoPair Nine Six Two               -> 4413
          TwoPair Nine Six Three             -> 4414
          TwoPair Nine Six Four              -> 4415
          TwoPair Nine Six Five              -> 4416
          TwoPair Nine Six Seven             -> 4417
          TwoPair Nine Six Eight             -> 4418
          TwoPair Nine Six Ten               -> 4419
          TwoPair Nine Six Jack              -> 4420
          TwoPair Nine Six Queen             -> 4421
          TwoPair Nine Six King              -> 4422
          TwoPair Nine Six Ace               -> 4423
          TwoPair Nine Seven Two             -> 4424
          TwoPair Nine Seven Three           -> 4425
          TwoPair Nine Seven Four            -> 4426
          TwoPair Nine Seven Five            -> 4427
          TwoPair Nine Seven Six             -> 4428
          TwoPair Nine Seven Eight           -> 4429
          TwoPair Nine Seven Ten             -> 4430
          TwoPair Nine Seven Jack            -> 4431
          TwoPair Nine Seven Queen           -> 4432
          TwoPair Nine Seven King            -> 4433
          TwoPair Nine Seven Ace             -> 4434
          TwoPair Nine Eight Two             -> 4435
          TwoPair Nine Eight Three           -> 4436
          TwoPair Nine Eight Four            -> 4437
          TwoPair Nine Eight Five            -> 4438
          TwoPair Nine Eight Six             -> 4439
          TwoPair Nine Eight Seven           -> 4440
          TwoPair Nine Eight Ten             -> 4441
          TwoPair Nine Eight Jack            -> 4442
          TwoPair Nine Eight Queen           -> 4443
          TwoPair Nine Eight King            -> 4444
          TwoPair Nine Eight Ace             -> 4445
          TwoPair Ten Two Three              -> 4446
          TwoPair Ten Two Four               -> 4447
          TwoPair Ten Two Five               -> 4448
          TwoPair Ten Two Six                -> 4449
          TwoPair Ten Two Seven              -> 4450
          TwoPair Ten Two Eight              -> 4451
          TwoPair Ten Two Nine               -> 4452
          TwoPair Ten Two Jack               -> 4453
          TwoPair Ten Two Queen              -> 4454
          TwoPair Ten Two King               -> 4455
          TwoPair Ten Two Ace                -> 4456
          TwoPair Ten Three Two              -> 4457
          TwoPair Ten Three Four             -> 4458
          TwoPair Ten Three Five             -> 4459
          TwoPair Ten Three Six              -> 4460
          TwoPair Ten Three Seven            -> 4461
          TwoPair Ten Three Eight            -> 4462
          TwoPair Ten Three Nine             -> 4463
          TwoPair Ten Three Jack             -> 4464
          TwoPair Ten Three Queen            -> 4465
          TwoPair Ten Three King             -> 4466
          TwoPair Ten Three Ace              -> 4467
          TwoPair Ten Four Two               -> 4468
          TwoPair Ten Four Three             -> 4469
          TwoPair Ten Four Five              -> 4470
          TwoPair Ten Four Six               -> 4471
          TwoPair Ten Four Seven             -> 4472
          TwoPair Ten Four Eight             -> 4473
          TwoPair Ten Four Nine              -> 4474
          TwoPair Ten Four Jack              -> 4475
          TwoPair Ten Four Queen             -> 4476
          TwoPair Ten Four King              -> 4477
          TwoPair Ten Four Ace               -> 4478
          TwoPair Ten Five Two               -> 4479
          TwoPair Ten Five Three             -> 4480
          TwoPair Ten Five Four              -> 4481
          TwoPair Ten Five Six               -> 4482
          TwoPair Ten Five Seven             -> 4483
          TwoPair Ten Five Eight             -> 4484
          TwoPair Ten Five Nine              -> 4485
          TwoPair Ten Five Jack              -> 4486
          TwoPair Ten Five Queen             -> 4487
          TwoPair Ten Five King              -> 4488
          TwoPair Ten Five Ace               -> 4489
          TwoPair Ten Six Two                -> 4490
          TwoPair Ten Six Three              -> 4491
          TwoPair Ten Six Four               -> 4492
          TwoPair Ten Six Five               -> 4493
          TwoPair Ten Six Seven              -> 4494
          TwoPair Ten Six Eight              -> 4495
          TwoPair Ten Six Nine               -> 4496
          TwoPair Ten Six Jack               -> 4497
          TwoPair Ten Six Queen              -> 4498
          TwoPair Ten Six King               -> 4499
          TwoPair Ten Six Ace                -> 4500
          TwoPair Ten Seven Two              -> 4501
          TwoPair Ten Seven Three            -> 4502
          TwoPair Ten Seven Four             -> 4503
          TwoPair Ten Seven Five             -> 4504
          TwoPair Ten Seven Six              -> 4505
          TwoPair Ten Seven Eight            -> 4506
          TwoPair Ten Seven Nine             -> 4507
          TwoPair Ten Seven Jack             -> 4508
          TwoPair Ten Seven Queen            -> 4509
          TwoPair Ten Seven King             -> 4510
          TwoPair Ten Seven Ace              -> 4511
          TwoPair Ten Eight Two              -> 4512
          TwoPair Ten Eight Three            -> 4513
          TwoPair Ten Eight Four             -> 4514
          TwoPair Ten Eight Five             -> 4515
          TwoPair Ten Eight Six              -> 4516
          TwoPair Ten Eight Seven            -> 4517
          TwoPair Ten Eight Nine             -> 4518
          TwoPair Ten Eight Jack             -> 4519
          TwoPair Ten Eight Queen            -> 4520
          TwoPair Ten Eight King             -> 4521
          TwoPair Ten Eight Ace              -> 4522
          TwoPair Ten Nine Two               -> 4523
          TwoPair Ten Nine Three             -> 4524
          TwoPair Ten Nine Four              -> 4525
          TwoPair Ten Nine Five              -> 4526
          TwoPair Ten Nine Six               -> 4527
          TwoPair Ten Nine Seven             -> 4528
          TwoPair Ten Nine Eight             -> 4529
          TwoPair Ten Nine Jack              -> 4530
          TwoPair Ten Nine Queen             -> 4531
          TwoPair Ten Nine King              -> 4532
          TwoPair Ten Nine Ace               -> 4533
          TwoPair Jack Two Three             -> 4534
          TwoPair Jack Two Four              -> 4535
          TwoPair Jack Two Five              -> 4536
          TwoPair Jack Two Six               -> 4537
          TwoPair Jack Two Seven             -> 4538
          TwoPair Jack Two Eight             -> 4539
          TwoPair Jack Two Nine              -> 4540
          TwoPair Jack Two Ten               -> 4541
          TwoPair Jack Two Queen             -> 4542
          TwoPair Jack Two King              -> 4543
          TwoPair Jack Two Ace               -> 4544
          TwoPair Jack Three Two             -> 4545
          TwoPair Jack Three Four            -> 4546
          TwoPair Jack Three Five            -> 4547
          TwoPair Jack Three Six             -> 4548
          TwoPair Jack Three Seven           -> 4549
          TwoPair Jack Three Eight           -> 4550
          TwoPair Jack Three Nine            -> 4551
          TwoPair Jack Three Ten             -> 4552
          TwoPair Jack Three Queen           -> 4553
          TwoPair Jack Three King            -> 4554
          TwoPair Jack Three Ace             -> 4555
          TwoPair Jack Four Two              -> 4556
          TwoPair Jack Four Three            -> 4557
          TwoPair Jack Four Five             -> 4558
          TwoPair Jack Four Six              -> 4559
          TwoPair Jack Four Seven            -> 4560
          TwoPair Jack Four Eight            -> 4561
          TwoPair Jack Four Nine             -> 4562
          TwoPair Jack Four Ten              -> 4563
          TwoPair Jack Four Queen            -> 4564
          TwoPair Jack Four King             -> 4565
          TwoPair Jack Four Ace              -> 4566
          TwoPair Jack Five Two              -> 4567
          TwoPair Jack Five Three            -> 4568
          TwoPair Jack Five Four             -> 4569
          TwoPair Jack Five Six              -> 4570
          TwoPair Jack Five Seven            -> 4571
          TwoPair Jack Five Eight            -> 4572
          TwoPair Jack Five Nine             -> 4573
          TwoPair Jack Five Ten              -> 4574
          TwoPair Jack Five Queen            -> 4575
          TwoPair Jack Five King             -> 4576
          TwoPair Jack Five Ace              -> 4577
          TwoPair Jack Six Two               -> 4578
          TwoPair Jack Six Three             -> 4579
          TwoPair Jack Six Four              -> 4580
          TwoPair Jack Six Five              -> 4581
          TwoPair Jack Six Seven             -> 4582
          TwoPair Jack Six Eight             -> 4583
          TwoPair Jack Six Nine              -> 4584
          TwoPair Jack Six Ten               -> 4585
          TwoPair Jack Six Queen             -> 4586
          TwoPair Jack Six King              -> 4587
          TwoPair Jack Six Ace               -> 4588
          TwoPair Jack Seven Two             -> 4589
          TwoPair Jack Seven Three           -> 4590
          TwoPair Jack Seven Four            -> 4591
          TwoPair Jack Seven Five            -> 4592
          TwoPair Jack Seven Six             -> 4593
          TwoPair Jack Seven Eight           -> 4594
          TwoPair Jack Seven Nine            -> 4595
          TwoPair Jack Seven Ten             -> 4596
          TwoPair Jack Seven Queen           -> 4597
          TwoPair Jack Seven King            -> 4598
          TwoPair Jack Seven Ace             -> 4599
          TwoPair Jack Eight Two             -> 4600
          TwoPair Jack Eight Three           -> 4601
          TwoPair Jack Eight Four            -> 4602
          TwoPair Jack Eight Five            -> 4603
          TwoPair Jack Eight Six             -> 4604
          TwoPair Jack Eight Seven           -> 4605
          TwoPair Jack Eight Nine            -> 4606
          TwoPair Jack Eight Ten             -> 4607
          TwoPair Jack Eight Queen           -> 4608
          TwoPair Jack Eight King            -> 4609
          TwoPair Jack Eight Ace             -> 4610
          TwoPair Jack Nine Two              -> 4611
          TwoPair Jack Nine Three            -> 4612
          TwoPair Jack Nine Four             -> 4613
          TwoPair Jack Nine Five             -> 4614
          TwoPair Jack Nine Six              -> 4615
          TwoPair Jack Nine Seven            -> 4616
          TwoPair Jack Nine Eight            -> 4617
          TwoPair Jack Nine Ten              -> 4618
          TwoPair Jack Nine Queen            -> 4619
          TwoPair Jack Nine King             -> 4620
          TwoPair Jack Nine Ace              -> 4621
          TwoPair Jack Ten Two               -> 4622
          TwoPair Jack Ten Three             -> 4623
          TwoPair Jack Ten Four              -> 4624
          TwoPair Jack Ten Five              -> 4625
          TwoPair Jack Ten Six               -> 4626
          TwoPair Jack Ten Seven             -> 4627
          TwoPair Jack Ten Eight             -> 4628
          TwoPair Jack Ten Nine              -> 4629
          TwoPair Jack Ten Queen             -> 4630
          TwoPair Jack Ten King              -> 4631
          TwoPair Jack Ten Ace               -> 4632
          TwoPair Queen Two Three            -> 4633
          TwoPair Queen Two Four             -> 4634
          TwoPair Queen Two Five             -> 4635
          TwoPair Queen Two Six              -> 4636
          TwoPair Queen Two Seven            -> 4637
          TwoPair Queen Two Eight            -> 4638
          TwoPair Queen Two Nine             -> 4639
          TwoPair Queen Two Ten              -> 4640
          TwoPair Queen Two Jack             -> 4641
          TwoPair Queen Two King             -> 4642
          TwoPair Queen Two Ace              -> 4643
          TwoPair Queen Three Two            -> 4644
          TwoPair Queen Three Four           -> 4645
          TwoPair Queen Three Five           -> 4646
          TwoPair Queen Three Six            -> 4647
          TwoPair Queen Three Seven          -> 4648
          TwoPair Queen Three Eight          -> 4649
          TwoPair Queen Three Nine           -> 4650
          TwoPair Queen Three Ten            -> 4651
          TwoPair Queen Three Jack           -> 4652
          TwoPair Queen Three King           -> 4653
          TwoPair Queen Three Ace            -> 4654
          TwoPair Queen Four Two             -> 4655
          TwoPair Queen Four Three           -> 4656
          TwoPair Queen Four Five            -> 4657
          TwoPair Queen Four Six             -> 4658
          TwoPair Queen Four Seven           -> 4659
          TwoPair Queen Four Eight           -> 4660
          TwoPair Queen Four Nine            -> 4661
          TwoPair Queen Four Ten             -> 4662
          TwoPair Queen Four Jack            -> 4663
          TwoPair Queen Four King            -> 4664
          TwoPair Queen Four Ace             -> 4665
          TwoPair Queen Five Two             -> 4666
          TwoPair Queen Five Three           -> 4667
          TwoPair Queen Five Four            -> 4668
          TwoPair Queen Five Six             -> 4669
          TwoPair Queen Five Seven           -> 4670
          TwoPair Queen Five Eight           -> 4671
          TwoPair Queen Five Nine            -> 4672
          TwoPair Queen Five Ten             -> 4673
          TwoPair Queen Five Jack            -> 4674
          TwoPair Queen Five King            -> 4675
          TwoPair Queen Five Ace             -> 4676
          TwoPair Queen Six Two              -> 4677
          TwoPair Queen Six Three            -> 4678
          TwoPair Queen Six Four             -> 4679
          TwoPair Queen Six Five             -> 4680
          TwoPair Queen Six Seven            -> 4681
          TwoPair Queen Six Eight            -> 4682
          TwoPair Queen Six Nine             -> 4683
          TwoPair Queen Six Ten              -> 4684
          TwoPair Queen Six Jack             -> 4685
          TwoPair Queen Six King             -> 4686
          TwoPair Queen Six Ace              -> 4687
          TwoPair Queen Seven Two            -> 4688
          TwoPair Queen Seven Three          -> 4689
          TwoPair Queen Seven Four           -> 4690
          TwoPair Queen Seven Five           -> 4691
          TwoPair Queen Seven Six            -> 4692
          TwoPair Queen Seven Eight          -> 4693
          TwoPair Queen Seven Nine           -> 4694
          TwoPair Queen Seven Ten            -> 4695
          TwoPair Queen Seven Jack           -> 4696
          TwoPair Queen Seven King           -> 4697
          TwoPair Queen Seven Ace            -> 4698
          TwoPair Queen Eight Two            -> 4699
          TwoPair Queen Eight Three          -> 4700
          TwoPair Queen Eight Four           -> 4701
          TwoPair Queen Eight Five           -> 4702
          TwoPair Queen Eight Six            -> 4703
          TwoPair Queen Eight Seven          -> 4704
          TwoPair Queen Eight Nine           -> 4705
          TwoPair Queen Eight Ten            -> 4706
          TwoPair Queen Eight Jack           -> 4707
          TwoPair Queen Eight King           -> 4708
          TwoPair Queen Eight Ace            -> 4709
          TwoPair Queen Nine Two             -> 4710
          TwoPair Queen Nine Three           -> 4711
          TwoPair Queen Nine Four            -> 4712
          TwoPair Queen Nine Five            -> 4713
          TwoPair Queen Nine Six             -> 4714
          TwoPair Queen Nine Seven           -> 4715
          TwoPair Queen Nine Eight           -> 4716
          TwoPair Queen Nine Ten             -> 4717
          TwoPair Queen Nine Jack            -> 4718
          TwoPair Queen Nine King            -> 4719
          TwoPair Queen Nine Ace             -> 4720
          TwoPair Queen Ten Two              -> 4721
          TwoPair Queen Ten Three            -> 4722
          TwoPair Queen Ten Four             -> 4723
          TwoPair Queen Ten Five             -> 4724
          TwoPair Queen Ten Six              -> 4725
          TwoPair Queen Ten Seven            -> 4726
          TwoPair Queen Ten Eight            -> 4727
          TwoPair Queen Ten Nine             -> 4728
          TwoPair Queen Ten Jack             -> 4729
          TwoPair Queen Ten King             -> 4730
          TwoPair Queen Ten Ace              -> 4731
          TwoPair Queen Jack Two             -> 4732
          TwoPair Queen Jack Three           -> 4733
          TwoPair Queen Jack Four            -> 4734
          TwoPair Queen Jack Five            -> 4735
          TwoPair Queen Jack Six             -> 4736
          TwoPair Queen Jack Seven           -> 4737
          TwoPair Queen Jack Eight           -> 4738
          TwoPair Queen Jack Nine            -> 4739
          TwoPair Queen Jack Ten             -> 4740
          TwoPair Queen Jack King            -> 4741
          TwoPair Queen Jack Ace             -> 4742
          TwoPair King Two Three             -> 4743
          TwoPair King Two Four              -> 4744
          TwoPair King Two Five              -> 4745
          TwoPair King Two Six               -> 4746
          TwoPair King Two Seven             -> 4747
          TwoPair King Two Eight             -> 4748
          TwoPair King Two Nine              -> 4749
          TwoPair King Two Ten               -> 4750
          TwoPair King Two Jack              -> 4751
          TwoPair King Two Queen             -> 4752
          TwoPair King Two Ace               -> 4753
          TwoPair King Three Two             -> 4754
          TwoPair King Three Four            -> 4755
          TwoPair King Three Five            -> 4756
          TwoPair King Three Six             -> 4757
          TwoPair King Three Seven           -> 4758
          TwoPair King Three Eight           -> 4759
          TwoPair King Three Nine            -> 4760
          TwoPair King Three Ten             -> 4761
          TwoPair King Three Jack            -> 4762
          TwoPair King Three Queen           -> 4763
          TwoPair King Three Ace             -> 4764
          TwoPair King Four Two              -> 4765
          TwoPair King Four Three            -> 4766
          TwoPair King Four Five             -> 4767
          TwoPair King Four Six              -> 4768
          TwoPair King Four Seven            -> 4769
          TwoPair King Four Eight            -> 4770
          TwoPair King Four Nine             -> 4771
          TwoPair King Four Ten              -> 4772
          TwoPair King Four Jack             -> 4773
          TwoPair King Four Queen            -> 4774
          TwoPair King Four Ace              -> 4775
          TwoPair King Five Two              -> 4776
          TwoPair King Five Three            -> 4777
          TwoPair King Five Four             -> 4778
          TwoPair King Five Six              -> 4779
          TwoPair King Five Seven            -> 4780
          TwoPair King Five Eight            -> 4781
          TwoPair King Five Nine             -> 4782
          TwoPair King Five Ten              -> 4783
          TwoPair King Five Jack             -> 4784
          TwoPair King Five Queen            -> 4785
          TwoPair King Five Ace              -> 4786
          TwoPair King Six Two               -> 4787
          TwoPair King Six Three             -> 4788
          TwoPair King Six Four              -> 4789
          TwoPair King Six Five              -> 4790
          TwoPair King Six Seven             -> 4791
          TwoPair King Six Eight             -> 4792
          TwoPair King Six Nine              -> 4793
          TwoPair King Six Ten               -> 4794
          TwoPair King Six Jack              -> 4795
          TwoPair King Six Queen             -> 4796
          TwoPair King Six Ace               -> 4797
          TwoPair King Seven Two             -> 4798
          TwoPair King Seven Three           -> 4799
          TwoPair King Seven Four            -> 4800
          TwoPair King Seven Five            -> 4801
          TwoPair King Seven Six             -> 4802
          TwoPair King Seven Eight           -> 4803
          TwoPair King Seven Nine            -> 4804
          TwoPair King Seven Ten             -> 4805
          TwoPair King Seven Jack            -> 4806
          TwoPair King Seven Queen           -> 4807
          TwoPair King Seven Ace             -> 4808
          TwoPair King Eight Two             -> 4809
          TwoPair King Eight Three           -> 4810
          TwoPair King Eight Four            -> 4811
          TwoPair King Eight Five            -> 4812
          TwoPair King Eight Six             -> 4813
          TwoPair King Eight Seven           -> 4814
          TwoPair King Eight Nine            -> 4815
          TwoPair King Eight Ten             -> 4816
          TwoPair King Eight Jack            -> 4817
          TwoPair King Eight Queen           -> 4818
          TwoPair King Eight Ace             -> 4819
          TwoPair King Nine Two              -> 4820
          TwoPair King Nine Three            -> 4821
          TwoPair King Nine Four             -> 4822
          TwoPair King Nine Five             -> 4823
          TwoPair King Nine Six              -> 4824
          TwoPair King Nine Seven            -> 4825
          TwoPair King Nine Eight            -> 4826
          TwoPair King Nine Ten              -> 4827
          TwoPair King Nine Jack             -> 4828
          TwoPair King Nine Queen            -> 4829
          TwoPair King Nine Ace              -> 4830
          TwoPair King Ten Two               -> 4831
          TwoPair King Ten Three             -> 4832
          TwoPair King Ten Four              -> 4833
          TwoPair King Ten Five              -> 4834
          TwoPair King Ten Six               -> 4835
          TwoPair King Ten Seven             -> 4836
          TwoPair King Ten Eight             -> 4837
          TwoPair King Ten Nine              -> 4838
          TwoPair King Ten Jack              -> 4839
          TwoPair King Ten Queen             -> 4840
          TwoPair King Ten Ace               -> 4841
          TwoPair King Jack Two              -> 4842
          TwoPair King Jack Three            -> 4843
          TwoPair King Jack Four             -> 4844
          TwoPair King Jack Five             -> 4845
          TwoPair King Jack Six              -> 4846
          TwoPair King Jack Seven            -> 4847
          TwoPair King Jack Eight            -> 4848
          TwoPair King Jack Nine             -> 4849
          TwoPair King Jack Ten              -> 4850
          TwoPair King Jack Queen            -> 4851
          TwoPair King Jack Ace              -> 4852
          TwoPair King Queen Two             -> 4853
          TwoPair King Queen Three           -> 4854
          TwoPair King Queen Four            -> 4855
          TwoPair King Queen Five            -> 4856
          TwoPair King Queen Six             -> 4857
          TwoPair King Queen Seven           -> 4858
          TwoPair King Queen Eight           -> 4859
          TwoPair King Queen Nine            -> 4860
          TwoPair King Queen Ten             -> 4861
          TwoPair King Queen Jack            -> 4862
          TwoPair King Queen Ace             -> 4863
          TwoPair Ace Two Three              -> 4864
          TwoPair Ace Two Four               -> 4865
          TwoPair Ace Two Five               -> 4866
          TwoPair Ace Two Six                -> 4867
          TwoPair Ace Two Seven              -> 4868
          TwoPair Ace Two Eight              -> 4869
          TwoPair Ace Two Nine               -> 4870
          TwoPair Ace Two Ten                -> 4871
          TwoPair Ace Two Jack               -> 4872
          TwoPair Ace Two Queen              -> 4873
          TwoPair Ace Two King               -> 4874
          TwoPair Ace Three Two              -> 4875
          TwoPair Ace Three Four             -> 4876
          TwoPair Ace Three Five             -> 4877
          TwoPair Ace Three Six              -> 4878
          TwoPair Ace Three Seven            -> 4879
          TwoPair Ace Three Eight            -> 4880
          TwoPair Ace Three Nine             -> 4881
          TwoPair Ace Three Ten              -> 4882
          TwoPair Ace Three Jack             -> 4883
          TwoPair Ace Three Queen            -> 4884
          TwoPair Ace Three King             -> 4885
          TwoPair Ace Four Two               -> 4886
          TwoPair Ace Four Three             -> 4887
          TwoPair Ace Four Five              -> 4888
          TwoPair Ace Four Six               -> 4889
          TwoPair Ace Four Seven             -> 4890
          TwoPair Ace Four Eight             -> 4891
          TwoPair Ace Four Nine              -> 4892
          TwoPair Ace Four Ten               -> 4893
          TwoPair Ace Four Jack              -> 4894
          TwoPair Ace Four Queen             -> 4895
          TwoPair Ace Four King              -> 4896
          TwoPair Ace Five Two               -> 4897
          TwoPair Ace Five Three             -> 4898
          TwoPair Ace Five Four              -> 4899
          TwoPair Ace Five Six               -> 4900
          TwoPair Ace Five Seven             -> 4901
          TwoPair Ace Five Eight             -> 4902
          TwoPair Ace Five Nine              -> 4903
          TwoPair Ace Five Ten               -> 4904
          TwoPair Ace Five Jack              -> 4905
          TwoPair Ace Five Queen             -> 4906
          TwoPair Ace Five King              -> 4907
          TwoPair Ace Six Two                -> 4908
          TwoPair Ace Six Three              -> 4909
          TwoPair Ace Six Four               -> 4910
          TwoPair Ace Six Five               -> 4911
          TwoPair Ace Six Seven              -> 4912
          TwoPair Ace Six Eight              -> 4913
          TwoPair Ace Six Nine               -> 4914
          TwoPair Ace Six Ten                -> 4915
          TwoPair Ace Six Jack               -> 4916
          TwoPair Ace Six Queen              -> 4917
          TwoPair Ace Six King               -> 4918
          TwoPair Ace Seven Two              -> 4919
          TwoPair Ace Seven Three            -> 4920
          TwoPair Ace Seven Four             -> 4921
          TwoPair Ace Seven Five             -> 4922
          TwoPair Ace Seven Six              -> 4923
          TwoPair Ace Seven Eight            -> 4924
          TwoPair Ace Seven Nine             -> 4925
          TwoPair Ace Seven Ten              -> 4926
          TwoPair Ace Seven Jack             -> 4927
          TwoPair Ace Seven Queen            -> 4928
          TwoPair Ace Seven King             -> 4929
          TwoPair Ace Eight Two              -> 4930
          TwoPair Ace Eight Three            -> 4931
          TwoPair Ace Eight Four             -> 4932
          TwoPair Ace Eight Five             -> 4933
          TwoPair Ace Eight Six              -> 4934
          TwoPair Ace Eight Seven            -> 4935
          TwoPair Ace Eight Nine             -> 4936
          TwoPair Ace Eight Ten              -> 4937
          TwoPair Ace Eight Jack             -> 4938
          TwoPair Ace Eight Queen            -> 4939
          TwoPair Ace Eight King             -> 4940
          TwoPair Ace Nine Two               -> 4941
          TwoPair Ace Nine Three             -> 4942
          TwoPair Ace Nine Four              -> 4943
          TwoPair Ace Nine Five              -> 4944
          TwoPair Ace Nine Six               -> 4945
          TwoPair Ace Nine Seven             -> 4946
          TwoPair Ace Nine Eight             -> 4947
          TwoPair Ace Nine Ten               -> 4948
          TwoPair Ace Nine Jack              -> 4949
          TwoPair Ace Nine Queen             -> 4950
          TwoPair Ace Nine King              -> 4951
          TwoPair Ace Ten Two                -> 4952
          TwoPair Ace Ten Three              -> 4953
          TwoPair Ace Ten Four               -> 4954
          TwoPair Ace Ten Five               -> 4955
          TwoPair Ace Ten Six                -> 4956
          TwoPair Ace Ten Seven              -> 4957
          TwoPair Ace Ten Eight              -> 4958
          TwoPair Ace Ten Nine               -> 4959
          TwoPair Ace Ten Jack               -> 4960
          TwoPair Ace Ten Queen              -> 4961
          TwoPair Ace Ten King               -> 4962
          TwoPair Ace Jack Two               -> 4963
          TwoPair Ace Jack Three             -> 4964
          TwoPair Ace Jack Four              -> 4965
          TwoPair Ace Jack Five              -> 4966
          TwoPair Ace Jack Six               -> 4967
          TwoPair Ace Jack Seven             -> 4968
          TwoPair Ace Jack Eight             -> 4969
          TwoPair Ace Jack Nine              -> 4970
          TwoPair Ace Jack Ten               -> 4971
          TwoPair Ace Jack Queen             -> 4972
          TwoPair Ace Jack King              -> 4973
          TwoPair Ace Queen Two              -> 4974
          TwoPair Ace Queen Three            -> 4975
          TwoPair Ace Queen Four             -> 4976
          TwoPair Ace Queen Five             -> 4977
          TwoPair Ace Queen Six              -> 4978
          TwoPair Ace Queen Seven            -> 4979
          TwoPair Ace Queen Eight            -> 4980
          TwoPair Ace Queen Nine             -> 4981
          TwoPair Ace Queen Ten              -> 4982
          TwoPair Ace Queen Jack             -> 4983
          TwoPair Ace Queen King             -> 4984
          TwoPair Ace King Two               -> 4985
          TwoPair Ace King Three             -> 4986
          TwoPair Ace King Four              -> 4987
          TwoPair Ace King Five              -> 4988
          TwoPair Ace King Six               -> 4989
          TwoPair Ace King Seven             -> 4990
          TwoPair Ace King Eight             -> 4991
          TwoPair Ace King Nine              -> 4992
          TwoPair Ace King Ten               -> 4993
          TwoPair Ace King Jack              -> 4994
          TwoPair Ace King Queen             -> 4995
          ThreeOfAKind Two Four Three        -> 4996
          ThreeOfAKind Two Five Three        -> 4997
          ThreeOfAKind Two Five Four         -> 4998
          ThreeOfAKind Two Six Three         -> 4999
          ThreeOfAKind Two Six Four          -> 5000
          ThreeOfAKind Two Six Five          -> 5001
          ThreeOfAKind Two Seven Three       -> 5002
          ThreeOfAKind Two Seven Four        -> 5003
          ThreeOfAKind Two Seven Five        -> 5004
          ThreeOfAKind Two Seven Six         -> 5005
          ThreeOfAKind Two Eight Three       -> 5006
          ThreeOfAKind Two Eight Four        -> 5007
          ThreeOfAKind Two Eight Five        -> 5008
          ThreeOfAKind Two Eight Six         -> 5009
          ThreeOfAKind Two Eight Seven       -> 5010
          ThreeOfAKind Two Nine Three        -> 5011
          ThreeOfAKind Two Nine Four         -> 5012
          ThreeOfAKind Two Nine Five         -> 5013
          ThreeOfAKind Two Nine Six          -> 5014
          ThreeOfAKind Two Nine Seven        -> 5015
          ThreeOfAKind Two Nine Eight        -> 5016
          ThreeOfAKind Two Ten Three         -> 5017
          ThreeOfAKind Two Ten Four          -> 5018
          ThreeOfAKind Two Ten Five          -> 5019
          ThreeOfAKind Two Ten Six           -> 5020
          ThreeOfAKind Two Ten Seven         -> 5021
          ThreeOfAKind Two Ten Eight         -> 5022
          ThreeOfAKind Two Ten Nine          -> 5023
          ThreeOfAKind Two Jack Three        -> 5024
          ThreeOfAKind Two Jack Four         -> 5025
          ThreeOfAKind Two Jack Five         -> 5026
          ThreeOfAKind Two Jack Six          -> 5027
          ThreeOfAKind Two Jack Seven        -> 5028
          ThreeOfAKind Two Jack Eight        -> 5029
          ThreeOfAKind Two Jack Nine         -> 5030
          ThreeOfAKind Two Jack Ten          -> 5031
          ThreeOfAKind Two Queen Three       -> 5032
          ThreeOfAKind Two Queen Four        -> 5033
          ThreeOfAKind Two Queen Five        -> 5034
          ThreeOfAKind Two Queen Six         -> 5035
          ThreeOfAKind Two Queen Seven       -> 5036
          ThreeOfAKind Two Queen Eight       -> 5037
          ThreeOfAKind Two Queen Nine        -> 5038
          ThreeOfAKind Two Queen Ten         -> 5039
          ThreeOfAKind Two Queen Jack        -> 5040
          ThreeOfAKind Two King Three        -> 5041
          ThreeOfAKind Two King Four         -> 5042
          ThreeOfAKind Two King Five         -> 5043
          ThreeOfAKind Two King Six          -> 5044
          ThreeOfAKind Two King Seven        -> 5045
          ThreeOfAKind Two King Eight        -> 5046
          ThreeOfAKind Two King Nine         -> 5047
          ThreeOfAKind Two King Ten          -> 5048
          ThreeOfAKind Two King Jack         -> 5049
          ThreeOfAKind Two King Queen        -> 5050
          ThreeOfAKind Two Ace Three         -> 5051
          ThreeOfAKind Two Ace Four          -> 5052
          ThreeOfAKind Two Ace Five          -> 5053
          ThreeOfAKind Two Ace Six           -> 5054
          ThreeOfAKind Two Ace Seven         -> 5055
          ThreeOfAKind Two Ace Eight         -> 5056
          ThreeOfAKind Two Ace Nine          -> 5057
          ThreeOfAKind Two Ace Ten           -> 5058
          ThreeOfAKind Two Ace Jack          -> 5059
          ThreeOfAKind Two Ace Queen         -> 5060
          ThreeOfAKind Two Ace King          -> 5061
          ThreeOfAKind Three Four Two        -> 5062
          ThreeOfAKind Three Five Two        -> 5063
          ThreeOfAKind Three Five Four       -> 5064
          ThreeOfAKind Three Six Two         -> 5065
          ThreeOfAKind Three Six Four        -> 5066
          ThreeOfAKind Three Six Five        -> 5067
          ThreeOfAKind Three Seven Two       -> 5068
          ThreeOfAKind Three Seven Four      -> 5069
          ThreeOfAKind Three Seven Five      -> 5070
          ThreeOfAKind Three Seven Six       -> 5071
          ThreeOfAKind Three Eight Two       -> 5072
          ThreeOfAKind Three Eight Four      -> 5073
          ThreeOfAKind Three Eight Five      -> 5074
          ThreeOfAKind Three Eight Six       -> 5075
          ThreeOfAKind Three Eight Seven     -> 5076
          ThreeOfAKind Three Nine Two        -> 5077
          ThreeOfAKind Three Nine Four       -> 5078
          ThreeOfAKind Three Nine Five       -> 5079
          ThreeOfAKind Three Nine Six        -> 5080
          ThreeOfAKind Three Nine Seven      -> 5081
          ThreeOfAKind Three Nine Eight      -> 5082
          ThreeOfAKind Three Ten Two         -> 5083
          ThreeOfAKind Three Ten Four        -> 5084
          ThreeOfAKind Three Ten Five        -> 5085
          ThreeOfAKind Three Ten Six         -> 5086
          ThreeOfAKind Three Ten Seven       -> 5087
          ThreeOfAKind Three Ten Eight       -> 5088
          ThreeOfAKind Three Ten Nine        -> 5089
          ThreeOfAKind Three Jack Two        -> 5090
          ThreeOfAKind Three Jack Four       -> 5091
          ThreeOfAKind Three Jack Five       -> 5092
          ThreeOfAKind Three Jack Six        -> 5093
          ThreeOfAKind Three Jack Seven      -> 5094
          ThreeOfAKind Three Jack Eight      -> 5095
          ThreeOfAKind Three Jack Nine       -> 5096
          ThreeOfAKind Three Jack Ten        -> 5097
          ThreeOfAKind Three Queen Two       -> 5098
          ThreeOfAKind Three Queen Four      -> 5099
          ThreeOfAKind Three Queen Five      -> 5100
          ThreeOfAKind Three Queen Six       -> 5101
          ThreeOfAKind Three Queen Seven     -> 5102
          ThreeOfAKind Three Queen Eight     -> 5103
          ThreeOfAKind Three Queen Nine      -> 5104
          ThreeOfAKind Three Queen Ten       -> 5105
          ThreeOfAKind Three Queen Jack      -> 5106
          ThreeOfAKind Three King Two        -> 5107
          ThreeOfAKind Three King Four       -> 5108
          ThreeOfAKind Three King Five       -> 5109
          ThreeOfAKind Three King Six        -> 5110
          ThreeOfAKind Three King Seven      -> 5111
          ThreeOfAKind Three King Eight      -> 5112
          ThreeOfAKind Three King Nine       -> 5113
          ThreeOfAKind Three King Ten        -> 5114
          ThreeOfAKind Three King Jack       -> 5115
          ThreeOfAKind Three King Queen      -> 5116
          ThreeOfAKind Three Ace Two         -> 5117
          ThreeOfAKind Three Ace Four        -> 5118
          ThreeOfAKind Three Ace Five        -> 5119
          ThreeOfAKind Three Ace Six         -> 5120
          ThreeOfAKind Three Ace Seven       -> 5121
          ThreeOfAKind Three Ace Eight       -> 5122
          ThreeOfAKind Three Ace Nine        -> 5123
          ThreeOfAKind Three Ace Ten         -> 5124
          ThreeOfAKind Three Ace Jack        -> 5125
          ThreeOfAKind Three Ace Queen       -> 5126
          ThreeOfAKind Three Ace King        -> 5127
          ThreeOfAKind Four Three Two        -> 5128
          ThreeOfAKind Four Five Two         -> 5129
          ThreeOfAKind Four Five Three       -> 5130
          ThreeOfAKind Four Six Two          -> 5131
          ThreeOfAKind Four Six Three        -> 5132
          ThreeOfAKind Four Six Five         -> 5133
          ThreeOfAKind Four Seven Two        -> 5134
          ThreeOfAKind Four Seven Three      -> 5135
          ThreeOfAKind Four Seven Five       -> 5136
          ThreeOfAKind Four Seven Six        -> 5137
          ThreeOfAKind Four Eight Two        -> 5138
          ThreeOfAKind Four Eight Three      -> 5139
          ThreeOfAKind Four Eight Five       -> 5140
          ThreeOfAKind Four Eight Six        -> 5141
          ThreeOfAKind Four Eight Seven      -> 5142
          ThreeOfAKind Four Nine Two         -> 5143
          ThreeOfAKind Four Nine Three       -> 5144
          ThreeOfAKind Four Nine Five        -> 5145
          ThreeOfAKind Four Nine Six         -> 5146
          ThreeOfAKind Four Nine Seven       -> 5147
          ThreeOfAKind Four Nine Eight       -> 5148
          ThreeOfAKind Four Ten Two          -> 5149
          ThreeOfAKind Four Ten Three        -> 5150
          ThreeOfAKind Four Ten Five         -> 5151
          ThreeOfAKind Four Ten Six          -> 5152
          ThreeOfAKind Four Ten Seven        -> 5153
          ThreeOfAKind Four Ten Eight        -> 5154
          ThreeOfAKind Four Ten Nine         -> 5155
          ThreeOfAKind Four Jack Two         -> 5156
          ThreeOfAKind Four Jack Three       -> 5157
          ThreeOfAKind Four Jack Five        -> 5158
          ThreeOfAKind Four Jack Six         -> 5159
          ThreeOfAKind Four Jack Seven       -> 5160
          ThreeOfAKind Four Jack Eight       -> 5161
          ThreeOfAKind Four Jack Nine        -> 5162
          ThreeOfAKind Four Jack Ten         -> 5163
          ThreeOfAKind Four Queen Two        -> 5164
          ThreeOfAKind Four Queen Three      -> 5165
          ThreeOfAKind Four Queen Five       -> 5166
          ThreeOfAKind Four Queen Six        -> 5167
          ThreeOfAKind Four Queen Seven      -> 5168
          ThreeOfAKind Four Queen Eight      -> 5169
          ThreeOfAKind Four Queen Nine       -> 5170
          ThreeOfAKind Four Queen Ten        -> 5171
          ThreeOfAKind Four Queen Jack       -> 5172
          ThreeOfAKind Four King Two         -> 5173
          ThreeOfAKind Four King Three       -> 5174
          ThreeOfAKind Four King Five        -> 5175
          ThreeOfAKind Four King Six         -> 5176
          ThreeOfAKind Four King Seven       -> 5177
          ThreeOfAKind Four King Eight       -> 5178
          ThreeOfAKind Four King Nine        -> 5179
          ThreeOfAKind Four King Ten         -> 5180
          ThreeOfAKind Four King Jack        -> 5181
          ThreeOfAKind Four King Queen       -> 5182
          ThreeOfAKind Four Ace Two          -> 5183
          ThreeOfAKind Four Ace Three        -> 5184
          ThreeOfAKind Four Ace Five         -> 5185
          ThreeOfAKind Four Ace Six          -> 5186
          ThreeOfAKind Four Ace Seven        -> 5187
          ThreeOfAKind Four Ace Eight        -> 5188
          ThreeOfAKind Four Ace Nine         -> 5189
          ThreeOfAKind Four Ace Ten          -> 5190
          ThreeOfAKind Four Ace Jack         -> 5191
          ThreeOfAKind Four Ace Queen        -> 5192
          ThreeOfAKind Four Ace King         -> 5193
          ThreeOfAKind Five Three Two        -> 5194
          ThreeOfAKind Five Four Two         -> 5195
          ThreeOfAKind Five Four Three       -> 5196
          ThreeOfAKind Five Six Two          -> 5197
          ThreeOfAKind Five Six Three        -> 5198
          ThreeOfAKind Five Six Four         -> 5199
          ThreeOfAKind Five Seven Two        -> 5200
          ThreeOfAKind Five Seven Three      -> 5201
          ThreeOfAKind Five Seven Four       -> 5202
          ThreeOfAKind Five Seven Six        -> 5203
          ThreeOfAKind Five Eight Two        -> 5204
          ThreeOfAKind Five Eight Three      -> 5205
          ThreeOfAKind Five Eight Four       -> 5206
          ThreeOfAKind Five Eight Six        -> 5207
          ThreeOfAKind Five Eight Seven      -> 5208
          ThreeOfAKind Five Nine Two         -> 5209
          ThreeOfAKind Five Nine Three       -> 5210
          ThreeOfAKind Five Nine Four        -> 5211
          ThreeOfAKind Five Nine Six         -> 5212
          ThreeOfAKind Five Nine Seven       -> 5213
          ThreeOfAKind Five Nine Eight       -> 5214
          ThreeOfAKind Five Ten Two          -> 5215
          ThreeOfAKind Five Ten Three        -> 5216
          ThreeOfAKind Five Ten Four         -> 5217
          ThreeOfAKind Five Ten Six          -> 5218
          ThreeOfAKind Five Ten Seven        -> 5219
          ThreeOfAKind Five Ten Eight        -> 5220
          ThreeOfAKind Five Ten Nine         -> 5221
          ThreeOfAKind Five Jack Two         -> 5222
          ThreeOfAKind Five Jack Three       -> 5223
          ThreeOfAKind Five Jack Four        -> 5224
          ThreeOfAKind Five Jack Six         -> 5225
          ThreeOfAKind Five Jack Seven       -> 5226
          ThreeOfAKind Five Jack Eight       -> 5227
          ThreeOfAKind Five Jack Nine        -> 5228
          ThreeOfAKind Five Jack Ten         -> 5229
          ThreeOfAKind Five Queen Two        -> 5230
          ThreeOfAKind Five Queen Three      -> 5231
          ThreeOfAKind Five Queen Four       -> 5232
          ThreeOfAKind Five Queen Six        -> 5233
          ThreeOfAKind Five Queen Seven      -> 5234
          ThreeOfAKind Five Queen Eight      -> 5235
          ThreeOfAKind Five Queen Nine       -> 5236
          ThreeOfAKind Five Queen Ten        -> 5237
          ThreeOfAKind Five Queen Jack       -> 5238
          ThreeOfAKind Five King Two         -> 5239
          ThreeOfAKind Five King Three       -> 5240
          ThreeOfAKind Five King Four        -> 5241
          ThreeOfAKind Five King Six         -> 5242
          ThreeOfAKind Five King Seven       -> 5243
          ThreeOfAKind Five King Eight       -> 5244
          ThreeOfAKind Five King Nine        -> 5245
          ThreeOfAKind Five King Ten         -> 5246
          ThreeOfAKind Five King Jack        -> 5247
          ThreeOfAKind Five King Queen       -> 5248
          ThreeOfAKind Five Ace Two          -> 5249
          ThreeOfAKind Five Ace Three        -> 5250
          ThreeOfAKind Five Ace Four         -> 5251
          ThreeOfAKind Five Ace Six          -> 5252
          ThreeOfAKind Five Ace Seven        -> 5253
          ThreeOfAKind Five Ace Eight        -> 5254
          ThreeOfAKind Five Ace Nine         -> 5255
          ThreeOfAKind Five Ace Ten          -> 5256
          ThreeOfAKind Five Ace Jack         -> 5257
          ThreeOfAKind Five Ace Queen        -> 5258
          ThreeOfAKind Five Ace King         -> 5259
          ThreeOfAKind Six Three Two         -> 5260
          ThreeOfAKind Six Four Two          -> 5261
          ThreeOfAKind Six Four Three        -> 5262
          ThreeOfAKind Six Five Two          -> 5263
          ThreeOfAKind Six Five Three        -> 5264
          ThreeOfAKind Six Five Four         -> 5265
          ThreeOfAKind Six Seven Two         -> 5266
          ThreeOfAKind Six Seven Three       -> 5267
          ThreeOfAKind Six Seven Four        -> 5268
          ThreeOfAKind Six Seven Five        -> 5269
          ThreeOfAKind Six Eight Two         -> 5270
          ThreeOfAKind Six Eight Three       -> 5271
          ThreeOfAKind Six Eight Four        -> 5272
          ThreeOfAKind Six Eight Five        -> 5273
          ThreeOfAKind Six Eight Seven       -> 5274
          ThreeOfAKind Six Nine Two          -> 5275
          ThreeOfAKind Six Nine Three        -> 5276
          ThreeOfAKind Six Nine Four         -> 5277
          ThreeOfAKind Six Nine Five         -> 5278
          ThreeOfAKind Six Nine Seven        -> 5279
          ThreeOfAKind Six Nine Eight        -> 5280
          ThreeOfAKind Six Ten Two           -> 5281
          ThreeOfAKind Six Ten Three         -> 5282
          ThreeOfAKind Six Ten Four          -> 5283
          ThreeOfAKind Six Ten Five          -> 5284
          ThreeOfAKind Six Ten Seven         -> 5285
          ThreeOfAKind Six Ten Eight         -> 5286
          ThreeOfAKind Six Ten Nine          -> 5287
          ThreeOfAKind Six Jack Two          -> 5288
          ThreeOfAKind Six Jack Three        -> 5289
          ThreeOfAKind Six Jack Four         -> 5290
          ThreeOfAKind Six Jack Five         -> 5291
          ThreeOfAKind Six Jack Seven        -> 5292
          ThreeOfAKind Six Jack Eight        -> 5293
          ThreeOfAKind Six Jack Nine         -> 5294
          ThreeOfAKind Six Jack Ten          -> 5295
          ThreeOfAKind Six Queen Two         -> 5296
          ThreeOfAKind Six Queen Three       -> 5297
          ThreeOfAKind Six Queen Four        -> 5298
          ThreeOfAKind Six Queen Five        -> 5299
          ThreeOfAKind Six Queen Seven       -> 5300
          ThreeOfAKind Six Queen Eight       -> 5301
          ThreeOfAKind Six Queen Nine        -> 5302
          ThreeOfAKind Six Queen Ten         -> 5303
          ThreeOfAKind Six Queen Jack        -> 5304
          ThreeOfAKind Six King Two          -> 5305
          ThreeOfAKind Six King Three        -> 5306
          ThreeOfAKind Six King Four         -> 5307
          ThreeOfAKind Six King Five         -> 5308
          ThreeOfAKind Six King Seven        -> 5309
          ThreeOfAKind Six King Eight        -> 5310
          ThreeOfAKind Six King Nine         -> 5311
          ThreeOfAKind Six King Ten          -> 5312
          ThreeOfAKind Six King Jack         -> 5313
          ThreeOfAKind Six King Queen        -> 5314
          ThreeOfAKind Six Ace Two           -> 5315
          ThreeOfAKind Six Ace Three         -> 5316
          ThreeOfAKind Six Ace Four          -> 5317
          ThreeOfAKind Six Ace Five          -> 5318
          ThreeOfAKind Six Ace Seven         -> 5319
          ThreeOfAKind Six Ace Eight         -> 5320
          ThreeOfAKind Six Ace Nine          -> 5321
          ThreeOfAKind Six Ace Ten           -> 5322
          ThreeOfAKind Six Ace Jack          -> 5323
          ThreeOfAKind Six Ace Queen         -> 5324
          ThreeOfAKind Six Ace King          -> 5325
          ThreeOfAKind Seven Three Two       -> 5326
          ThreeOfAKind Seven Four Two        -> 5327
          ThreeOfAKind Seven Four Three      -> 5328
          ThreeOfAKind Seven Five Two        -> 5329
          ThreeOfAKind Seven Five Three      -> 5330
          ThreeOfAKind Seven Five Four       -> 5331
          ThreeOfAKind Seven Six Two         -> 5332
          ThreeOfAKind Seven Six Three       -> 5333
          ThreeOfAKind Seven Six Four        -> 5334
          ThreeOfAKind Seven Six Five        -> 5335
          ThreeOfAKind Seven Eight Two       -> 5336
          ThreeOfAKind Seven Eight Three     -> 5337
          ThreeOfAKind Seven Eight Four      -> 5338
          ThreeOfAKind Seven Eight Five      -> 5339
          ThreeOfAKind Seven Eight Six       -> 5340
          ThreeOfAKind Seven Nine Two        -> 5341
          ThreeOfAKind Seven Nine Three      -> 5342
          ThreeOfAKind Seven Nine Four       -> 5343
          ThreeOfAKind Seven Nine Five       -> 5344
          ThreeOfAKind Seven Nine Six        -> 5345
          ThreeOfAKind Seven Nine Eight      -> 5346
          ThreeOfAKind Seven Ten Two         -> 5347
          ThreeOfAKind Seven Ten Three       -> 5348
          ThreeOfAKind Seven Ten Four        -> 5349
          ThreeOfAKind Seven Ten Five        -> 5350
          ThreeOfAKind Seven Ten Six         -> 5351
          ThreeOfAKind Seven Ten Eight       -> 5352
          ThreeOfAKind Seven Ten Nine        -> 5353
          ThreeOfAKind Seven Jack Two        -> 5354
          ThreeOfAKind Seven Jack Three      -> 5355
          ThreeOfAKind Seven Jack Four       -> 5356
          ThreeOfAKind Seven Jack Five       -> 5357
          ThreeOfAKind Seven Jack Six        -> 5358
          ThreeOfAKind Seven Jack Eight      -> 5359
          ThreeOfAKind Seven Jack Nine       -> 5360
          ThreeOfAKind Seven Jack Ten        -> 5361
          ThreeOfAKind Seven Queen Two       -> 5362
          ThreeOfAKind Seven Queen Three     -> 5363
          ThreeOfAKind Seven Queen Four      -> 5364
          ThreeOfAKind Seven Queen Five      -> 5365
          ThreeOfAKind Seven Queen Six       -> 5366
          ThreeOfAKind Seven Queen Eight     -> 5367
          ThreeOfAKind Seven Queen Nine      -> 5368
          ThreeOfAKind Seven Queen Ten       -> 5369
          ThreeOfAKind Seven Queen Jack      -> 5370
          ThreeOfAKind Seven King Two        -> 5371
          ThreeOfAKind Seven King Three      -> 5372
          ThreeOfAKind Seven King Four       -> 5373
          ThreeOfAKind Seven King Five       -> 5374
          ThreeOfAKind Seven King Six        -> 5375
          ThreeOfAKind Seven King Eight      -> 5376
          ThreeOfAKind Seven King Nine       -> 5377
          ThreeOfAKind Seven King Ten        -> 5378
          ThreeOfAKind Seven King Jack       -> 5379
          ThreeOfAKind Seven King Queen      -> 5380
          ThreeOfAKind Seven Ace Two         -> 5381
          ThreeOfAKind Seven Ace Three       -> 5382
          ThreeOfAKind Seven Ace Four        -> 5383
          ThreeOfAKind Seven Ace Five        -> 5384
          ThreeOfAKind Seven Ace Six         -> 5385
          ThreeOfAKind Seven Ace Eight       -> 5386
          ThreeOfAKind Seven Ace Nine        -> 5387
          ThreeOfAKind Seven Ace Ten         -> 5388
          ThreeOfAKind Seven Ace Jack        -> 5389
          ThreeOfAKind Seven Ace Queen       -> 5390
          ThreeOfAKind Seven Ace King        -> 5391
          ThreeOfAKind Eight Three Two       -> 5392
          ThreeOfAKind Eight Four Two        -> 5393
          ThreeOfAKind Eight Four Three      -> 5394
          ThreeOfAKind Eight Five Two        -> 5395
          ThreeOfAKind Eight Five Three      -> 5396
          ThreeOfAKind Eight Five Four       -> 5397
          ThreeOfAKind Eight Six Two         -> 5398
          ThreeOfAKind Eight Six Three       -> 5399
          ThreeOfAKind Eight Six Four        -> 5400
          ThreeOfAKind Eight Six Five        -> 5401
          ThreeOfAKind Eight Seven Two       -> 5402
          ThreeOfAKind Eight Seven Three     -> 5403
          ThreeOfAKind Eight Seven Four      -> 5404
          ThreeOfAKind Eight Seven Five      -> 5405
          ThreeOfAKind Eight Seven Six       -> 5406
          ThreeOfAKind Eight Nine Two        -> 5407
          ThreeOfAKind Eight Nine Three      -> 5408
          ThreeOfAKind Eight Nine Four       -> 5409
          ThreeOfAKind Eight Nine Five       -> 5410
          ThreeOfAKind Eight Nine Six        -> 5411
          ThreeOfAKind Eight Nine Seven      -> 5412
          ThreeOfAKind Eight Ten Two         -> 5413
          ThreeOfAKind Eight Ten Three       -> 5414
          ThreeOfAKind Eight Ten Four        -> 5415
          ThreeOfAKind Eight Ten Five        -> 5416
          ThreeOfAKind Eight Ten Six         -> 5417
          ThreeOfAKind Eight Ten Seven       -> 5418
          ThreeOfAKind Eight Ten Nine        -> 5419
          ThreeOfAKind Eight Jack Two        -> 5420
          ThreeOfAKind Eight Jack Three      -> 5421
          ThreeOfAKind Eight Jack Four       -> 5422
          ThreeOfAKind Eight Jack Five       -> 5423
          ThreeOfAKind Eight Jack Six        -> 5424
          ThreeOfAKind Eight Jack Seven      -> 5425
          ThreeOfAKind Eight Jack Nine       -> 5426
          ThreeOfAKind Eight Jack Ten        -> 5427
          ThreeOfAKind Eight Queen Two       -> 5428
          ThreeOfAKind Eight Queen Three     -> 5429
          ThreeOfAKind Eight Queen Four      -> 5430
          ThreeOfAKind Eight Queen Five      -> 5431
          ThreeOfAKind Eight Queen Six       -> 5432
          ThreeOfAKind Eight Queen Seven     -> 5433
          ThreeOfAKind Eight Queen Nine      -> 5434
          ThreeOfAKind Eight Queen Ten       -> 5435
          ThreeOfAKind Eight Queen Jack      -> 5436
          ThreeOfAKind Eight King Two        -> 5437
          ThreeOfAKind Eight King Three      -> 5438
          ThreeOfAKind Eight King Four       -> 5439
          ThreeOfAKind Eight King Five       -> 5440
          ThreeOfAKind Eight King Six        -> 5441
          ThreeOfAKind Eight King Seven      -> 5442
          ThreeOfAKind Eight King Nine       -> 5443
          ThreeOfAKind Eight King Ten        -> 5444
          ThreeOfAKind Eight King Jack       -> 5445
          ThreeOfAKind Eight King Queen      -> 5446
          ThreeOfAKind Eight Ace Two         -> 5447
          ThreeOfAKind Eight Ace Three       -> 5448
          ThreeOfAKind Eight Ace Four        -> 5449
          ThreeOfAKind Eight Ace Five        -> 5450
          ThreeOfAKind Eight Ace Six         -> 5451
          ThreeOfAKind Eight Ace Seven       -> 5452
          ThreeOfAKind Eight Ace Nine        -> 5453
          ThreeOfAKind Eight Ace Ten         -> 5454
          ThreeOfAKind Eight Ace Jack        -> 5455
          ThreeOfAKind Eight Ace Queen       -> 5456
          ThreeOfAKind Eight Ace King        -> 5457
          ThreeOfAKind Nine Three Two        -> 5458
          ThreeOfAKind Nine Four Two         -> 5459
          ThreeOfAKind Nine Four Three       -> 5460
          ThreeOfAKind Nine Five Two         -> 5461
          ThreeOfAKind Nine Five Three       -> 5462
          ThreeOfAKind Nine Five Four        -> 5463
          ThreeOfAKind Nine Six Two          -> 5464
          ThreeOfAKind Nine Six Three        -> 5465
          ThreeOfAKind Nine Six Four         -> 5466
          ThreeOfAKind Nine Six Five         -> 5467
          ThreeOfAKind Nine Seven Two        -> 5468
          ThreeOfAKind Nine Seven Three      -> 5469
          ThreeOfAKind Nine Seven Four       -> 5470
          ThreeOfAKind Nine Seven Five       -> 5471
          ThreeOfAKind Nine Seven Six        -> 5472
          ThreeOfAKind Nine Eight Two        -> 5473
          ThreeOfAKind Nine Eight Three      -> 5474
          ThreeOfAKind Nine Eight Four       -> 5475
          ThreeOfAKind Nine Eight Five       -> 5476
          ThreeOfAKind Nine Eight Six        -> 5477
          ThreeOfAKind Nine Eight Seven      -> 5478
          ThreeOfAKind Nine Ten Two          -> 5479
          ThreeOfAKind Nine Ten Three        -> 5480
          ThreeOfAKind Nine Ten Four         -> 5481
          ThreeOfAKind Nine Ten Five         -> 5482
          ThreeOfAKind Nine Ten Six          -> 5483
          ThreeOfAKind Nine Ten Seven        -> 5484
          ThreeOfAKind Nine Ten Eight        -> 5485
          ThreeOfAKind Nine Jack Two         -> 5486
          ThreeOfAKind Nine Jack Three       -> 5487
          ThreeOfAKind Nine Jack Four        -> 5488
          ThreeOfAKind Nine Jack Five        -> 5489
          ThreeOfAKind Nine Jack Six         -> 5490
          ThreeOfAKind Nine Jack Seven       -> 5491
          ThreeOfAKind Nine Jack Eight       -> 5492
          ThreeOfAKind Nine Jack Ten         -> 5493
          ThreeOfAKind Nine Queen Two        -> 5494
          ThreeOfAKind Nine Queen Three      -> 5495
          ThreeOfAKind Nine Queen Four       -> 5496
          ThreeOfAKind Nine Queen Five       -> 5497
          ThreeOfAKind Nine Queen Six        -> 5498
          ThreeOfAKind Nine Queen Seven      -> 5499
          ThreeOfAKind Nine Queen Eight      -> 5500
          ThreeOfAKind Nine Queen Ten        -> 5501
          ThreeOfAKind Nine Queen Jack       -> 5502
          ThreeOfAKind Nine King Two         -> 5503
          ThreeOfAKind Nine King Three       -> 5504
          ThreeOfAKind Nine King Four        -> 5505
          ThreeOfAKind Nine King Five        -> 5506
          ThreeOfAKind Nine King Six         -> 5507
          ThreeOfAKind Nine King Seven       -> 5508
          ThreeOfAKind Nine King Eight       -> 5509
          ThreeOfAKind Nine King Ten         -> 5510
          ThreeOfAKind Nine King Jack        -> 5511
          ThreeOfAKind Nine King Queen       -> 5512
          ThreeOfAKind Nine Ace Two          -> 5513
          ThreeOfAKind Nine Ace Three        -> 5514
          ThreeOfAKind Nine Ace Four         -> 5515
          ThreeOfAKind Nine Ace Five         -> 5516
          ThreeOfAKind Nine Ace Six          -> 5517
          ThreeOfAKind Nine Ace Seven        -> 5518
          ThreeOfAKind Nine Ace Eight        -> 5519
          ThreeOfAKind Nine Ace Ten          -> 5520
          ThreeOfAKind Nine Ace Jack         -> 5521
          ThreeOfAKind Nine Ace Queen        -> 5522
          ThreeOfAKind Nine Ace King         -> 5523
          ThreeOfAKind Ten Three Two         -> 5524
          ThreeOfAKind Ten Four Two          -> 5525
          ThreeOfAKind Ten Four Three        -> 5526
          ThreeOfAKind Ten Five Two          -> 5527
          ThreeOfAKind Ten Five Three        -> 5528
          ThreeOfAKind Ten Five Four         -> 5529
          ThreeOfAKind Ten Six Two           -> 5530
          ThreeOfAKind Ten Six Three         -> 5531
          ThreeOfAKind Ten Six Four          -> 5532
          ThreeOfAKind Ten Six Five          -> 5533
          ThreeOfAKind Ten Seven Two         -> 5534
          ThreeOfAKind Ten Seven Three       -> 5535
          ThreeOfAKind Ten Seven Four        -> 5536
          ThreeOfAKind Ten Seven Five        -> 5537
          ThreeOfAKind Ten Seven Six         -> 5538
          ThreeOfAKind Ten Eight Two         -> 5539
          ThreeOfAKind Ten Eight Three       -> 5540
          ThreeOfAKind Ten Eight Four        -> 5541
          ThreeOfAKind Ten Eight Five        -> 5542
          ThreeOfAKind Ten Eight Six         -> 5543
          ThreeOfAKind Ten Eight Seven       -> 5544
          ThreeOfAKind Ten Nine Two          -> 5545
          ThreeOfAKind Ten Nine Three        -> 5546
          ThreeOfAKind Ten Nine Four         -> 5547
          ThreeOfAKind Ten Nine Five         -> 5548
          ThreeOfAKind Ten Nine Six          -> 5549
          ThreeOfAKind Ten Nine Seven        -> 5550
          ThreeOfAKind Ten Nine Eight        -> 5551
          ThreeOfAKind Ten Jack Two          -> 5552
          ThreeOfAKind Ten Jack Three        -> 5553
          ThreeOfAKind Ten Jack Four         -> 5554
          ThreeOfAKind Ten Jack Five         -> 5555
          ThreeOfAKind Ten Jack Six          -> 5556
          ThreeOfAKind Ten Jack Seven        -> 5557
          ThreeOfAKind Ten Jack Eight        -> 5558
          ThreeOfAKind Ten Jack Nine         -> 5559
          ThreeOfAKind Ten Queen Two         -> 5560
          ThreeOfAKind Ten Queen Three       -> 5561
          ThreeOfAKind Ten Queen Four        -> 5562
          ThreeOfAKind Ten Queen Five        -> 5563
          ThreeOfAKind Ten Queen Six         -> 5564
          ThreeOfAKind Ten Queen Seven       -> 5565
          ThreeOfAKind Ten Queen Eight       -> 5566
          ThreeOfAKind Ten Queen Nine        -> 5567
          ThreeOfAKind Ten Queen Jack        -> 5568
          ThreeOfAKind Ten King Two          -> 5569
          ThreeOfAKind Ten King Three        -> 5570
          ThreeOfAKind Ten King Four         -> 5571
          ThreeOfAKind Ten King Five         -> 5572
          ThreeOfAKind Ten King Six          -> 5573
          ThreeOfAKind Ten King Seven        -> 5574
          ThreeOfAKind Ten King Eight        -> 5575
          ThreeOfAKind Ten King Nine         -> 5576
          ThreeOfAKind Ten King Jack         -> 5577
          ThreeOfAKind Ten King Queen        -> 5578
          ThreeOfAKind Ten Ace Two           -> 5579
          ThreeOfAKind Ten Ace Three         -> 5580
          ThreeOfAKind Ten Ace Four          -> 5581
          ThreeOfAKind Ten Ace Five          -> 5582
          ThreeOfAKind Ten Ace Six           -> 5583
          ThreeOfAKind Ten Ace Seven         -> 5584
          ThreeOfAKind Ten Ace Eight         -> 5585
          ThreeOfAKind Ten Ace Nine          -> 5586
          ThreeOfAKind Ten Ace Jack          -> 5587
          ThreeOfAKind Ten Ace Queen         -> 5588
          ThreeOfAKind Ten Ace King          -> 5589
          ThreeOfAKind Jack Three Two        -> 5590
          ThreeOfAKind Jack Four Two         -> 5591
          ThreeOfAKind Jack Four Three       -> 5592
          ThreeOfAKind Jack Five Two         -> 5593
          ThreeOfAKind Jack Five Three       -> 5594
          ThreeOfAKind Jack Five Four        -> 5595
          ThreeOfAKind Jack Six Two          -> 5596
          ThreeOfAKind Jack Six Three        -> 5597
          ThreeOfAKind Jack Six Four         -> 5598
          ThreeOfAKind Jack Six Five         -> 5599
          ThreeOfAKind Jack Seven Two        -> 5600
          ThreeOfAKind Jack Seven Three      -> 5601
          ThreeOfAKind Jack Seven Four       -> 5602
          ThreeOfAKind Jack Seven Five       -> 5603
          ThreeOfAKind Jack Seven Six        -> 5604
          ThreeOfAKind Jack Eight Two        -> 5605
          ThreeOfAKind Jack Eight Three      -> 5606
          ThreeOfAKind Jack Eight Four       -> 5607
          ThreeOfAKind Jack Eight Five       -> 5608
          ThreeOfAKind Jack Eight Six        -> 5609
          ThreeOfAKind Jack Eight Seven      -> 5610
          ThreeOfAKind Jack Nine Two         -> 5611
          ThreeOfAKind Jack Nine Three       -> 5612
          ThreeOfAKind Jack Nine Four        -> 5613
          ThreeOfAKind Jack Nine Five        -> 5614
          ThreeOfAKind Jack Nine Six         -> 5615
          ThreeOfAKind Jack Nine Seven       -> 5616
          ThreeOfAKind Jack Nine Eight       -> 5617
          ThreeOfAKind Jack Ten Two          -> 5618
          ThreeOfAKind Jack Ten Three        -> 5619
          ThreeOfAKind Jack Ten Four         -> 5620
          ThreeOfAKind Jack Ten Five         -> 5621
          ThreeOfAKind Jack Ten Six          -> 5622
          ThreeOfAKind Jack Ten Seven        -> 5623
          ThreeOfAKind Jack Ten Eight        -> 5624
          ThreeOfAKind Jack Ten Nine         -> 5625
          ThreeOfAKind Jack Queen Two        -> 5626
          ThreeOfAKind Jack Queen Three      -> 5627
          ThreeOfAKind Jack Queen Four       -> 5628
          ThreeOfAKind Jack Queen Five       -> 5629
          ThreeOfAKind Jack Queen Six        -> 5630
          ThreeOfAKind Jack Queen Seven      -> 5631
          ThreeOfAKind Jack Queen Eight      -> 5632
          ThreeOfAKind Jack Queen Nine       -> 5633
          ThreeOfAKind Jack Queen Ten        -> 5634
          ThreeOfAKind Jack King Two         -> 5635
          ThreeOfAKind Jack King Three       -> 5636
          ThreeOfAKind Jack King Four        -> 5637
          ThreeOfAKind Jack King Five        -> 5638
          ThreeOfAKind Jack King Six         -> 5639
          ThreeOfAKind Jack King Seven       -> 5640
          ThreeOfAKind Jack King Eight       -> 5641
          ThreeOfAKind Jack King Nine        -> 5642
          ThreeOfAKind Jack King Ten         -> 5643
          ThreeOfAKind Jack King Queen       -> 5644
          ThreeOfAKind Jack Ace Two          -> 5645
          ThreeOfAKind Jack Ace Three        -> 5646
          ThreeOfAKind Jack Ace Four         -> 5647
          ThreeOfAKind Jack Ace Five         -> 5648
          ThreeOfAKind Jack Ace Six          -> 5649
          ThreeOfAKind Jack Ace Seven        -> 5650
          ThreeOfAKind Jack Ace Eight        -> 5651
          ThreeOfAKind Jack Ace Nine         -> 5652
          ThreeOfAKind Jack Ace Ten          -> 5653
          ThreeOfAKind Jack Ace Queen        -> 5654
          ThreeOfAKind Jack Ace King         -> 5655
          ThreeOfAKind Queen Three Two       -> 5656
          ThreeOfAKind Queen Four Two        -> 5657
          ThreeOfAKind Queen Four Three      -> 5658
          ThreeOfAKind Queen Five Two        -> 5659
          ThreeOfAKind Queen Five Three      -> 5660
          ThreeOfAKind Queen Five Four       -> 5661
          ThreeOfAKind Queen Six Two         -> 5662
          ThreeOfAKind Queen Six Three       -> 5663
          ThreeOfAKind Queen Six Four        -> 5664
          ThreeOfAKind Queen Six Five        -> 5665
          ThreeOfAKind Queen Seven Two       -> 5666
          ThreeOfAKind Queen Seven Three     -> 5667
          ThreeOfAKind Queen Seven Four      -> 5668
          ThreeOfAKind Queen Seven Five      -> 5669
          ThreeOfAKind Queen Seven Six       -> 5670
          ThreeOfAKind Queen Eight Two       -> 5671
          ThreeOfAKind Queen Eight Three     -> 5672
          ThreeOfAKind Queen Eight Four      -> 5673
          ThreeOfAKind Queen Eight Five      -> 5674
          ThreeOfAKind Queen Eight Six       -> 5675
          ThreeOfAKind Queen Eight Seven     -> 5676
          ThreeOfAKind Queen Nine Two        -> 5677
          ThreeOfAKind Queen Nine Three      -> 5678
          ThreeOfAKind Queen Nine Four       -> 5679
          ThreeOfAKind Queen Nine Five       -> 5680
          ThreeOfAKind Queen Nine Six        -> 5681
          ThreeOfAKind Queen Nine Seven      -> 5682
          ThreeOfAKind Queen Nine Eight      -> 5683
          ThreeOfAKind Queen Ten Two         -> 5684
          ThreeOfAKind Queen Ten Three       -> 5685
          ThreeOfAKind Queen Ten Four        -> 5686
          ThreeOfAKind Queen Ten Five        -> 5687
          ThreeOfAKind Queen Ten Six         -> 5688
          ThreeOfAKind Queen Ten Seven       -> 5689
          ThreeOfAKind Queen Ten Eight       -> 5690
          ThreeOfAKind Queen Ten Nine        -> 5691
          ThreeOfAKind Queen Jack Two        -> 5692
          ThreeOfAKind Queen Jack Three      -> 5693
          ThreeOfAKind Queen Jack Four       -> 5694
          ThreeOfAKind Queen Jack Five       -> 5695
          ThreeOfAKind Queen Jack Six        -> 5696
          ThreeOfAKind Queen Jack Seven      -> 5697
          ThreeOfAKind Queen Jack Eight      -> 5698
          ThreeOfAKind Queen Jack Nine       -> 5699
          ThreeOfAKind Queen Jack Ten        -> 5700
          ThreeOfAKind Queen King Two        -> 5701
          ThreeOfAKind Queen King Three      -> 5702
          ThreeOfAKind Queen King Four       -> 5703
          ThreeOfAKind Queen King Five       -> 5704
          ThreeOfAKind Queen King Six        -> 5705
          ThreeOfAKind Queen King Seven      -> 5706
          ThreeOfAKind Queen King Eight      -> 5707
          ThreeOfAKind Queen King Nine       -> 5708
          ThreeOfAKind Queen King Ten        -> 5709
          ThreeOfAKind Queen King Jack       -> 5710
          ThreeOfAKind Queen Ace Two         -> 5711
          ThreeOfAKind Queen Ace Three       -> 5712
          ThreeOfAKind Queen Ace Four        -> 5713
          ThreeOfAKind Queen Ace Five        -> 5714
          ThreeOfAKind Queen Ace Six         -> 5715
          ThreeOfAKind Queen Ace Seven       -> 5716
          ThreeOfAKind Queen Ace Eight       -> 5717
          ThreeOfAKind Queen Ace Nine        -> 5718
          ThreeOfAKind Queen Ace Ten         -> 5719
          ThreeOfAKind Queen Ace Jack        -> 5720
          ThreeOfAKind Queen Ace King        -> 5721
          ThreeOfAKind King Three Two        -> 5722
          ThreeOfAKind King Four Two         -> 5723
          ThreeOfAKind King Four Three       -> 5724
          ThreeOfAKind King Five Two         -> 5725
          ThreeOfAKind King Five Three       -> 5726
          ThreeOfAKind King Five Four        -> 5727
          ThreeOfAKind King Six Two          -> 5728
          ThreeOfAKind King Six Three        -> 5729
          ThreeOfAKind King Six Four         -> 5730
          ThreeOfAKind King Six Five         -> 5731
          ThreeOfAKind King Seven Two        -> 5732
          ThreeOfAKind King Seven Three      -> 5733
          ThreeOfAKind King Seven Four       -> 5734
          ThreeOfAKind King Seven Five       -> 5735
          ThreeOfAKind King Seven Six        -> 5736
          ThreeOfAKind King Eight Two        -> 5737
          ThreeOfAKind King Eight Three      -> 5738
          ThreeOfAKind King Eight Four       -> 5739
          ThreeOfAKind King Eight Five       -> 5740
          ThreeOfAKind King Eight Six        -> 5741
          ThreeOfAKind King Eight Seven      -> 5742
          ThreeOfAKind King Nine Two         -> 5743
          ThreeOfAKind King Nine Three       -> 5744
          ThreeOfAKind King Nine Four        -> 5745
          ThreeOfAKind King Nine Five        -> 5746
          ThreeOfAKind King Nine Six         -> 5747
          ThreeOfAKind King Nine Seven       -> 5748
          ThreeOfAKind King Nine Eight       -> 5749
          ThreeOfAKind King Ten Two          -> 5750
          ThreeOfAKind King Ten Three        -> 5751
          ThreeOfAKind King Ten Four         -> 5752
          ThreeOfAKind King Ten Five         -> 5753
          ThreeOfAKind King Ten Six          -> 5754
          ThreeOfAKind King Ten Seven        -> 5755
          ThreeOfAKind King Ten Eight        -> 5756
          ThreeOfAKind King Ten Nine         -> 5757
          ThreeOfAKind King Jack Two         -> 5758
          ThreeOfAKind King Jack Three       -> 5759
          ThreeOfAKind King Jack Four        -> 5760
          ThreeOfAKind King Jack Five        -> 5761
          ThreeOfAKind King Jack Six         -> 5762
          ThreeOfAKind King Jack Seven       -> 5763
          ThreeOfAKind King Jack Eight       -> 5764
          ThreeOfAKind King Jack Nine        -> 5765
          ThreeOfAKind King Jack Ten         -> 5766
          ThreeOfAKind King Queen Two        -> 5767
          ThreeOfAKind King Queen Three      -> 5768
          ThreeOfAKind King Queen Four       -> 5769
          ThreeOfAKind King Queen Five       -> 5770
          ThreeOfAKind King Queen Six        -> 5771
          ThreeOfAKind King Queen Seven      -> 5772
          ThreeOfAKind King Queen Eight      -> 5773
          ThreeOfAKind King Queen Nine       -> 5774
          ThreeOfAKind King Queen Ten        -> 5775
          ThreeOfAKind King Queen Jack       -> 5776
          ThreeOfAKind King Ace Two          -> 5777
          ThreeOfAKind King Ace Three        -> 5778
          ThreeOfAKind King Ace Four         -> 5779
          ThreeOfAKind King Ace Five         -> 5780
          ThreeOfAKind King Ace Six          -> 5781
          ThreeOfAKind King Ace Seven        -> 5782
          ThreeOfAKind King Ace Eight        -> 5783
          ThreeOfAKind King Ace Nine         -> 5784
          ThreeOfAKind King Ace Ten          -> 5785
          ThreeOfAKind King Ace Jack         -> 5786
          ThreeOfAKind King Ace Queen        -> 5787
          ThreeOfAKind Ace Three Two         -> 5788
          ThreeOfAKind Ace Four Two          -> 5789
          ThreeOfAKind Ace Four Three        -> 5790
          ThreeOfAKind Ace Five Two          -> 5791
          ThreeOfAKind Ace Five Three        -> 5792
          ThreeOfAKind Ace Five Four         -> 5793
          ThreeOfAKind Ace Six Two           -> 5794
          ThreeOfAKind Ace Six Three         -> 5795
          ThreeOfAKind Ace Six Four          -> 5796
          ThreeOfAKind Ace Six Five          -> 5797
          ThreeOfAKind Ace Seven Two         -> 5798
          ThreeOfAKind Ace Seven Three       -> 5799
          ThreeOfAKind Ace Seven Four        -> 5800
          ThreeOfAKind Ace Seven Five        -> 5801
          ThreeOfAKind Ace Seven Six         -> 5802
          ThreeOfAKind Ace Eight Two         -> 5803
          ThreeOfAKind Ace Eight Three       -> 5804
          ThreeOfAKind Ace Eight Four        -> 5805
          ThreeOfAKind Ace Eight Five        -> 5806
          ThreeOfAKind Ace Eight Six         -> 5807
          ThreeOfAKind Ace Eight Seven       -> 5808
          ThreeOfAKind Ace Nine Two          -> 5809
          ThreeOfAKind Ace Nine Three        -> 5810
          ThreeOfAKind Ace Nine Four         -> 5811
          ThreeOfAKind Ace Nine Five         -> 5812
          ThreeOfAKind Ace Nine Six          -> 5813
          ThreeOfAKind Ace Nine Seven        -> 5814
          ThreeOfAKind Ace Nine Eight        -> 5815
          ThreeOfAKind Ace Ten Two           -> 5816
          ThreeOfAKind Ace Ten Three         -> 5817
          ThreeOfAKind Ace Ten Four          -> 5818
          ThreeOfAKind Ace Ten Five          -> 5819
          ThreeOfAKind Ace Ten Six           -> 5820
          ThreeOfAKind Ace Ten Seven         -> 5821
          ThreeOfAKind Ace Ten Eight         -> 5822
          ThreeOfAKind Ace Ten Nine          -> 5823
          ThreeOfAKind Ace Jack Two          -> 5824
          ThreeOfAKind Ace Jack Three        -> 5825
          ThreeOfAKind Ace Jack Four         -> 5826
          ThreeOfAKind Ace Jack Five         -> 5827
          ThreeOfAKind Ace Jack Six          -> 5828
          ThreeOfAKind Ace Jack Seven        -> 5829
          ThreeOfAKind Ace Jack Eight        -> 5830
          ThreeOfAKind Ace Jack Nine         -> 5831
          ThreeOfAKind Ace Jack Ten          -> 5832
          ThreeOfAKind Ace Queen Two         -> 5833
          ThreeOfAKind Ace Queen Three       -> 5834
          ThreeOfAKind Ace Queen Four        -> 5835
          ThreeOfAKind Ace Queen Five        -> 5836
          ThreeOfAKind Ace Queen Six         -> 5837
          ThreeOfAKind Ace Queen Seven       -> 5838
          ThreeOfAKind Ace Queen Eight       -> 5839
          ThreeOfAKind Ace Queen Nine        -> 5840
          ThreeOfAKind Ace Queen Ten         -> 5841
          ThreeOfAKind Ace Queen Jack        -> 5842
          ThreeOfAKind Ace King Two          -> 5843
          ThreeOfAKind Ace King Three        -> 5844
          ThreeOfAKind Ace King Four         -> 5845
          ThreeOfAKind Ace King Five         -> 5846
          ThreeOfAKind Ace King Six          -> 5847
          ThreeOfAKind Ace King Seven        -> 5848
          ThreeOfAKind Ace King Eight        -> 5849
          ThreeOfAKind Ace King Nine         -> 5850
          ThreeOfAKind Ace King Ten          -> 5851
          ThreeOfAKind Ace King Jack         -> 5852
          ThreeOfAKind Ace King Queen        -> 5853
          Straight Five                      -> 5854
          Straight Six                       -> 5855
          Straight Seven                     -> 5856
          Straight Eight                     -> 5857
          Straight Nine                      -> 5858
          Straight Ten                       -> 5859
          Straight Jack                      -> 5860
          Straight Queen                     -> 5861
          Straight King                      -> 5862
          Straight Ace                       -> 5863
          Flush Six Five Four Three Two      -> 5864
          Flush Seven Five Four Three Two    -> 5865
          Flush Seven Six Four Three Two     -> 5866
          Flush Seven Six Five Three Two     -> 5867
          Flush Seven Six Five Four Two      -> 5868
          Flush Seven Six Five Four Three    -> 5869
          Flush Eight Five Four Three Two    -> 5870
          Flush Eight Six Four Three Two     -> 5871
          Flush Eight Six Five Three Two     -> 5872
          Flush Eight Six Five Four Two      -> 5873
          Flush Eight Six Five Four Three    -> 5874
          Flush Eight Seven Four Three Two   -> 5875
          Flush Eight Seven Five Three Two   -> 5876
          Flush Eight Seven Five Four Two    -> 5877
          Flush Eight Seven Five Four Three  -> 5878
          Flush Eight Seven Six Three Two    -> 5879
          Flush Eight Seven Six Four Two     -> 5880
          Flush Eight Seven Six Four Three   -> 5881
          Flush Eight Seven Six Five Two     -> 5882
          Flush Eight Seven Six Five Three   -> 5883
          Flush Eight Seven Six Five Four    -> 5884
          Flush Nine Five Four Three Two     -> 5885
          Flush Nine Six Four Three Two      -> 5886
          Flush Nine Six Five Three Two      -> 5887
          Flush Nine Six Five Four Two       -> 5888
          Flush Nine Six Five Four Three     -> 5889
          Flush Nine Seven Four Three Two    -> 5890
          Flush Nine Seven Five Three Two    -> 5891
          Flush Nine Seven Five Four Two     -> 5892
          Flush Nine Seven Five Four Three   -> 5893
          Flush Nine Seven Six Three Two     -> 5894
          Flush Nine Seven Six Four Two      -> 5895
          Flush Nine Seven Six Four Three    -> 5896
          Flush Nine Seven Six Five Two      -> 5897
          Flush Nine Seven Six Five Three    -> 5898
          Flush Nine Seven Six Five Four     -> 5899
          Flush Nine Eight Four Three Two    -> 5900
          Flush Nine Eight Five Three Two    -> 5901
          Flush Nine Eight Five Four Two     -> 5902
          Flush Nine Eight Five Four Three   -> 5903
          Flush Nine Eight Six Three Two     -> 5904
          Flush Nine Eight Six Four Two      -> 5905
          Flush Nine Eight Six Four Three    -> 5906
          Flush Nine Eight Six Five Two      -> 5907
          Flush Nine Eight Six Five Three    -> 5908
          Flush Nine Eight Six Five Four     -> 5909
          Flush Nine Eight Seven Three Two   -> 5910
          Flush Nine Eight Seven Four Two    -> 5911
          Flush Nine Eight Seven Four Three  -> 5912
          Flush Nine Eight Seven Five Two    -> 5913
          Flush Nine Eight Seven Five Three  -> 5914
          Flush Nine Eight Seven Five Four   -> 5915
          Flush Nine Eight Seven Six Two     -> 5916
          Flush Nine Eight Seven Six Three   -> 5917
          Flush Nine Eight Seven Six Four    -> 5918
          Flush Nine Eight Seven Six Five    -> 5919
          Flush Ten Five Four Three Two      -> 5920
          Flush Ten Six Four Three Two       -> 5921
          Flush Ten Six Five Three Two       -> 5922
          Flush Ten Six Five Four Two        -> 5923
          Flush Ten Six Five Four Three      -> 5924
          Flush Ten Seven Four Three Two     -> 5925
          Flush Ten Seven Five Three Two     -> 5926
          Flush Ten Seven Five Four Two      -> 5927
          Flush Ten Seven Five Four Three    -> 5928
          Flush Ten Seven Six Three Two      -> 5929
          Flush Ten Seven Six Four Two       -> 5930
          Flush Ten Seven Six Four Three     -> 5931
          Flush Ten Seven Six Five Two       -> 5932
          Flush Ten Seven Six Five Three     -> 5933
          Flush Ten Seven Six Five Four      -> 5934
          Flush Ten Eight Four Three Two     -> 5935
          Flush Ten Eight Five Three Two     -> 5936
          Flush Ten Eight Five Four Two      -> 5937
          Flush Ten Eight Five Four Three    -> 5938
          Flush Ten Eight Six Three Two      -> 5939
          Flush Ten Eight Six Four Two       -> 5940
          Flush Ten Eight Six Four Three     -> 5941
          Flush Ten Eight Six Five Two       -> 5942
          Flush Ten Eight Six Five Three     -> 5943
          Flush Ten Eight Six Five Four      -> 5944
          Flush Ten Eight Seven Three Two    -> 5945
          Flush Ten Eight Seven Four Two     -> 5946
          Flush Ten Eight Seven Four Three   -> 5947
          Flush Ten Eight Seven Five Two     -> 5948
          Flush Ten Eight Seven Five Three   -> 5949
          Flush Ten Eight Seven Five Four    -> 5950
          Flush Ten Eight Seven Six Two      -> 5951
          Flush Ten Eight Seven Six Three    -> 5952
          Flush Ten Eight Seven Six Four     -> 5953
          Flush Ten Eight Seven Six Five     -> 5954
          Flush Ten Nine Four Three Two      -> 5955
          Flush Ten Nine Five Three Two      -> 5956
          Flush Ten Nine Five Four Two       -> 5957
          Flush Ten Nine Five Four Three     -> 5958
          Flush Ten Nine Six Three Two       -> 5959
          Flush Ten Nine Six Four Two        -> 5960
          Flush Ten Nine Six Four Three      -> 5961
          Flush Ten Nine Six Five Two        -> 5962
          Flush Ten Nine Six Five Three      -> 5963
          Flush Ten Nine Six Five Four       -> 5964
          Flush Ten Nine Seven Three Two     -> 5965
          Flush Ten Nine Seven Four Two      -> 5966
          Flush Ten Nine Seven Four Three    -> 5967
          Flush Ten Nine Seven Five Two      -> 5968
          Flush Ten Nine Seven Five Three    -> 5969
          Flush Ten Nine Seven Five Four     -> 5970
          Flush Ten Nine Seven Six Two       -> 5971
          Flush Ten Nine Seven Six Three     -> 5972
          Flush Ten Nine Seven Six Four      -> 5973
          Flush Ten Nine Seven Six Five      -> 5974
          Flush Ten Nine Eight Three Two     -> 5975
          Flush Ten Nine Eight Four Two      -> 5976
          Flush Ten Nine Eight Four Three    -> 5977
          Flush Ten Nine Eight Five Two      -> 5978
          Flush Ten Nine Eight Five Three    -> 5979
          Flush Ten Nine Eight Five Four     -> 5980
          Flush Ten Nine Eight Six Two       -> 5981
          Flush Ten Nine Eight Six Three     -> 5982
          Flush Ten Nine Eight Six Four      -> 5983
          Flush Ten Nine Eight Six Five      -> 5984
          Flush Ten Nine Eight Seven Two     -> 5985
          Flush Ten Nine Eight Seven Three   -> 5986
          Flush Ten Nine Eight Seven Four    -> 5987
          Flush Ten Nine Eight Seven Five    -> 5988
          Flush Ten Nine Eight Seven Six     -> 5989
          Flush Jack Five Four Three Two     -> 5990
          Flush Jack Six Four Three Two      -> 5991
          Flush Jack Six Five Three Two      -> 5992
          Flush Jack Six Five Four Two       -> 5993
          Flush Jack Six Five Four Three     -> 5994
          Flush Jack Seven Four Three Two    -> 5995
          Flush Jack Seven Five Three Two    -> 5996
          Flush Jack Seven Five Four Two     -> 5997
          Flush Jack Seven Five Four Three   -> 5998
          Flush Jack Seven Six Three Two     -> 5999
          Flush Jack Seven Six Four Two      -> 6000
          Flush Jack Seven Six Four Three    -> 6001
          Flush Jack Seven Six Five Two      -> 6002
          Flush Jack Seven Six Five Three    -> 6003
          Flush Jack Seven Six Five Four     -> 6004
          Flush Jack Eight Four Three Two    -> 6005
          Flush Jack Eight Five Three Two    -> 6006
          Flush Jack Eight Five Four Two     -> 6007
          Flush Jack Eight Five Four Three   -> 6008
          Flush Jack Eight Six Three Two     -> 6009
          Flush Jack Eight Six Four Two      -> 6010
          Flush Jack Eight Six Four Three    -> 6011
          Flush Jack Eight Six Five Two      -> 6012
          Flush Jack Eight Six Five Three    -> 6013
          Flush Jack Eight Six Five Four     -> 6014
          Flush Jack Eight Seven Three Two   -> 6015
          Flush Jack Eight Seven Four Two    -> 6016
          Flush Jack Eight Seven Four Three  -> 6017
          Flush Jack Eight Seven Five Two    -> 6018
          Flush Jack Eight Seven Five Three  -> 6019
          Flush Jack Eight Seven Five Four   -> 6020
          Flush Jack Eight Seven Six Two     -> 6021
          Flush Jack Eight Seven Six Three   -> 6022
          Flush Jack Eight Seven Six Four    -> 6023
          Flush Jack Eight Seven Six Five    -> 6024
          Flush Jack Nine Four Three Two     -> 6025
          Flush Jack Nine Five Three Two     -> 6026
          Flush Jack Nine Five Four Two      -> 6027
          Flush Jack Nine Five Four Three    -> 6028
          Flush Jack Nine Six Three Two      -> 6029
          Flush Jack Nine Six Four Two       -> 6030
          Flush Jack Nine Six Four Three     -> 6031
          Flush Jack Nine Six Five Two       -> 6032
          Flush Jack Nine Six Five Three     -> 6033
          Flush Jack Nine Six Five Four      -> 6034
          Flush Jack Nine Seven Three Two    -> 6035
          Flush Jack Nine Seven Four Two     -> 6036
          Flush Jack Nine Seven Four Three   -> 6037
          Flush Jack Nine Seven Five Two     -> 6038
          Flush Jack Nine Seven Five Three   -> 6039
          Flush Jack Nine Seven Five Four    -> 6040
          Flush Jack Nine Seven Six Two      -> 6041
          Flush Jack Nine Seven Six Three    -> 6042
          Flush Jack Nine Seven Six Four     -> 6043
          Flush Jack Nine Seven Six Five     -> 6044
          Flush Jack Nine Eight Three Two    -> 6045
          Flush Jack Nine Eight Four Two     -> 6046
          Flush Jack Nine Eight Four Three   -> 6047
          Flush Jack Nine Eight Five Two     -> 6048
          Flush Jack Nine Eight Five Three   -> 6049
          Flush Jack Nine Eight Five Four    -> 6050
          Flush Jack Nine Eight Six Two      -> 6051
          Flush Jack Nine Eight Six Three    -> 6052
          Flush Jack Nine Eight Six Four     -> 6053
          Flush Jack Nine Eight Six Five     -> 6054
          Flush Jack Nine Eight Seven Two    -> 6055
          Flush Jack Nine Eight Seven Three  -> 6056
          Flush Jack Nine Eight Seven Four   -> 6057
          Flush Jack Nine Eight Seven Five   -> 6058
          Flush Jack Nine Eight Seven Six    -> 6059
          Flush Jack Ten Four Three Two      -> 6060
          Flush Jack Ten Five Three Two      -> 6061
          Flush Jack Ten Five Four Two       -> 6062
          Flush Jack Ten Five Four Three     -> 6063
          Flush Jack Ten Six Three Two       -> 6064
          Flush Jack Ten Six Four Two        -> 6065
          Flush Jack Ten Six Four Three      -> 6066
          Flush Jack Ten Six Five Two        -> 6067
          Flush Jack Ten Six Five Three      -> 6068
          Flush Jack Ten Six Five Four       -> 6069
          Flush Jack Ten Seven Three Two     -> 6070
          Flush Jack Ten Seven Four Two      -> 6071
          Flush Jack Ten Seven Four Three    -> 6072
          Flush Jack Ten Seven Five Two      -> 6073
          Flush Jack Ten Seven Five Three    -> 6074
          Flush Jack Ten Seven Five Four     -> 6075
          Flush Jack Ten Seven Six Two       -> 6076
          Flush Jack Ten Seven Six Three     -> 6077
          Flush Jack Ten Seven Six Four      -> 6078
          Flush Jack Ten Seven Six Five      -> 6079
          Flush Jack Ten Eight Three Two     -> 6080
          Flush Jack Ten Eight Four Two      -> 6081
          Flush Jack Ten Eight Four Three    -> 6082
          Flush Jack Ten Eight Five Two      -> 6083
          Flush Jack Ten Eight Five Three    -> 6084
          Flush Jack Ten Eight Five Four     -> 6085
          Flush Jack Ten Eight Six Two       -> 6086
          Flush Jack Ten Eight Six Three     -> 6087
          Flush Jack Ten Eight Six Four      -> 6088
          Flush Jack Ten Eight Six Five      -> 6089
          Flush Jack Ten Eight Seven Two     -> 6090
          Flush Jack Ten Eight Seven Three   -> 6091
          Flush Jack Ten Eight Seven Four    -> 6092
          Flush Jack Ten Eight Seven Five    -> 6093
          Flush Jack Ten Eight Seven Six     -> 6094
          Flush Jack Ten Nine Three Two      -> 6095
          Flush Jack Ten Nine Four Two       -> 6096
          Flush Jack Ten Nine Four Three     -> 6097
          Flush Jack Ten Nine Five Two       -> 6098
          Flush Jack Ten Nine Five Three     -> 6099
          Flush Jack Ten Nine Five Four      -> 6100
          Flush Jack Ten Nine Six Two        -> 6101
          Flush Jack Ten Nine Six Three      -> 6102
          Flush Jack Ten Nine Six Four       -> 6103
          Flush Jack Ten Nine Six Five       -> 6104
          Flush Jack Ten Nine Seven Two      -> 6105
          Flush Jack Ten Nine Seven Three    -> 6106
          Flush Jack Ten Nine Seven Four     -> 6107
          Flush Jack Ten Nine Seven Five     -> 6108
          Flush Jack Ten Nine Seven Six      -> 6109
          Flush Jack Ten Nine Eight Two      -> 6110
          Flush Jack Ten Nine Eight Three    -> 6111
          Flush Jack Ten Nine Eight Four     -> 6112
          Flush Jack Ten Nine Eight Five     -> 6113
          Flush Jack Ten Nine Eight Six      -> 6114
          Flush Jack Ten Nine Eight Seven    -> 6115
          Flush Queen Five Four Three Two    -> 6116
          Flush Queen Six Four Three Two     -> 6117
          Flush Queen Six Five Three Two     -> 6118
          Flush Queen Six Five Four Two      -> 6119
          Flush Queen Six Five Four Three    -> 6120
          Flush Queen Seven Four Three Two   -> 6121
          Flush Queen Seven Five Three Two   -> 6122
          Flush Queen Seven Five Four Two    -> 6123
          Flush Queen Seven Five Four Three  -> 6124
          Flush Queen Seven Six Three Two    -> 6125
          Flush Queen Seven Six Four Two     -> 6126
          Flush Queen Seven Six Four Three   -> 6127
          Flush Queen Seven Six Five Two     -> 6128
          Flush Queen Seven Six Five Three   -> 6129
          Flush Queen Seven Six Five Four    -> 6130
          Flush Queen Eight Four Three Two   -> 6131
          Flush Queen Eight Five Three Two   -> 6132
          Flush Queen Eight Five Four Two    -> 6133
          Flush Queen Eight Five Four Three  -> 6134
          Flush Queen Eight Six Three Two    -> 6135
          Flush Queen Eight Six Four Two     -> 6136
          Flush Queen Eight Six Four Three   -> 6137
          Flush Queen Eight Six Five Two     -> 6138
          Flush Queen Eight Six Five Three   -> 6139
          Flush Queen Eight Six Five Four    -> 6140
          Flush Queen Eight Seven Three Two  -> 6141
          Flush Queen Eight Seven Four Two   -> 6142
          Flush Queen Eight Seven Four Three -> 6143
          Flush Queen Eight Seven Five Two   -> 6144
          Flush Queen Eight Seven Five Three -> 6145
          Flush Queen Eight Seven Five Four  -> 6146
          Flush Queen Eight Seven Six Two    -> 6147
          Flush Queen Eight Seven Six Three  -> 6148
          Flush Queen Eight Seven Six Four   -> 6149
          Flush Queen Eight Seven Six Five   -> 6150
          Flush Queen Nine Four Three Two    -> 6151
          Flush Queen Nine Five Three Two    -> 6152
          Flush Queen Nine Five Four Two     -> 6153
          Flush Queen Nine Five Four Three   -> 6154
          Flush Queen Nine Six Three Two     -> 6155
          Flush Queen Nine Six Four Two      -> 6156
          Flush Queen Nine Six Four Three    -> 6157
          Flush Queen Nine Six Five Two      -> 6158
          Flush Queen Nine Six Five Three    -> 6159
          Flush Queen Nine Six Five Four     -> 6160
          Flush Queen Nine Seven Three Two   -> 6161
          Flush Queen Nine Seven Four Two    -> 6162
          Flush Queen Nine Seven Four Three  -> 6163
          Flush Queen Nine Seven Five Two    -> 6164
          Flush Queen Nine Seven Five Three  -> 6165
          Flush Queen Nine Seven Five Four   -> 6166
          Flush Queen Nine Seven Six Two     -> 6167
          Flush Queen Nine Seven Six Three   -> 6168
          Flush Queen Nine Seven Six Four    -> 6169
          Flush Queen Nine Seven Six Five    -> 6170
          Flush Queen Nine Eight Three Two   -> 6171
          Flush Queen Nine Eight Four Two    -> 6172
          Flush Queen Nine Eight Four Three  -> 6173
          Flush Queen Nine Eight Five Two    -> 6174
          Flush Queen Nine Eight Five Three  -> 6175
          Flush Queen Nine Eight Five Four   -> 6176
          Flush Queen Nine Eight Six Two     -> 6177
          Flush Queen Nine Eight Six Three   -> 6178
          Flush Queen Nine Eight Six Four    -> 6179
          Flush Queen Nine Eight Six Five    -> 6180
          Flush Queen Nine Eight Seven Two   -> 6181
          Flush Queen Nine Eight Seven Three -> 6182
          Flush Queen Nine Eight Seven Four  -> 6183
          Flush Queen Nine Eight Seven Five  -> 6184
          Flush Queen Nine Eight Seven Six   -> 6185
          Flush Queen Ten Four Three Two     -> 6186
          Flush Queen Ten Five Three Two     -> 6187
          Flush Queen Ten Five Four Two      -> 6188
          Flush Queen Ten Five Four Three    -> 6189
          Flush Queen Ten Six Three Two      -> 6190
          Flush Queen Ten Six Four Two       -> 6191
          Flush Queen Ten Six Four Three     -> 6192
          Flush Queen Ten Six Five Two       -> 6193
          Flush Queen Ten Six Five Three     -> 6194
          Flush Queen Ten Six Five Four      -> 6195
          Flush Queen Ten Seven Three Two    -> 6196
          Flush Queen Ten Seven Four Two     -> 6197
          Flush Queen Ten Seven Four Three   -> 6198
          Flush Queen Ten Seven Five Two     -> 6199
          Flush Queen Ten Seven Five Three   -> 6200
          Flush Queen Ten Seven Five Four    -> 6201
          Flush Queen Ten Seven Six Two      -> 6202
          Flush Queen Ten Seven Six Three    -> 6203
          Flush Queen Ten Seven Six Four     -> 6204
          Flush Queen Ten Seven Six Five     -> 6205
          Flush Queen Ten Eight Three Two    -> 6206
          Flush Queen Ten Eight Four Two     -> 6207
          Flush Queen Ten Eight Four Three   -> 6208
          Flush Queen Ten Eight Five Two     -> 6209
          Flush Queen Ten Eight Five Three   -> 6210
          Flush Queen Ten Eight Five Four    -> 6211
          Flush Queen Ten Eight Six Two      -> 6212
          Flush Queen Ten Eight Six Three    -> 6213
          Flush Queen Ten Eight Six Four     -> 6214
          Flush Queen Ten Eight Six Five     -> 6215
          Flush Queen Ten Eight Seven Two    -> 6216
          Flush Queen Ten Eight Seven Three  -> 6217
          Flush Queen Ten Eight Seven Four   -> 6218
          Flush Queen Ten Eight Seven Five   -> 6219
          Flush Queen Ten Eight Seven Six    -> 6220
          Flush Queen Ten Nine Three Two     -> 6221
          Flush Queen Ten Nine Four Two      -> 6222
          Flush Queen Ten Nine Four Three    -> 6223
          Flush Queen Ten Nine Five Two      -> 6224
          Flush Queen Ten Nine Five Three    -> 6225
          Flush Queen Ten Nine Five Four     -> 6226
          Flush Queen Ten Nine Six Two       -> 6227
          Flush Queen Ten Nine Six Three     -> 6228
          Flush Queen Ten Nine Six Four      -> 6229
          Flush Queen Ten Nine Six Five      -> 6230
          Flush Queen Ten Nine Seven Two     -> 6231
          Flush Queen Ten Nine Seven Three   -> 6232
          Flush Queen Ten Nine Seven Four    -> 6233
          Flush Queen Ten Nine Seven Five    -> 6234
          Flush Queen Ten Nine Seven Six     -> 6235
          Flush Queen Ten Nine Eight Two     -> 6236
          Flush Queen Ten Nine Eight Three   -> 6237
          Flush Queen Ten Nine Eight Four    -> 6238
          Flush Queen Ten Nine Eight Five    -> 6239
          Flush Queen Ten Nine Eight Six     -> 6240
          Flush Queen Ten Nine Eight Seven   -> 6241
          Flush Queen Jack Four Three Two    -> 6242
          Flush Queen Jack Five Three Two    -> 6243
          Flush Queen Jack Five Four Two     -> 6244
          Flush Queen Jack Five Four Three   -> 6245
          Flush Queen Jack Six Three Two     -> 6246
          Flush Queen Jack Six Four Two      -> 6247
          Flush Queen Jack Six Four Three    -> 6248
          Flush Queen Jack Six Five Two      -> 6249
          Flush Queen Jack Six Five Three    -> 6250
          Flush Queen Jack Six Five Four     -> 6251
          Flush Queen Jack Seven Three Two   -> 6252
          Flush Queen Jack Seven Four Two    -> 6253
          Flush Queen Jack Seven Four Three  -> 6254
          Flush Queen Jack Seven Five Two    -> 6255
          Flush Queen Jack Seven Five Three  -> 6256
          Flush Queen Jack Seven Five Four   -> 6257
          Flush Queen Jack Seven Six Two     -> 6258
          Flush Queen Jack Seven Six Three   -> 6259
          Flush Queen Jack Seven Six Four    -> 6260
          Flush Queen Jack Seven Six Five    -> 6261
          Flush Queen Jack Eight Three Two   -> 6262
          Flush Queen Jack Eight Four Two    -> 6263
          Flush Queen Jack Eight Four Three  -> 6264
          Flush Queen Jack Eight Five Two    -> 6265
          Flush Queen Jack Eight Five Three  -> 6266
          Flush Queen Jack Eight Five Four   -> 6267
          Flush Queen Jack Eight Six Two     -> 6268
          Flush Queen Jack Eight Six Three   -> 6269
          Flush Queen Jack Eight Six Four    -> 6270
          Flush Queen Jack Eight Six Five    -> 6271
          Flush Queen Jack Eight Seven Two   -> 6272
          Flush Queen Jack Eight Seven Three -> 6273
          Flush Queen Jack Eight Seven Four  -> 6274
          Flush Queen Jack Eight Seven Five  -> 6275
          Flush Queen Jack Eight Seven Six   -> 6276
          Flush Queen Jack Nine Three Two    -> 6277
          Flush Queen Jack Nine Four Two     -> 6278
          Flush Queen Jack Nine Four Three   -> 6279
          Flush Queen Jack Nine Five Two     -> 6280
          Flush Queen Jack Nine Five Three   -> 6281
          Flush Queen Jack Nine Five Four    -> 6282
          Flush Queen Jack Nine Six Two      -> 6283
          Flush Queen Jack Nine Six Three    -> 6284
          Flush Queen Jack Nine Six Four     -> 6285
          Flush Queen Jack Nine Six Five     -> 6286
          Flush Queen Jack Nine Seven Two    -> 6287
          Flush Queen Jack Nine Seven Three  -> 6288
          Flush Queen Jack Nine Seven Four   -> 6289
          Flush Queen Jack Nine Seven Five   -> 6290
          Flush Queen Jack Nine Seven Six    -> 6291
          Flush Queen Jack Nine Eight Two    -> 6292
          Flush Queen Jack Nine Eight Three  -> 6293
          Flush Queen Jack Nine Eight Four   -> 6294
          Flush Queen Jack Nine Eight Five   -> 6295
          Flush Queen Jack Nine Eight Six    -> 6296
          Flush Queen Jack Nine Eight Seven  -> 6297
          Flush Queen Jack Ten Three Two     -> 6298
          Flush Queen Jack Ten Four Two      -> 6299
          Flush Queen Jack Ten Four Three    -> 6300
          Flush Queen Jack Ten Five Two      -> 6301
          Flush Queen Jack Ten Five Three    -> 6302
          Flush Queen Jack Ten Five Four     -> 6303
          Flush Queen Jack Ten Six Two       -> 6304
          Flush Queen Jack Ten Six Three     -> 6305
          Flush Queen Jack Ten Six Four      -> 6306
          Flush Queen Jack Ten Six Five      -> 6307
          Flush Queen Jack Ten Seven Two     -> 6308
          Flush Queen Jack Ten Seven Three   -> 6309
          Flush Queen Jack Ten Seven Four    -> 6310
          Flush Queen Jack Ten Seven Five    -> 6311
          Flush Queen Jack Ten Seven Six     -> 6312
          Flush Queen Jack Ten Eight Two     -> 6313
          Flush Queen Jack Ten Eight Three   -> 6314
          Flush Queen Jack Ten Eight Four    -> 6315
          Flush Queen Jack Ten Eight Five    -> 6316
          Flush Queen Jack Ten Eight Six     -> 6317
          Flush Queen Jack Ten Eight Seven   -> 6318
          Flush Queen Jack Ten Nine Two      -> 6319
          Flush Queen Jack Ten Nine Three    -> 6320
          Flush Queen Jack Ten Nine Four     -> 6321
          Flush Queen Jack Ten Nine Five     -> 6322
          Flush Queen Jack Ten Nine Six      -> 6323
          Flush Queen Jack Ten Nine Seven    -> 6324
          Flush Queen Jack Ten Nine Eight    -> 6325
          Flush King Five Four Three Two     -> 6326
          Flush King Six Four Three Two      -> 6327
          Flush King Six Five Three Two      -> 6328
          Flush King Six Five Four Two       -> 6329
          Flush King Six Five Four Three     -> 6330
          Flush King Seven Four Three Two    -> 6331
          Flush King Seven Five Three Two    -> 6332
          Flush King Seven Five Four Two     -> 6333
          Flush King Seven Five Four Three   -> 6334
          Flush King Seven Six Three Two     -> 6335
          Flush King Seven Six Four Two      -> 6336
          Flush King Seven Six Four Three    -> 6337
          Flush King Seven Six Five Two      -> 6338
          Flush King Seven Six Five Three    -> 6339
          Flush King Seven Six Five Four     -> 6340
          Flush King Eight Four Three Two    -> 6341
          Flush King Eight Five Three Two    -> 6342
          Flush King Eight Five Four Two     -> 6343
          Flush King Eight Five Four Three   -> 6344
          Flush King Eight Six Three Two     -> 6345
          Flush King Eight Six Four Two      -> 6346
          Flush King Eight Six Four Three    -> 6347
          Flush King Eight Six Five Two      -> 6348
          Flush King Eight Six Five Three    -> 6349
          Flush King Eight Six Five Four     -> 6350
          Flush King Eight Seven Three Two   -> 6351
          Flush King Eight Seven Four Two    -> 6352
          Flush King Eight Seven Four Three  -> 6353
          Flush King Eight Seven Five Two    -> 6354
          Flush King Eight Seven Five Three  -> 6355
          Flush King Eight Seven Five Four   -> 6356
          Flush King Eight Seven Six Two     -> 6357
          Flush King Eight Seven Six Three   -> 6358
          Flush King Eight Seven Six Four    -> 6359
          Flush King Eight Seven Six Five    -> 6360
          Flush King Nine Four Three Two     -> 6361
          Flush King Nine Five Three Two     -> 6362
          Flush King Nine Five Four Two      -> 6363
          Flush King Nine Five Four Three    -> 6364
          Flush King Nine Six Three Two      -> 6365
          Flush King Nine Six Four Two       -> 6366
          Flush King Nine Six Four Three     -> 6367
          Flush King Nine Six Five Two       -> 6368
          Flush King Nine Six Five Three     -> 6369
          Flush King Nine Six Five Four      -> 6370
          Flush King Nine Seven Three Two    -> 6371
          Flush King Nine Seven Four Two     -> 6372
          Flush King Nine Seven Four Three   -> 6373
          Flush King Nine Seven Five Two     -> 6374
          Flush King Nine Seven Five Three   -> 6375
          Flush King Nine Seven Five Four    -> 6376
          Flush King Nine Seven Six Two      -> 6377
          Flush King Nine Seven Six Three    -> 6378
          Flush King Nine Seven Six Four     -> 6379
          Flush King Nine Seven Six Five     -> 6380
          Flush King Nine Eight Three Two    -> 6381
          Flush King Nine Eight Four Two     -> 6382
          Flush King Nine Eight Four Three   -> 6383
          Flush King Nine Eight Five Two     -> 6384
          Flush King Nine Eight Five Three   -> 6385
          Flush King Nine Eight Five Four    -> 6386
          Flush King Nine Eight Six Two      -> 6387
          Flush King Nine Eight Six Three    -> 6388
          Flush King Nine Eight Six Four     -> 6389
          Flush King Nine Eight Six Five     -> 6390
          Flush King Nine Eight Seven Two    -> 6391
          Flush King Nine Eight Seven Three  -> 6392
          Flush King Nine Eight Seven Four   -> 6393
          Flush King Nine Eight Seven Five   -> 6394
          Flush King Nine Eight Seven Six    -> 6395
          Flush King Ten Four Three Two      -> 6396
          Flush King Ten Five Three Two      -> 6397
          Flush King Ten Five Four Two       -> 6398
          Flush King Ten Five Four Three     -> 6399
          Flush King Ten Six Three Two       -> 6400
          Flush King Ten Six Four Two        -> 6401
          Flush King Ten Six Four Three      -> 6402
          Flush King Ten Six Five Two        -> 6403
          Flush King Ten Six Five Three      -> 6404
          Flush King Ten Six Five Four       -> 6405
          Flush King Ten Seven Three Two     -> 6406
          Flush King Ten Seven Four Two      -> 6407
          Flush King Ten Seven Four Three    -> 6408
          Flush King Ten Seven Five Two      -> 6409
          Flush King Ten Seven Five Three    -> 6410
          Flush King Ten Seven Five Four     -> 6411
          Flush King Ten Seven Six Two       -> 6412
          Flush King Ten Seven Six Three     -> 6413
          Flush King Ten Seven Six Four      -> 6414
          Flush King Ten Seven Six Five      -> 6415
          Flush King Ten Eight Three Two     -> 6416
          Flush King Ten Eight Four Two      -> 6417
          Flush King Ten Eight Four Three    -> 6418
          Flush King Ten Eight Five Two      -> 6419
          Flush King Ten Eight Five Three    -> 6420
          Flush King Ten Eight Five Four     -> 6421
          Flush King Ten Eight Six Two       -> 6422
          Flush King Ten Eight Six Three     -> 6423
          Flush King Ten Eight Six Four      -> 6424
          Flush King Ten Eight Six Five      -> 6425
          Flush King Ten Eight Seven Two     -> 6426
          Flush King Ten Eight Seven Three   -> 6427
          Flush King Ten Eight Seven Four    -> 6428
          Flush King Ten Eight Seven Five    -> 6429
          Flush King Ten Eight Seven Six     -> 6430
          Flush King Ten Nine Three Two      -> 6431
          Flush King Ten Nine Four Two       -> 6432
          Flush King Ten Nine Four Three     -> 6433
          Flush King Ten Nine Five Two       -> 6434
          Flush King Ten Nine Five Three     -> 6435
          Flush King Ten Nine Five Four      -> 6436
          Flush King Ten Nine Six Two        -> 6437
          Flush King Ten Nine Six Three      -> 6438
          Flush King Ten Nine Six Four       -> 6439
          Flush King Ten Nine Six Five       -> 6440
          Flush King Ten Nine Seven Two      -> 6441
          Flush King Ten Nine Seven Three    -> 6442
          Flush King Ten Nine Seven Four     -> 6443
          Flush King Ten Nine Seven Five     -> 6444
          Flush King Ten Nine Seven Six      -> 6445
          Flush King Ten Nine Eight Two      -> 6446
          Flush King Ten Nine Eight Three    -> 6447
          Flush King Ten Nine Eight Four     -> 6448
          Flush King Ten Nine Eight Five     -> 6449
          Flush King Ten Nine Eight Six      -> 6450
          Flush King Ten Nine Eight Seven    -> 6451
          Flush King Jack Four Three Two     -> 6452
          Flush King Jack Five Three Two     -> 6453
          Flush King Jack Five Four Two      -> 6454
          Flush King Jack Five Four Three    -> 6455
          Flush King Jack Six Three Two      -> 6456
          Flush King Jack Six Four Two       -> 6457
          Flush King Jack Six Four Three     -> 6458
          Flush King Jack Six Five Two       -> 6459
          Flush King Jack Six Five Three     -> 6460
          Flush King Jack Six Five Four      -> 6461
          Flush King Jack Seven Three Two    -> 6462
          Flush King Jack Seven Four Two     -> 6463
          Flush King Jack Seven Four Three   -> 6464
          Flush King Jack Seven Five Two     -> 6465
          Flush King Jack Seven Five Three   -> 6466
          Flush King Jack Seven Five Four    -> 6467
          Flush King Jack Seven Six Two      -> 6468
          Flush King Jack Seven Six Three    -> 6469
          Flush King Jack Seven Six Four     -> 6470
          Flush King Jack Seven Six Five     -> 6471
          Flush King Jack Eight Three Two    -> 6472
          Flush King Jack Eight Four Two     -> 6473
          Flush King Jack Eight Four Three   -> 6474
          Flush King Jack Eight Five Two     -> 6475
          Flush King Jack Eight Five Three   -> 6476
          Flush King Jack Eight Five Four    -> 6477
          Flush King Jack Eight Six Two      -> 6478
          Flush King Jack Eight Six Three    -> 6479
          Flush King Jack Eight Six Four     -> 6480
          Flush King Jack Eight Six Five     -> 6481
          Flush King Jack Eight Seven Two    -> 6482
          Flush King Jack Eight Seven Three  -> 6483
          Flush King Jack Eight Seven Four   -> 6484
          Flush King Jack Eight Seven Five   -> 6485
          Flush King Jack Eight Seven Six    -> 6486
          Flush King Jack Nine Three Two     -> 6487
          Flush King Jack Nine Four Two      -> 6488
          Flush King Jack Nine Four Three    -> 6489
          Flush King Jack Nine Five Two      -> 6490
          Flush King Jack Nine Five Three    -> 6491
          Flush King Jack Nine Five Four     -> 6492
          Flush King Jack Nine Six Two       -> 6493
          Flush King Jack Nine Six Three     -> 6494
          Flush King Jack Nine Six Four      -> 6495
          Flush King Jack Nine Six Five      -> 6496
          Flush King Jack Nine Seven Two     -> 6497
          Flush King Jack Nine Seven Three   -> 6498
          Flush King Jack Nine Seven Four    -> 6499
          Flush King Jack Nine Seven Five    -> 6500
          Flush King Jack Nine Seven Six     -> 6501
          Flush King Jack Nine Eight Two     -> 6502
          Flush King Jack Nine Eight Three   -> 6503
          Flush King Jack Nine Eight Four    -> 6504
          Flush King Jack Nine Eight Five    -> 6505
          Flush King Jack Nine Eight Six     -> 6506
          Flush King Jack Nine Eight Seven   -> 6507
          Flush King Jack Ten Three Two      -> 6508
          Flush King Jack Ten Four Two       -> 6509
          Flush King Jack Ten Four Three     -> 6510
          Flush King Jack Ten Five Two       -> 6511
          Flush King Jack Ten Five Three     -> 6512
          Flush King Jack Ten Five Four      -> 6513
          Flush King Jack Ten Six Two        -> 6514
          Flush King Jack Ten Six Three      -> 6515
          Flush King Jack Ten Six Four       -> 6516
          Flush King Jack Ten Six Five       -> 6517
          Flush King Jack Ten Seven Two      -> 6518
          Flush King Jack Ten Seven Three    -> 6519
          Flush King Jack Ten Seven Four     -> 6520
          Flush King Jack Ten Seven Five     -> 6521
          Flush King Jack Ten Seven Six      -> 6522
          Flush King Jack Ten Eight Two      -> 6523
          Flush King Jack Ten Eight Three    -> 6524
          Flush King Jack Ten Eight Four     -> 6525
          Flush King Jack Ten Eight Five     -> 6526
          Flush King Jack Ten Eight Six      -> 6527
          Flush King Jack Ten Eight Seven    -> 6528
          Flush King Jack Ten Nine Two       -> 6529
          Flush King Jack Ten Nine Three     -> 6530
          Flush King Jack Ten Nine Four      -> 6531
          Flush King Jack Ten Nine Five      -> 6532
          Flush King Jack Ten Nine Six       -> 6533
          Flush King Jack Ten Nine Seven     -> 6534
          Flush King Jack Ten Nine Eight     -> 6535
          Flush King Queen Four Three Two    -> 6536
          Flush King Queen Five Three Two    -> 6537
          Flush King Queen Five Four Two     -> 6538
          Flush King Queen Five Four Three   -> 6539
          Flush King Queen Six Three Two     -> 6540
          Flush King Queen Six Four Two      -> 6541
          Flush King Queen Six Four Three    -> 6542
          Flush King Queen Six Five Two      -> 6543
          Flush King Queen Six Five Three    -> 6544
          Flush King Queen Six Five Four     -> 6545
          Flush King Queen Seven Three Two   -> 6546
          Flush King Queen Seven Four Two    -> 6547
          Flush King Queen Seven Four Three  -> 6548
          Flush King Queen Seven Five Two    -> 6549
          Flush King Queen Seven Five Three  -> 6550
          Flush King Queen Seven Five Four   -> 6551
          Flush King Queen Seven Six Two     -> 6552
          Flush King Queen Seven Six Three   -> 6553
          Flush King Queen Seven Six Four    -> 6554
          Flush King Queen Seven Six Five    -> 6555
          Flush King Queen Eight Three Two   -> 6556
          Flush King Queen Eight Four Two    -> 6557
          Flush King Queen Eight Four Three  -> 6558
          Flush King Queen Eight Five Two    -> 6559
          Flush King Queen Eight Five Three  -> 6560
          Flush King Queen Eight Five Four   -> 6561
          Flush King Queen Eight Six Two     -> 6562
          Flush King Queen Eight Six Three   -> 6563
          Flush King Queen Eight Six Four    -> 6564
          Flush King Queen Eight Six Five    -> 6565
          Flush King Queen Eight Seven Two   -> 6566
          Flush King Queen Eight Seven Three -> 6567
          Flush King Queen Eight Seven Four  -> 6568
          Flush King Queen Eight Seven Five  -> 6569
          Flush King Queen Eight Seven Six   -> 6570
          Flush King Queen Nine Three Two    -> 6571
          Flush King Queen Nine Four Two     -> 6572
          Flush King Queen Nine Four Three   -> 6573
          Flush King Queen Nine Five Two     -> 6574
          Flush King Queen Nine Five Three   -> 6575
          Flush King Queen Nine Five Four    -> 6576
          Flush King Queen Nine Six Two      -> 6577
          Flush King Queen Nine Six Three    -> 6578
          Flush King Queen Nine Six Four     -> 6579
          Flush King Queen Nine Six Five     -> 6580
          Flush King Queen Nine Seven Two    -> 6581
          Flush King Queen Nine Seven Three  -> 6582
          Flush King Queen Nine Seven Four   -> 6583
          Flush King Queen Nine Seven Five   -> 6584
          Flush King Queen Nine Seven Six    -> 6585
          Flush King Queen Nine Eight Two    -> 6586
          Flush King Queen Nine Eight Three  -> 6587
          Flush King Queen Nine Eight Four   -> 6588
          Flush King Queen Nine Eight Five   -> 6589
          Flush King Queen Nine Eight Six    -> 6590
          Flush King Queen Nine Eight Seven  -> 6591
          Flush King Queen Ten Three Two     -> 6592
          Flush King Queen Ten Four Two      -> 6593
          Flush King Queen Ten Four Three    -> 6594
          Flush King Queen Ten Five Two      -> 6595
          Flush King Queen Ten Five Three    -> 6596
          Flush King Queen Ten Five Four     -> 6597
          Flush King Queen Ten Six Two       -> 6598
          Flush King Queen Ten Six Three     -> 6599
          Flush King Queen Ten Six Four      -> 6600
          Flush King Queen Ten Six Five      -> 6601
          Flush King Queen Ten Seven Two     -> 6602
          Flush King Queen Ten Seven Three   -> 6603
          Flush King Queen Ten Seven Four    -> 6604
          Flush King Queen Ten Seven Five    -> 6605
          Flush King Queen Ten Seven Six     -> 6606
          Flush King Queen Ten Eight Two     -> 6607
          Flush King Queen Ten Eight Three   -> 6608
          Flush King Queen Ten Eight Four    -> 6609
          Flush King Queen Ten Eight Five    -> 6610
          Flush King Queen Ten Eight Six     -> 6611
          Flush King Queen Ten Eight Seven   -> 6612
          Flush King Queen Ten Nine Two      -> 6613
          Flush King Queen Ten Nine Three    -> 6614
          Flush King Queen Ten Nine Four     -> 6615
          Flush King Queen Ten Nine Five     -> 6616
          Flush King Queen Ten Nine Six      -> 6617
          Flush King Queen Ten Nine Seven    -> 6618
          Flush King Queen Ten Nine Eight    -> 6619
          Flush King Queen Jack Three Two    -> 6620
          Flush King Queen Jack Four Two     -> 6621
          Flush King Queen Jack Four Three   -> 6622
          Flush King Queen Jack Five Two     -> 6623
          Flush King Queen Jack Five Three   -> 6624
          Flush King Queen Jack Five Four    -> 6625
          Flush King Queen Jack Six Two      -> 6626
          Flush King Queen Jack Six Three    -> 6627
          Flush King Queen Jack Six Four     -> 6628
          Flush King Queen Jack Six Five     -> 6629
          Flush King Queen Jack Seven Two    -> 6630
          Flush King Queen Jack Seven Three  -> 6631
          Flush King Queen Jack Seven Four   -> 6632
          Flush King Queen Jack Seven Five   -> 6633
          Flush King Queen Jack Seven Six    -> 6634
          Flush King Queen Jack Eight Two    -> 6635
          Flush King Queen Jack Eight Three  -> 6636
          Flush King Queen Jack Eight Four   -> 6637
          Flush King Queen Jack Eight Five   -> 6638
          Flush King Queen Jack Eight Six    -> 6639
          Flush King Queen Jack Eight Seven  -> 6640
          Flush King Queen Jack Nine Two     -> 6641
          Flush King Queen Jack Nine Three   -> 6642
          Flush King Queen Jack Nine Four    -> 6643
          Flush King Queen Jack Nine Five    -> 6644
          Flush King Queen Jack Nine Six     -> 6645
          Flush King Queen Jack Nine Seven   -> 6646
          Flush King Queen Jack Nine Eight   -> 6647
          Flush King Queen Jack Ten Two      -> 6648
          Flush King Queen Jack Ten Three    -> 6649
          Flush King Queen Jack Ten Four     -> 6650
          Flush King Queen Jack Ten Five     -> 6651
          Flush King Queen Jack Ten Six      -> 6652
          Flush King Queen Jack Ten Seven    -> 6653
          Flush King Queen Jack Ten Eight    -> 6654
          Flush King Queen Jack Ten Nine     -> 6655
          Flush Ace Five Four Three Two      -> 6656
          Flush Ace Six Four Three Two       -> 6657
          Flush Ace Six Five Three Two       -> 6658
          Flush Ace Six Five Four Two        -> 6659
          Flush Ace Six Five Four Three      -> 6660
          Flush Ace Seven Four Three Two     -> 6661
          Flush Ace Seven Five Three Two     -> 6662
          Flush Ace Seven Five Four Two      -> 6663
          Flush Ace Seven Five Four Three    -> 6664
          Flush Ace Seven Six Three Two      -> 6665
          Flush Ace Seven Six Four Two       -> 6666
          Flush Ace Seven Six Four Three     -> 6667
          Flush Ace Seven Six Five Two       -> 6668
          Flush Ace Seven Six Five Three     -> 6669
          Flush Ace Seven Six Five Four      -> 6670
          Flush Ace Eight Four Three Two     -> 6671
          Flush Ace Eight Five Three Two     -> 6672
          Flush Ace Eight Five Four Two      -> 6673
          Flush Ace Eight Five Four Three    -> 6674
          Flush Ace Eight Six Three Two      -> 6675
          Flush Ace Eight Six Four Two       -> 6676
          Flush Ace Eight Six Four Three     -> 6677
          Flush Ace Eight Six Five Two       -> 6678
          Flush Ace Eight Six Five Three     -> 6679
          Flush Ace Eight Six Five Four      -> 6680
          Flush Ace Eight Seven Three Two    -> 6681
          Flush Ace Eight Seven Four Two     -> 6682
          Flush Ace Eight Seven Four Three   -> 6683
          Flush Ace Eight Seven Five Two     -> 6684
          Flush Ace Eight Seven Five Three   -> 6685
          Flush Ace Eight Seven Five Four    -> 6686
          Flush Ace Eight Seven Six Two      -> 6687
          Flush Ace Eight Seven Six Three    -> 6688
          Flush Ace Eight Seven Six Four     -> 6689
          Flush Ace Eight Seven Six Five     -> 6690
          Flush Ace Nine Four Three Two      -> 6691
          Flush Ace Nine Five Three Two      -> 6692
          Flush Ace Nine Five Four Two       -> 6693
          Flush Ace Nine Five Four Three     -> 6694
          Flush Ace Nine Six Three Two       -> 6695
          Flush Ace Nine Six Four Two        -> 6696
          Flush Ace Nine Six Four Three      -> 6697
          Flush Ace Nine Six Five Two        -> 6698
          Flush Ace Nine Six Five Three      -> 6699
          Flush Ace Nine Six Five Four       -> 6700
          Flush Ace Nine Seven Three Two     -> 6701
          Flush Ace Nine Seven Four Two      -> 6702
          Flush Ace Nine Seven Four Three    -> 6703
          Flush Ace Nine Seven Five Two      -> 6704
          Flush Ace Nine Seven Five Three    -> 6705
          Flush Ace Nine Seven Five Four     -> 6706
          Flush Ace Nine Seven Six Two       -> 6707
          Flush Ace Nine Seven Six Three     -> 6708
          Flush Ace Nine Seven Six Four      -> 6709
          Flush Ace Nine Seven Six Five      -> 6710
          Flush Ace Nine Eight Three Two     -> 6711
          Flush Ace Nine Eight Four Two      -> 6712
          Flush Ace Nine Eight Four Three    -> 6713
          Flush Ace Nine Eight Five Two      -> 6714
          Flush Ace Nine Eight Five Three    -> 6715
          Flush Ace Nine Eight Five Four     -> 6716
          Flush Ace Nine Eight Six Two       -> 6717
          Flush Ace Nine Eight Six Three     -> 6718
          Flush Ace Nine Eight Six Four      -> 6719
          Flush Ace Nine Eight Six Five      -> 6720
          Flush Ace Nine Eight Seven Two     -> 6721
          Flush Ace Nine Eight Seven Three   -> 6722
          Flush Ace Nine Eight Seven Four    -> 6723
          Flush Ace Nine Eight Seven Five    -> 6724
          Flush Ace Nine Eight Seven Six     -> 6725
          Flush Ace Ten Four Three Two       -> 6726
          Flush Ace Ten Five Three Two       -> 6727
          Flush Ace Ten Five Four Two        -> 6728
          Flush Ace Ten Five Four Three      -> 6729
          Flush Ace Ten Six Three Two        -> 6730
          Flush Ace Ten Six Four Two         -> 6731
          Flush Ace Ten Six Four Three       -> 6732
          Flush Ace Ten Six Five Two         -> 6733
          Flush Ace Ten Six Five Three       -> 6734
          Flush Ace Ten Six Five Four        -> 6735
          Flush Ace Ten Seven Three Two      -> 6736
          Flush Ace Ten Seven Four Two       -> 6737
          Flush Ace Ten Seven Four Three     -> 6738
          Flush Ace Ten Seven Five Two       -> 6739
          Flush Ace Ten Seven Five Three     -> 6740
          Flush Ace Ten Seven Five Four      -> 6741
          Flush Ace Ten Seven Six Two        -> 6742
          Flush Ace Ten Seven Six Three      -> 6743
          Flush Ace Ten Seven Six Four       -> 6744
          Flush Ace Ten Seven Six Five       -> 6745
          Flush Ace Ten Eight Three Two      -> 6746
          Flush Ace Ten Eight Four Two       -> 6747
          Flush Ace Ten Eight Four Three     -> 6748
          Flush Ace Ten Eight Five Two       -> 6749
          Flush Ace Ten Eight Five Three     -> 6750
          Flush Ace Ten Eight Five Four      -> 6751
          Flush Ace Ten Eight Six Two        -> 6752
          Flush Ace Ten Eight Six Three      -> 6753
          Flush Ace Ten Eight Six Four       -> 6754
          Flush Ace Ten Eight Six Five       -> 6755
          Flush Ace Ten Eight Seven Two      -> 6756
          Flush Ace Ten Eight Seven Three    -> 6757
          Flush Ace Ten Eight Seven Four     -> 6758
          Flush Ace Ten Eight Seven Five     -> 6759
          Flush Ace Ten Eight Seven Six      -> 6760
          Flush Ace Ten Nine Three Two       -> 6761
          Flush Ace Ten Nine Four Two        -> 6762
          Flush Ace Ten Nine Four Three      -> 6763
          Flush Ace Ten Nine Five Two        -> 6764
          Flush Ace Ten Nine Five Three      -> 6765
          Flush Ace Ten Nine Five Four       -> 6766
          Flush Ace Ten Nine Six Two         -> 6767
          Flush Ace Ten Nine Six Three       -> 6768
          Flush Ace Ten Nine Six Four        -> 6769
          Flush Ace Ten Nine Six Five        -> 6770
          Flush Ace Ten Nine Seven Two       -> 6771
          Flush Ace Ten Nine Seven Three     -> 6772
          Flush Ace Ten Nine Seven Four      -> 6773
          Flush Ace Ten Nine Seven Five      -> 6774
          Flush Ace Ten Nine Seven Six       -> 6775
          Flush Ace Ten Nine Eight Two       -> 6776
          Flush Ace Ten Nine Eight Three     -> 6777
          Flush Ace Ten Nine Eight Four      -> 6778
          Flush Ace Ten Nine Eight Five      -> 6779
          Flush Ace Ten Nine Eight Six       -> 6780
          Flush Ace Ten Nine Eight Seven     -> 6781
          Flush Ace Jack Four Three Two      -> 6782
          Flush Ace Jack Five Three Two      -> 6783
          Flush Ace Jack Five Four Two       -> 6784
          Flush Ace Jack Five Four Three     -> 6785
          Flush Ace Jack Six Three Two       -> 6786
          Flush Ace Jack Six Four Two        -> 6787
          Flush Ace Jack Six Four Three      -> 6788
          Flush Ace Jack Six Five Two        -> 6789
          Flush Ace Jack Six Five Three      -> 6790
          Flush Ace Jack Six Five Four       -> 6791
          Flush Ace Jack Seven Three Two     -> 6792
          Flush Ace Jack Seven Four Two      -> 6793
          Flush Ace Jack Seven Four Three    -> 6794
          Flush Ace Jack Seven Five Two      -> 6795
          Flush Ace Jack Seven Five Three    -> 6796
          Flush Ace Jack Seven Five Four     -> 6797
          Flush Ace Jack Seven Six Two       -> 6798
          Flush Ace Jack Seven Six Three     -> 6799
          Flush Ace Jack Seven Six Four      -> 6800
          Flush Ace Jack Seven Six Five      -> 6801
          Flush Ace Jack Eight Three Two     -> 6802
          Flush Ace Jack Eight Four Two      -> 6803
          Flush Ace Jack Eight Four Three    -> 6804
          Flush Ace Jack Eight Five Two      -> 6805
          Flush Ace Jack Eight Five Three    -> 6806
          Flush Ace Jack Eight Five Four     -> 6807
          Flush Ace Jack Eight Six Two       -> 6808
          Flush Ace Jack Eight Six Three     -> 6809
          Flush Ace Jack Eight Six Four      -> 6810
          Flush Ace Jack Eight Six Five      -> 6811
          Flush Ace Jack Eight Seven Two     -> 6812
          Flush Ace Jack Eight Seven Three   -> 6813
          Flush Ace Jack Eight Seven Four    -> 6814
          Flush Ace Jack Eight Seven Five    -> 6815
          Flush Ace Jack Eight Seven Six     -> 6816
          Flush Ace Jack Nine Three Two      -> 6817
          Flush Ace Jack Nine Four Two       -> 6818
          Flush Ace Jack Nine Four Three     -> 6819
          Flush Ace Jack Nine Five Two       -> 6820
          Flush Ace Jack Nine Five Three     -> 6821
          Flush Ace Jack Nine Five Four      -> 6822
          Flush Ace Jack Nine Six Two        -> 6823
          Flush Ace Jack Nine Six Three      -> 6824
          Flush Ace Jack Nine Six Four       -> 6825
          Flush Ace Jack Nine Six Five       -> 6826
          Flush Ace Jack Nine Seven Two      -> 6827
          Flush Ace Jack Nine Seven Three    -> 6828
          Flush Ace Jack Nine Seven Four     -> 6829
          Flush Ace Jack Nine Seven Five     -> 6830
          Flush Ace Jack Nine Seven Six      -> 6831
          Flush Ace Jack Nine Eight Two      -> 6832
          Flush Ace Jack Nine Eight Three    -> 6833
          Flush Ace Jack Nine Eight Four     -> 6834
          Flush Ace Jack Nine Eight Five     -> 6835
          Flush Ace Jack Nine Eight Six      -> 6836
          Flush Ace Jack Nine Eight Seven    -> 6837
          Flush Ace Jack Ten Three Two       -> 6838
          Flush Ace Jack Ten Four Two        -> 6839
          Flush Ace Jack Ten Four Three      -> 6840
          Flush Ace Jack Ten Five Two        -> 6841
          Flush Ace Jack Ten Five Three      -> 6842
          Flush Ace Jack Ten Five Four       -> 6843
          Flush Ace Jack Ten Six Two         -> 6844
          Flush Ace Jack Ten Six Three       -> 6845
          Flush Ace Jack Ten Six Four        -> 6846
          Flush Ace Jack Ten Six Five        -> 6847
          Flush Ace Jack Ten Seven Two       -> 6848
          Flush Ace Jack Ten Seven Three     -> 6849
          Flush Ace Jack Ten Seven Four      -> 6850
          Flush Ace Jack Ten Seven Five      -> 6851
          Flush Ace Jack Ten Seven Six       -> 6852
          Flush Ace Jack Ten Eight Two       -> 6853
          Flush Ace Jack Ten Eight Three     -> 6854
          Flush Ace Jack Ten Eight Four      -> 6855
          Flush Ace Jack Ten Eight Five      -> 6856
          Flush Ace Jack Ten Eight Six       -> 6857
          Flush Ace Jack Ten Eight Seven     -> 6858
          Flush Ace Jack Ten Nine Two        -> 6859
          Flush Ace Jack Ten Nine Three      -> 6860
          Flush Ace Jack Ten Nine Four       -> 6861
          Flush Ace Jack Ten Nine Five       -> 6862
          Flush Ace Jack Ten Nine Six        -> 6863
          Flush Ace Jack Ten Nine Seven      -> 6864
          Flush Ace Jack Ten Nine Eight      -> 6865
          Flush Ace Queen Four Three Two     -> 6866
          Flush Ace Queen Five Three Two     -> 6867
          Flush Ace Queen Five Four Two      -> 6868
          Flush Ace Queen Five Four Three    -> 6869
          Flush Ace Queen Six Three Two      -> 6870
          Flush Ace Queen Six Four Two       -> 6871
          Flush Ace Queen Six Four Three     -> 6872
          Flush Ace Queen Six Five Two       -> 6873
          Flush Ace Queen Six Five Three     -> 6874
          Flush Ace Queen Six Five Four      -> 6875
          Flush Ace Queen Seven Three Two    -> 6876
          Flush Ace Queen Seven Four Two     -> 6877
          Flush Ace Queen Seven Four Three   -> 6878
          Flush Ace Queen Seven Five Two     -> 6879
          Flush Ace Queen Seven Five Three   -> 6880
          Flush Ace Queen Seven Five Four    -> 6881
          Flush Ace Queen Seven Six Two      -> 6882
          Flush Ace Queen Seven Six Three    -> 6883
          Flush Ace Queen Seven Six Four     -> 6884
          Flush Ace Queen Seven Six Five     -> 6885
          Flush Ace Queen Eight Three Two    -> 6886
          Flush Ace Queen Eight Four Two     -> 6887
          Flush Ace Queen Eight Four Three   -> 6888
          Flush Ace Queen Eight Five Two     -> 6889
          Flush Ace Queen Eight Five Three   -> 6890
          Flush Ace Queen Eight Five Four    -> 6891
          Flush Ace Queen Eight Six Two      -> 6892
          Flush Ace Queen Eight Six Three    -> 6893
          Flush Ace Queen Eight Six Four     -> 6894
          Flush Ace Queen Eight Six Five     -> 6895
          Flush Ace Queen Eight Seven Two    -> 6896
          Flush Ace Queen Eight Seven Three  -> 6897
          Flush Ace Queen Eight Seven Four   -> 6898
          Flush Ace Queen Eight Seven Five   -> 6899
          Flush Ace Queen Eight Seven Six    -> 6900
          Flush Ace Queen Nine Three Two     -> 6901
          Flush Ace Queen Nine Four Two      -> 6902
          Flush Ace Queen Nine Four Three    -> 6903
          Flush Ace Queen Nine Five Two      -> 6904
          Flush Ace Queen Nine Five Three    -> 6905
          Flush Ace Queen Nine Five Four     -> 6906
          Flush Ace Queen Nine Six Two       -> 6907
          Flush Ace Queen Nine Six Three     -> 6908
          Flush Ace Queen Nine Six Four      -> 6909
          Flush Ace Queen Nine Six Five      -> 6910
          Flush Ace Queen Nine Seven Two     -> 6911
          Flush Ace Queen Nine Seven Three   -> 6912
          Flush Ace Queen Nine Seven Four    -> 6913
          Flush Ace Queen Nine Seven Five    -> 6914
          Flush Ace Queen Nine Seven Six     -> 6915
          Flush Ace Queen Nine Eight Two     -> 6916
          Flush Ace Queen Nine Eight Three   -> 6917
          Flush Ace Queen Nine Eight Four    -> 6918
          Flush Ace Queen Nine Eight Five    -> 6919
          Flush Ace Queen Nine Eight Six     -> 6920
          Flush Ace Queen Nine Eight Seven   -> 6921
          Flush Ace Queen Ten Three Two      -> 6922
          Flush Ace Queen Ten Four Two       -> 6923
          Flush Ace Queen Ten Four Three     -> 6924
          Flush Ace Queen Ten Five Two       -> 6925
          Flush Ace Queen Ten Five Three     -> 6926
          Flush Ace Queen Ten Five Four      -> 6927
          Flush Ace Queen Ten Six Two        -> 6928
          Flush Ace Queen Ten Six Three      -> 6929
          Flush Ace Queen Ten Six Four       -> 6930
          Flush Ace Queen Ten Six Five       -> 6931
          Flush Ace Queen Ten Seven Two      -> 6932
          Flush Ace Queen Ten Seven Three    -> 6933
          Flush Ace Queen Ten Seven Four     -> 6934
          Flush Ace Queen Ten Seven Five     -> 6935
          Flush Ace Queen Ten Seven Six      -> 6936
          Flush Ace Queen Ten Eight Two      -> 6937
          Flush Ace Queen Ten Eight Three    -> 6938
          Flush Ace Queen Ten Eight Four     -> 6939
          Flush Ace Queen Ten Eight Five     -> 6940
          Flush Ace Queen Ten Eight Six      -> 6941
          Flush Ace Queen Ten Eight Seven    -> 6942
          Flush Ace Queen Ten Nine Two       -> 6943
          Flush Ace Queen Ten Nine Three     -> 6944
          Flush Ace Queen Ten Nine Four      -> 6945
          Flush Ace Queen Ten Nine Five      -> 6946
          Flush Ace Queen Ten Nine Six       -> 6947
          Flush Ace Queen Ten Nine Seven     -> 6948
          Flush Ace Queen Ten Nine Eight     -> 6949
          Flush Ace Queen Jack Three Two     -> 6950
          Flush Ace Queen Jack Four Two      -> 6951
          Flush Ace Queen Jack Four Three    -> 6952
          Flush Ace Queen Jack Five Two      -> 6953
          Flush Ace Queen Jack Five Three    -> 6954
          Flush Ace Queen Jack Five Four     -> 6955
          Flush Ace Queen Jack Six Two       -> 6956
          Flush Ace Queen Jack Six Three     -> 6957
          Flush Ace Queen Jack Six Four      -> 6958
          Flush Ace Queen Jack Six Five      -> 6959
          Flush Ace Queen Jack Seven Two     -> 6960
          Flush Ace Queen Jack Seven Three   -> 6961
          Flush Ace Queen Jack Seven Four    -> 6962
          Flush Ace Queen Jack Seven Five    -> 6963
          Flush Ace Queen Jack Seven Six     -> 6964
          Flush Ace Queen Jack Eight Two     -> 6965
          Flush Ace Queen Jack Eight Three   -> 6966
          Flush Ace Queen Jack Eight Four    -> 6967
          Flush Ace Queen Jack Eight Five    -> 6968
          Flush Ace Queen Jack Eight Six     -> 6969
          Flush Ace Queen Jack Eight Seven   -> 6970
          Flush Ace Queen Jack Nine Two      -> 6971
          Flush Ace Queen Jack Nine Three    -> 6972
          Flush Ace Queen Jack Nine Four     -> 6973
          Flush Ace Queen Jack Nine Five     -> 6974
          Flush Ace Queen Jack Nine Six      -> 6975
          Flush Ace Queen Jack Nine Seven    -> 6976
          Flush Ace Queen Jack Nine Eight    -> 6977
          Flush Ace Queen Jack Ten Two       -> 6978
          Flush Ace Queen Jack Ten Three     -> 6979
          Flush Ace Queen Jack Ten Four      -> 6980
          Flush Ace Queen Jack Ten Five      -> 6981
          Flush Ace Queen Jack Ten Six       -> 6982
          Flush Ace Queen Jack Ten Seven     -> 6983
          Flush Ace Queen Jack Ten Eight     -> 6984
          Flush Ace Queen Jack Ten Nine      -> 6985
          Flush Ace King Four Three Two      -> 6986
          Flush Ace King Five Three Two      -> 6987
          Flush Ace King Five Four Two       -> 6988
          Flush Ace King Five Four Three     -> 6989
          Flush Ace King Six Three Two       -> 6990
          Flush Ace King Six Four Two        -> 6991
          Flush Ace King Six Four Three      -> 6992
          Flush Ace King Six Five Two        -> 6993
          Flush Ace King Six Five Three      -> 6994
          Flush Ace King Six Five Four       -> 6995
          Flush Ace King Seven Three Two     -> 6996
          Flush Ace King Seven Four Two      -> 6997
          Flush Ace King Seven Four Three    -> 6998
          Flush Ace King Seven Five Two      -> 6999
          Flush Ace King Seven Five Three    -> 7000
          Flush Ace King Seven Five Four     -> 7001
          Flush Ace King Seven Six Two       -> 7002
          Flush Ace King Seven Six Three     -> 7003
          Flush Ace King Seven Six Four      -> 7004
          Flush Ace King Seven Six Five      -> 7005
          Flush Ace King Eight Three Two     -> 7006
          Flush Ace King Eight Four Two      -> 7007
          Flush Ace King Eight Four Three    -> 7008
          Flush Ace King Eight Five Two      -> 7009
          Flush Ace King Eight Five Three    -> 7010
          Flush Ace King Eight Five Four     -> 7011
          Flush Ace King Eight Six Two       -> 7012
          Flush Ace King Eight Six Three     -> 7013
          Flush Ace King Eight Six Four      -> 7014
          Flush Ace King Eight Six Five      -> 7015
          Flush Ace King Eight Seven Two     -> 7016
          Flush Ace King Eight Seven Three   -> 7017
          Flush Ace King Eight Seven Four    -> 7018
          Flush Ace King Eight Seven Five    -> 7019
          Flush Ace King Eight Seven Six     -> 7020
          Flush Ace King Nine Three Two      -> 7021
          Flush Ace King Nine Four Two       -> 7022
          Flush Ace King Nine Four Three     -> 7023
          Flush Ace King Nine Five Two       -> 7024
          Flush Ace King Nine Five Three     -> 7025
          Flush Ace King Nine Five Four      -> 7026
          Flush Ace King Nine Six Two        -> 7027
          Flush Ace King Nine Six Three      -> 7028
          Flush Ace King Nine Six Four       -> 7029
          Flush Ace King Nine Six Five       -> 7030
          Flush Ace King Nine Seven Two      -> 7031
          Flush Ace King Nine Seven Three    -> 7032
          Flush Ace King Nine Seven Four     -> 7033
          Flush Ace King Nine Seven Five     -> 7034
          Flush Ace King Nine Seven Six      -> 7035
          Flush Ace King Nine Eight Two      -> 7036
          Flush Ace King Nine Eight Three    -> 7037
          Flush Ace King Nine Eight Four     -> 7038
          Flush Ace King Nine Eight Five     -> 7039
          Flush Ace King Nine Eight Six      -> 7040
          Flush Ace King Nine Eight Seven    -> 7041
          Flush Ace King Ten Three Two       -> 7042
          Flush Ace King Ten Four Two        -> 7043
          Flush Ace King Ten Four Three      -> 7044
          Flush Ace King Ten Five Two        -> 7045
          Flush Ace King Ten Five Three      -> 7046
          Flush Ace King Ten Five Four       -> 7047
          Flush Ace King Ten Six Two         -> 7048
          Flush Ace King Ten Six Three       -> 7049
          Flush Ace King Ten Six Four        -> 7050
          Flush Ace King Ten Six Five        -> 7051
          Flush Ace King Ten Seven Two       -> 7052
          Flush Ace King Ten Seven Three     -> 7053
          Flush Ace King Ten Seven Four      -> 7054
          Flush Ace King Ten Seven Five      -> 7055
          Flush Ace King Ten Seven Six       -> 7056
          Flush Ace King Ten Eight Two       -> 7057
          Flush Ace King Ten Eight Three     -> 7058
          Flush Ace King Ten Eight Four      -> 7059
          Flush Ace King Ten Eight Five      -> 7060
          Flush Ace King Ten Eight Six       -> 7061
          Flush Ace King Ten Eight Seven     -> 7062
          Flush Ace King Ten Nine Two        -> 7063
          Flush Ace King Ten Nine Three      -> 7064
          Flush Ace King Ten Nine Four       -> 7065
          Flush Ace King Ten Nine Five       -> 7066
          Flush Ace King Ten Nine Six        -> 7067
          Flush Ace King Ten Nine Seven      -> 7068
          Flush Ace King Ten Nine Eight      -> 7069
          Flush Ace King Jack Three Two      -> 7070
          Flush Ace King Jack Four Two       -> 7071
          Flush Ace King Jack Four Three     -> 7072
          Flush Ace King Jack Five Two       -> 7073
          Flush Ace King Jack Five Three     -> 7074
          Flush Ace King Jack Five Four      -> 7075
          Flush Ace King Jack Six Two        -> 7076
          Flush Ace King Jack Six Three      -> 7077
          Flush Ace King Jack Six Four       -> 7078
          Flush Ace King Jack Six Five       -> 7079
          Flush Ace King Jack Seven Two      -> 7080
          Flush Ace King Jack Seven Three    -> 7081
          Flush Ace King Jack Seven Four     -> 7082
          Flush Ace King Jack Seven Five     -> 7083
          Flush Ace King Jack Seven Six      -> 7084
          Flush Ace King Jack Eight Two      -> 7085
          Flush Ace King Jack Eight Three    -> 7086
          Flush Ace King Jack Eight Four     -> 7087
          Flush Ace King Jack Eight Five     -> 7088
          Flush Ace King Jack Eight Six      -> 7089
          Flush Ace King Jack Eight Seven    -> 7090
          Flush Ace King Jack Nine Two       -> 7091
          Flush Ace King Jack Nine Three     -> 7092
          Flush Ace King Jack Nine Four      -> 7093
          Flush Ace King Jack Nine Five      -> 7094
          Flush Ace King Jack Nine Six       -> 7095
          Flush Ace King Jack Nine Seven     -> 7096
          Flush Ace King Jack Nine Eight     -> 7097
          Flush Ace King Jack Ten Two        -> 7098
          Flush Ace King Jack Ten Three      -> 7099
          Flush Ace King Jack Ten Four       -> 7100
          Flush Ace King Jack Ten Five       -> 7101
          Flush Ace King Jack Ten Six        -> 7102
          Flush Ace King Jack Ten Seven      -> 7103
          Flush Ace King Jack Ten Eight      -> 7104
          Flush Ace King Jack Ten Nine       -> 7105
          Flush Ace King Queen Three Two     -> 7106
          Flush Ace King Queen Four Two      -> 7107
          Flush Ace King Queen Four Three    -> 7108
          Flush Ace King Queen Five Two      -> 7109
          Flush Ace King Queen Five Three    -> 7110
          Flush Ace King Queen Five Four     -> 7111
          Flush Ace King Queen Six Two       -> 7112
          Flush Ace King Queen Six Three     -> 7113
          Flush Ace King Queen Six Four      -> 7114
          Flush Ace King Queen Six Five      -> 7115
          Flush Ace King Queen Seven Two     -> 7116
          Flush Ace King Queen Seven Three   -> 7117
          Flush Ace King Queen Seven Four    -> 7118
          Flush Ace King Queen Seven Five    -> 7119
          Flush Ace King Queen Seven Six     -> 7120
          Flush Ace King Queen Eight Two     -> 7121
          Flush Ace King Queen Eight Three   -> 7122
          Flush Ace King Queen Eight Four    -> 7123
          Flush Ace King Queen Eight Five    -> 7124
          Flush Ace King Queen Eight Six     -> 7125
          Flush Ace King Queen Eight Seven   -> 7126
          Flush Ace King Queen Nine Two      -> 7127
          Flush Ace King Queen Nine Three    -> 7128
          Flush Ace King Queen Nine Four     -> 7129
          Flush Ace King Queen Nine Five     -> 7130
          Flush Ace King Queen Nine Six      -> 7131
          Flush Ace King Queen Nine Seven    -> 7132
          Flush Ace King Queen Nine Eight    -> 7133
          Flush Ace King Queen Ten Two       -> 7134
          Flush Ace King Queen Ten Three     -> 7135
          Flush Ace King Queen Ten Four      -> 7136
          Flush Ace King Queen Ten Five      -> 7137
          Flush Ace King Queen Ten Six       -> 7138
          Flush Ace King Queen Ten Seven     -> 7139
          Flush Ace King Queen Ten Eight     -> 7140
          Flush Ace King Queen Ten Nine      -> 7141
          Flush Ace King Queen Jack Two      -> 7142
          Flush Ace King Queen Jack Three    -> 7143
          Flush Ace King Queen Jack Four     -> 7144
          Flush Ace King Queen Jack Five     -> 7145
          Flush Ace King Queen Jack Six      -> 7146
          Flush Ace King Queen Jack Seven    -> 7147
          Flush Ace King Queen Jack Eight    -> 7148
          Flush Ace King Queen Jack Nine     -> 7149
          Flush Ace King Queen Jack Ten      -> 7150
          FullHouse Two Three                -> 7151
          FullHouse Two Four                 -> 7152
          FullHouse Two Five                 -> 7153
          FullHouse Two Six                  -> 7154
          FullHouse Two Seven                -> 7155
          FullHouse Two Eight                -> 7156
          FullHouse Two Nine                 -> 7157
          FullHouse Two Ten                  -> 7158
          FullHouse Two Jack                 -> 7159
          FullHouse Two Queen                -> 7160
          FullHouse Two King                 -> 7161
          FullHouse Two Ace                  -> 7162
          FullHouse Three Two                -> 7163
          FullHouse Three Four               -> 7164
          FullHouse Three Five               -> 7165
          FullHouse Three Six                -> 7166
          FullHouse Three Seven              -> 7167
          FullHouse Three Eight              -> 7168
          FullHouse Three Nine               -> 7169
          FullHouse Three Ten                -> 7170
          FullHouse Three Jack               -> 7171
          FullHouse Three Queen              -> 7172
          FullHouse Three King               -> 7173
          FullHouse Three Ace                -> 7174
          FullHouse Four Two                 -> 7175
          FullHouse Four Three               -> 7176
          FullHouse Four Five                -> 7177
          FullHouse Four Six                 -> 7178
          FullHouse Four Seven               -> 7179
          FullHouse Four Eight               -> 7180
          FullHouse Four Nine                -> 7181
          FullHouse Four Ten                 -> 7182
          FullHouse Four Jack                -> 7183
          FullHouse Four Queen               -> 7184
          FullHouse Four King                -> 7185
          FullHouse Four Ace                 -> 7186
          FullHouse Five Two                 -> 7187
          FullHouse Five Three               -> 7188
          FullHouse Five Four                -> 7189
          FullHouse Five Six                 -> 7190
          FullHouse Five Seven               -> 7191
          FullHouse Five Eight               -> 7192
          FullHouse Five Nine                -> 7193
          FullHouse Five Ten                 -> 7194
          FullHouse Five Jack                -> 7195
          FullHouse Five Queen               -> 7196
          FullHouse Five King                -> 7197
          FullHouse Five Ace                 -> 7198
          FullHouse Six Two                  -> 7199
          FullHouse Six Three                -> 7200
          FullHouse Six Four                 -> 7201
          FullHouse Six Five                 -> 7202
          FullHouse Six Seven                -> 7203
          FullHouse Six Eight                -> 7204
          FullHouse Six Nine                 -> 7205
          FullHouse Six Ten                  -> 7206
          FullHouse Six Jack                 -> 7207
          FullHouse Six Queen                -> 7208
          FullHouse Six King                 -> 7209
          FullHouse Six Ace                  -> 7210
          FullHouse Seven Two                -> 7211
          FullHouse Seven Three              -> 7212
          FullHouse Seven Four               -> 7213
          FullHouse Seven Five               -> 7214
          FullHouse Seven Six                -> 7215
          FullHouse Seven Eight              -> 7216
          FullHouse Seven Nine               -> 7217
          FullHouse Seven Ten                -> 7218
          FullHouse Seven Jack               -> 7219
          FullHouse Seven Queen              -> 7220
          FullHouse Seven King               -> 7221
          FullHouse Seven Ace                -> 7222
          FullHouse Eight Two                -> 7223
          FullHouse Eight Three              -> 7224
          FullHouse Eight Four               -> 7225
          FullHouse Eight Five               -> 7226
          FullHouse Eight Six                -> 7227
          FullHouse Eight Seven              -> 7228
          FullHouse Eight Nine               -> 7229
          FullHouse Eight Ten                -> 7230
          FullHouse Eight Jack               -> 7231
          FullHouse Eight Queen              -> 7232
          FullHouse Eight King               -> 7233
          FullHouse Eight Ace                -> 7234
          FullHouse Nine Two                 -> 7235
          FullHouse Nine Three               -> 7236
          FullHouse Nine Four                -> 7237
          FullHouse Nine Five                -> 7238
          FullHouse Nine Six                 -> 7239
          FullHouse Nine Seven               -> 7240
          FullHouse Nine Eight               -> 7241
          FullHouse Nine Ten                 -> 7242
          FullHouse Nine Jack                -> 7243
          FullHouse Nine Queen               -> 7244
          FullHouse Nine King                -> 7245
          FullHouse Nine Ace                 -> 7246
          FullHouse Ten Two                  -> 7247
          FullHouse Ten Three                -> 7248
          FullHouse Ten Four                 -> 7249
          FullHouse Ten Five                 -> 7250
          FullHouse Ten Six                  -> 7251
          FullHouse Ten Seven                -> 7252
          FullHouse Ten Eight                -> 7253
          FullHouse Ten Nine                 -> 7254
          FullHouse Ten Jack                 -> 7255
          FullHouse Ten Queen                -> 7256
          FullHouse Ten King                 -> 7257
          FullHouse Ten Ace                  -> 7258
          FullHouse Jack Two                 -> 7259
          FullHouse Jack Three               -> 7260
          FullHouse Jack Four                -> 7261
          FullHouse Jack Five                -> 7262
          FullHouse Jack Six                 -> 7263
          FullHouse Jack Seven               -> 7264
          FullHouse Jack Eight               -> 7265
          FullHouse Jack Nine                -> 7266
          FullHouse Jack Ten                 -> 7267
          FullHouse Jack Queen               -> 7268
          FullHouse Jack King                -> 7269
          FullHouse Jack Ace                 -> 7270
          FullHouse Queen Two                -> 7271
          FullHouse Queen Three              -> 7272
          FullHouse Queen Four               -> 7273
          FullHouse Queen Five               -> 7274
          FullHouse Queen Six                -> 7275
          FullHouse Queen Seven              -> 7276
          FullHouse Queen Eight              -> 7277
          FullHouse Queen Nine               -> 7278
          FullHouse Queen Ten                -> 7279
          FullHouse Queen Jack               -> 7280
          FullHouse Queen King               -> 7281
          FullHouse Queen Ace                -> 7282
          FullHouse King Two                 -> 7283
          FullHouse King Three               -> 7284
          FullHouse King Four                -> 7285
          FullHouse King Five                -> 7286
          FullHouse King Six                 -> 7287
          FullHouse King Seven               -> 7288
          FullHouse King Eight               -> 7289
          FullHouse King Nine                -> 7290
          FullHouse King Ten                 -> 7291
          FullHouse King Jack                -> 7292
          FullHouse King Queen               -> 7293
          FullHouse King Ace                 -> 7294
          FullHouse Ace Two                  -> 7295
          FullHouse Ace Three                -> 7296
          FullHouse Ace Four                 -> 7297
          FullHouse Ace Five                 -> 7298
          FullHouse Ace Six                  -> 7299
          FullHouse Ace Seven                -> 7300
          FullHouse Ace Eight                -> 7301
          FullHouse Ace Nine                 -> 7302
          FullHouse Ace Ten                  -> 7303
          FullHouse Ace Jack                 -> 7304
          FullHouse Ace Queen                -> 7305
          FullHouse Ace King                 -> 7306
          FourOfAKind Two Three              -> 7307
          FourOfAKind Two Four               -> 7308
          FourOfAKind Two Five               -> 7309
          FourOfAKind Two Six                -> 7310
          FourOfAKind Two Seven              -> 7311
          FourOfAKind Two Eight              -> 7312
          FourOfAKind Two Nine               -> 7313
          FourOfAKind Two Ten                -> 7314
          FourOfAKind Two Jack               -> 7315
          FourOfAKind Two Queen              -> 7316
          FourOfAKind Two King               -> 7317
          FourOfAKind Two Ace                -> 7318
          FourOfAKind Three Two              -> 7319
          FourOfAKind Three Four             -> 7320
          FourOfAKind Three Five             -> 7321
          FourOfAKind Three Six              -> 7322
          FourOfAKind Three Seven            -> 7323
          FourOfAKind Three Eight            -> 7324
          FourOfAKind Three Nine             -> 7325
          FourOfAKind Three Ten              -> 7326
          FourOfAKind Three Jack             -> 7327
          FourOfAKind Three Queen            -> 7328
          FourOfAKind Three King             -> 7329
          FourOfAKind Three Ace              -> 7330
          FourOfAKind Four Two               -> 7331
          FourOfAKind Four Three             -> 7332
          FourOfAKind Four Five              -> 7333
          FourOfAKind Four Six               -> 7334
          FourOfAKind Four Seven             -> 7335
          FourOfAKind Four Eight             -> 7336
          FourOfAKind Four Nine              -> 7337
          FourOfAKind Four Ten               -> 7338
          FourOfAKind Four Jack              -> 7339
          FourOfAKind Four Queen             -> 7340
          FourOfAKind Four King              -> 7341
          FourOfAKind Four Ace               -> 7342
          FourOfAKind Five Two               -> 7343
          FourOfAKind Five Three             -> 7344
          FourOfAKind Five Four              -> 7345
          FourOfAKind Five Six               -> 7346
          FourOfAKind Five Seven             -> 7347
          FourOfAKind Five Eight             -> 7348
          FourOfAKind Five Nine              -> 7349
          FourOfAKind Five Ten               -> 7350
          FourOfAKind Five Jack              -> 7351
          FourOfAKind Five Queen             -> 7352
          FourOfAKind Five King              -> 7353
          FourOfAKind Five Ace               -> 7354
          FourOfAKind Six Two                -> 7355
          FourOfAKind Six Three              -> 7356
          FourOfAKind Six Four               -> 7357
          FourOfAKind Six Five               -> 7358
          FourOfAKind Six Seven              -> 7359
          FourOfAKind Six Eight              -> 7360
          FourOfAKind Six Nine               -> 7361
          FourOfAKind Six Ten                -> 7362
          FourOfAKind Six Jack               -> 7363
          FourOfAKind Six Queen              -> 7364
          FourOfAKind Six King               -> 7365
          FourOfAKind Six Ace                -> 7366
          FourOfAKind Seven Two              -> 7367
          FourOfAKind Seven Three            -> 7368
          FourOfAKind Seven Four             -> 7369
          FourOfAKind Seven Five             -> 7370
          FourOfAKind Seven Six              -> 7371
          FourOfAKind Seven Eight            -> 7372
          FourOfAKind Seven Nine             -> 7373
          FourOfAKind Seven Ten              -> 7374
          FourOfAKind Seven Jack             -> 7375
          FourOfAKind Seven Queen            -> 7376
          FourOfAKind Seven King             -> 7377
          FourOfAKind Seven Ace              -> 7378
          FourOfAKind Eight Two              -> 7379
          FourOfAKind Eight Three            -> 7380
          FourOfAKind Eight Four             -> 7381
          FourOfAKind Eight Five             -> 7382
          FourOfAKind Eight Six              -> 7383
          FourOfAKind Eight Seven            -> 7384
          FourOfAKind Eight Nine             -> 7385
          FourOfAKind Eight Ten              -> 7386
          FourOfAKind Eight Jack             -> 7387
          FourOfAKind Eight Queen            -> 7388
          FourOfAKind Eight King             -> 7389
          FourOfAKind Eight Ace              -> 7390
          FourOfAKind Nine Two               -> 7391
          FourOfAKind Nine Three             -> 7392
          FourOfAKind Nine Four              -> 7393
          FourOfAKind Nine Five              -> 7394
          FourOfAKind Nine Six               -> 7395
          FourOfAKind Nine Seven             -> 7396
          FourOfAKind Nine Eight             -> 7397
          FourOfAKind Nine Ten               -> 7398
          FourOfAKind Nine Jack              -> 7399
          FourOfAKind Nine Queen             -> 7400
          FourOfAKind Nine King              -> 7401
          FourOfAKind Nine Ace               -> 7402
          FourOfAKind Ten Two                -> 7403
          FourOfAKind Ten Three              -> 7404
          FourOfAKind Ten Four               -> 7405
          FourOfAKind Ten Five               -> 7406
          FourOfAKind Ten Six                -> 7407
          FourOfAKind Ten Seven              -> 7408
          FourOfAKind Ten Eight              -> 7409
          FourOfAKind Ten Nine               -> 7410
          FourOfAKind Ten Jack               -> 7411
          FourOfAKind Ten Queen              -> 7412
          FourOfAKind Ten King               -> 7413
          FourOfAKind Ten Ace                -> 7414
          FourOfAKind Jack Two               -> 7415
          FourOfAKind Jack Three             -> 7416
          FourOfAKind Jack Four              -> 7417
          FourOfAKind Jack Five              -> 7418
          FourOfAKind Jack Six               -> 7419
          FourOfAKind Jack Seven             -> 7420
          FourOfAKind Jack Eight             -> 7421
          FourOfAKind Jack Nine              -> 7422
          FourOfAKind Jack Ten               -> 7423
          FourOfAKind Jack Queen             -> 7424
          FourOfAKind Jack King              -> 7425
          FourOfAKind Jack Ace               -> 7426
          FourOfAKind Queen Two              -> 7427
          FourOfAKind Queen Three            -> 7428
          FourOfAKind Queen Four             -> 7429
          FourOfAKind Queen Five             -> 7430
          FourOfAKind Queen Six              -> 7431
          FourOfAKind Queen Seven            -> 7432
          FourOfAKind Queen Eight            -> 7433
          FourOfAKind Queen Nine             -> 7434
          FourOfAKind Queen Ten              -> 7435
          FourOfAKind Queen Jack             -> 7436
          FourOfAKind Queen King             -> 7437
          FourOfAKind Queen Ace              -> 7438
          FourOfAKind King Two               -> 7439
          FourOfAKind King Three             -> 7440
          FourOfAKind King Four              -> 7441
          FourOfAKind King Five              -> 7442
          FourOfAKind King Six               -> 7443
          FourOfAKind King Seven             -> 7444
          FourOfAKind King Eight             -> 7445
          FourOfAKind King Nine              -> 7446
          FourOfAKind King Ten               -> 7447
          FourOfAKind King Jack              -> 7448
          FourOfAKind King Queen             -> 7449
          FourOfAKind King Ace               -> 7450
          FourOfAKind Ace Two                -> 7451
          FourOfAKind Ace Three              -> 7452
          FourOfAKind Ace Four               -> 7453
          FourOfAKind Ace Five               -> 7454
          FourOfAKind Ace Six                -> 7455
          FourOfAKind Ace Seven              -> 7456
          FourOfAKind Ace Eight              -> 7457
          FourOfAKind Ace Nine               -> 7458
          FourOfAKind Ace Ten                -> 7459
          FourOfAKind Ace Jack               -> 7460
          FourOfAKind Ace Queen              -> 7461
          FourOfAKind Ace King               -> 7462
          StraightFlush Five                 -> 7463
          StraightFlush Six                  -> 7464
          StraightFlush Seven                -> 7465
          StraightFlush Eight                -> 7466
          StraightFlush Nine                 -> 7467
          StraightFlush Ten                  -> 7468
          StraightFlush Jack                 -> 7469
          StraightFlush Queen                -> 7470
          StraightFlush King                 -> 7471
          StraightFlush Ace                  -> 7472
          _                                  -> error "Data.Poker.Brute.toConsecutive: bad argument"

fromConsecutiveId :: ConsecutiveHandValue -> HandValue
fromConsecutiveId (ConsecutiveHandValue n) = allHandValuesVector V.! fromIntegral n

allHandValuesVector :: V.Vector HandValue
allHandValuesVector = V.fromList allHandValues

allHandValues :: [HandValue]
allHandValues =
  [ NoPair a b c d e
  | a <- ranks, b <- ranksLT a, c <- ranksLT b, d <- ranksLT c, e <- ranksLT d
  , not (a == succ b && b == succ c && c == succ d && d == s e) ] ++
  [ OnePair a b c d
  | a <- ranks
  , b <- ranks, c <- ranksLT b, d <- ranksLT c
  , a /= b, a /= c, a /= d ] ++
  [ TwoPair a b c
  | a <- ranks, b <- ranksLT a
  , c <- ranks, a /= c, b /= c ] ++
  [ ThreeOfAKind a b c
  | a <- ranks
  , b <- ranks, c <- ranksLT b
  , a /= b, a /= c ] ++
  [ Straight f| f <- ranksGE Five ] ++
  [ Flush a b c d e
  | a <- ranks, b <- ranksLT a, c <- ranksLT b, d <- ranksLT c, e <- ranksLT d ] ++
  [ FullHouse a b | a <- ranks, b <- ranks, a /= b ] ++
  [ FourOfAKind a b | a <- ranks, b <- ranks, a /= b ] ++
  [ StraightFlush f| f <- ranksGE Five ]
  where
    s Ace = Two
    s other = succ other
    ranks        = [Two .. Ace]
    ranksLT Two  = []
    ranksLT rank = [Two .. pred rank ]
    ranksGE rank = reverse [Ace, King .. rank ]

